#include "copyright.h"

#include <stdio.h>
#include <values.h>
#include <math.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

/* was 8 */
#define MIN_DIR			4		/* ignore change < */
/* was 50 */
#define DIR_DELAY		75		/* ignore if change delay > */

#define GOOD_TORP		16		/* not good torp if > */
#define REAL_GOOD_TORP		4		/* not real good torp if > */

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

static int get_torp_course P_((struct player *j, unsigned char *crs, int cx, int cy));
static unsigned char get_acourse P_((int x, int y, int mx, int my));

#undef P_
#endif

static int		_do_bd_torp = 1;

/* change direction quickness */
static unsigned int	_last_was_phaser;
static unsigned int	_last_time;
static unsigned char	_last_dir;
static int		_borg_counts, 
			_phaser_hits,
			_borg_val;

/* torp accuracy */
static int		_good_torps = 0,
			_real_good_torps = 0;

/*
 * gives approximate distance from (x1,y1) to (x2,y2) with only
 * overestimations, and then never by more than (9/8) + one bit uncertainty.
 */

static int 
ihypot(x1, y1)
   int             x1, y1;
{
   register        x2 = 0, y2 = 0;

   if ((x2 -= x1) < 0)
      x2 = -x2;
   if ((y2 -= y1) < 0)
      y2 = -y2;
   return (x2 + y2 - (((x2 > y2) ? y2 : x2) >> 1));
}


void
bd_phaser(dir)

   unsigned char	dir;
{
   unsigned int		now = mstime();
   int			angdiff;

   _phaser_hits ++;

   if(!_last_time){
      _last_dir = dir;
      _last_time = now;
   }

   if(_last_was_phaser){
/*
      _last_dir = dir;
      _last_time = now;
*/
      _last_was_phaser = 1;
      return;
   }
   _last_was_phaser = 1;

/*
printf("p: delay %d, angdist %d\n", now-_last_time, angdist(dir, _last_dir));
fflush(stdout);
*/

   delay = now - _last_time;
   if(delay > DIR_DELAY){
      _last_dir = dir;
      _last_time = now;
      return;
   }
   angdiff = angdist(dir, _last_dir);
   if(angdiff < MIN_DIR){
      _last_dir = dir;
      _last_time = now;
      return;
   }

   bd_update(delay, angdiff);

   _last_dir = dir;
   _last_time = now;

   return;
}

void
bd_dir(desc, dir)

   char			*desc;
   unsigned char	dir;
{
   unsigned int		now = mstime();
   int			angdiff, delay;

   if(!_last_time || !_last_was_phaser){
/*
      _last_dir = dir;
      _last_time = now;
*/
      _last_was_phaser = 0;
      return;
   }
   _last_was_phaser = 0;

/*
printf("%s: delay %d, angdist %d\n", desc, now - _last_time, angdist(dir, _last_dir));
fflush(stdout);
*/
   delay = now - _last_time;
   if(delay > DIR_DELAY) {
      _last_dir = dir;
      _last_time = now;
      return;		/* threshold, 50ms */
   }

   angdiff = angdist(dir, _last_dir);

   if(angdiff < MIN_DIR) {		/* threshold, 11 (8) degrees */
      _last_dir = dir;
      _last_time = now;
      return;
   }

   bd_update(delay, angdiff);

   _last_dir = dir;
   _last_time = now;
   return;
}

void
bd_update(delay, angdiff)

   int	delay, angdiff;
{
   /* the greater angdiff and smaller delay, the likelier the player is
      a borg */
   
/*
   printf("delay %d, angdiff %d\n", delay, angdiff);
   fflush(stdout);
*/

   _borg_val += (DIR_DELAY-delay) * angdiff;
   _borg_counts ++;
}

int
bd_output(fo, start)
   
   FILE	*fo;
   int	start;

{
   int	count, val, min, hits, gt, rgt;
   min = (me->p_stats.st_tticks - start) / 600;
   val = bd_borg(&count, &hits, &gt, &rgt);
   if(!gt) gt = -1;
   if(!hits) hits = 1;
/* TMP */
   if(1/* min > 15 && (double) count / (double) hits > 0.33 */){
      fprintf(fo, 
   "bd %8s@%-40s %5.2g%% (val: %4d, count: %4d, hits: %6d, min: %4d, tr: %3d (%5d/%5d)) \"%s\"\n",
	 me->p_login, me->p_monitor,
	 100.*(double)count / (double)hits, val, count, hits, min, 
	 (100 * rgt)/gt, rgt, gt, RSA_client_type);
   }

   return 0;
}

int
bd_borg(counts, hits, gt, rgt)
   
   int		*counts;
   int		*hits;
   int		*gt, *rgt;
{
   *counts = _borg_counts;
   *hits = _phaser_hits;

   *gt = _good_torps;
   *rgt = _real_good_torps;

   if(!_borg_counts) return 0;

   /* the maximum possible value here is 51200 */
   return _borg_val / _borg_counts;
}

void
bd_torp(dir)

   int	dir;
{
   register			i;
   register struct player	*j, *e = NULL;
   register int			d, mind = MAXINT;
   unsigned char		best_dir;
   int				diff;
   
   if(!_do_bd_torp) return;

   /* find closest enemy player */

   for(i=0,j=players; i< MAXPLAYER; i++,j++){
      if(j->p_status != PALIVE || j == me || friendlyPlayer(j) || 
	 (j->p_flags & PFCLOAK)) continue;
      d = ihypot(j->p_x - me->p_x, j->p_y - me->p_y);
      if(d < mind){
	 e = j;
	 mind = d;
      }
   }
   if(!e) return;
   /* this is best torp angle */
   if(!get_torp_course(e, &best_dir, me->p_x, me->p_y)) return;
   /* this is how many degrees (0-256) it was off */
   diff = angdist(dir, best_dir);
   if(diff < GOOD_TORP) {
      /* this counts */
      _good_torps ++;
      if(diff < REAL_GOOD_TORP)
	 _real_good_torps ++;
   }
}


/* get course from cx,cy to j */
static int
get_torp_course(j, crs, cx, cy)
    struct player  *j;
    unsigned char  *crs;
    int             cx, cy;
{
   double          vxa, vya, vxs, vys, vxt, vyt;
   double          dp, l, dx, dy;
   register int    x, y, mx, my, speed;
   unsigned char   dir;

   speed = j->p_speed;
   dir = j->p_dir;
   x = j->p_x;
   y = j->p_y;
   mx = cx;
   my = cy;

   vxa = (x - mx);
   vya = (y - my);

   l = (double) hypot(vxa, vya);
   if (l != 0) {
      vxa /= l;
      vya /= l;
   }
   vxs = (Cos[dir] * speed) * WARP1;
   vys = (Sin[dir] * speed) * WARP1;
   dp = vxs * vxa + vys * vya;  /* dot product of (va vs) */
   dx = vxs - dp * vxa;
   dy = vys - dp * vya;
   l = (double) hypot(dx, dy);
   dp = (double) (j->p_ship.s_torpspeed * WARP1);
   l = (dp * dp - l * l);
   if (l > 0) {
      l = sqrt(l);
      vxt = l * vxa + dx;
      vyt = l * vya + dy;
      *crs = get_acourse((int) vxt + mx, (int) vyt + my, cx, cy);
      return 1;
   }
   /* out of range? */
   return 0;
}

/* get course from (mx,my) to (x,y) */
static unsigned char 
get_acourse(x, y, mx, my)
    int             x, y, mx, my;
{
   if (x == mx && y == my)
      return 0;

   return (unsigned char) (atan2((double) (x - mx),
                                       (double) (my - y)) / 3.14159 * 128.);
}
