/*
 * daemonII.c
 */
#include "copyright.h"

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <setjmp.h>
#include <math.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "planets.h"
#include <sys/time.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/resource.h>
#include <errno.h>
#ifndef _NO_DEPEND
#   include "daemon_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   srandom P_((int));
extern pid_t                  getpid P_((void));
extern int                    shmget P_((key_t, int, int));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void                   perror P_((const char *));
extern int                    shmctl P_((int, int, ...));
extern uid_t                  geteuid P_((void));
extern void *                 shmat P_((int, void *, int));
extern int                    fflush P_((FILE *));
extern void                   bcopy P_((const void *, void *, int));
extern int                    read P_((int, void *, unsigned int));
extern void                   bzero P_((void *, size_t));
extern time_t                 time P_((time_t *));
extern pid_t                  fork P_((void));
extern int                    execl P_((const char *, const char *, ...));
extern int                    setitimer P_((int, struct itimerval *, struct itimerval *));
extern int                    pause P_((void));
extern int                    printf P_((const char *, ...));
extern int                    ioctl P_((int, int, ...));
extern int                    close P_((int));
extern int                    write P_((int, const void *, unsigned int));
extern int                    pclose P_((FILE *));
extern int                    sscanf P_((const char *, const char *, ...));
extern pid_t                  wait3 P_((union wait *, int, struct rusage *));
extern char *                 strcat P_((char *, const char *));
extern char *                 strstr P_((const char *, const char *));
extern int                    fclose P_((FILE *));
extern char *                 strncpy P_((char *, const char *, size_t));
extern int 		      gettimeofday P_((struct timeval *tp, struct timezone *tzp));

#undef P_
#endif

#define fuse(X) ((ticks % (X)) == 0)
#define TOURNEXTENSION 15	/* Tmode gone for 15 seconds 8/26/91 TC */
#define NotTmode(X) (!(status->tourn) && ((X - tourntimestamp)/10 > TOURNEXTENSION)) /* Run the game */
#define isSB(j)			(j->p_ship.s_type == STARBASE)

long            random();
long            lseek();
void            doResources();

static struct itimerval udt;
static int      shmid;

static int      debug = 0;
static int      ticks = 0;
static int      tickinc = UPDATE_NORM;
static int      tourntimestamp = 0;	/* ticks since last Tmode 8/2/91 TC */
static int      plfd = -1;	/* since 0 is valid descriptor for stdin */
static int      glfd;
int             didInitial = 0;
static struct player	*_winner;

jmp_buf         env;

char           *teamNames[9] = {"", "Federation", "Romulans", "", "Klingons",
"", "", "", "Orions"};
char           *teamVerbage[9] = {"", "has", "have", "", "have", "", "", "", "have"};

static int      tcount[ALLTEAM + 1];	/* all active players per team */
static int	pcount[ALLTEAM + 1];	/* non-robot players per team */
unsigned char   getbearing();

/*
 * These specify how often special actions will take place in UPDATE units
 * (0.10 seconds, currently.)
 */

#define PLAYERFUSE	1
#define TORPFUSE	1
#define PLASMAFUSE      1
#define PHASERFUSE	1
#define CLOAKFUSE       2/UPDATE_NORM
#define TEAMFUSE	5/UPDATE_NORM
#define PLFIGHTFUSE	5/UPDATE_NORM
#define BEAMFUSE	8/UPDATE_NORM	/* scott 8/25/90 -- was 10 */
#ifdef INL_POP
#   define PLANETFUSE	(400/MAXPLANETS)/UPDATE_NORM
#else
#   define PLANETFUSE	400/UPDATE_NORM	/* scott 8/25/90 -- was 600 */
#endif
#define SYNCFUSE	3000/UPDATE_NORM
#define MINUTEFUSE	600/UPDATE_NORM	/* 1 minute, surrender funct etc. */
#define SYSDEFFUSE	300/UPDATE_NORM	/* 30 seconds check sys defaults */
#define CHECKLOADFUSE   9000/UPDATE_NORM

#ifdef nodef			/* now in defs.h */
#define GHOSTTIME	(30 * 1000000 / UPDATE)	/* 30 secs */
#endif

#define OUTFITTIME	(6 * AUTOQUIT * 1000000 / UPDATE)	/* 6 * AQ secs */

#ifdef BASEPRACTICE
#define STATUSFUSE	100/UPDATE_NORM
#endif

#ifdef VOTE
#define RESETFUSE	10/UPDATE_NORM
#define VOTEFUSE	300/UPDATE_NORM	/* 30 seconds */
#endif

int             nplayers = 0;
int             dietime = -1;

#ifdef SHORT_PACKETS
int		m_args[8];
#endif

void
main(argc, argv)
   int             argc;
   char          **argv;
{
   register int    i;
   int             shmemKey = PKEY;
   struct memory  *sharedMemory;
   struct shmid_ds smbuf;
   int             x;
   struct stat     mstat;

   if (argc > 1)
      debug = 1;

   srandom(getpid());
   readsysdefaults();
#ifdef nodef
   if (timecheck) {		/* log */
      time_t            t = time(NULL);
      timecheck = 0;
      /* log the change */
      fprintf(stderr, "timecheck: Initial sysdef file from timecheck on %s",
	      ctime(&t));
      fprintf(stderr, "timecheck: File: \"%s\", description: \"%s\"\n",
	      sysdef_file, sysdef_comment);
      fprintf(stderr, "timecheck: This configuration ends on %s",
	      ctime(&timechecknew));
      fflush(stderr);
   }
#endif

   if (!debug) {
      for (i = 0; i < NSIG; i++) {
	 switch(i){
	    case EAGAIN:
	       (void) signal(i, SIG_IGN);
	       break;

	    default:
	       (void) signal(i, freemem);
	 }
      }
      detach();
   }

   shmid = shmget(shmemKey, sizeof(struct memory), IPC_CREAT | IPC_EXCL | 0700);
   if (shmid < 0) {
      switch (errno) {
      case EEXIST:
	 fprintf(stderr, "daemonII: the shared memory segment already exists.\n");
	 fprintf(stderr, "daemonII: Another daemonII may be running.  If not use 'nuke' to remove it.\n");
	 exit(1);
      default:
	 perror("can't open shared memory");
	 exit(1);
      }
   }
   /* Hose Ed's robots */
   shmctl(shmid, IPC_STAT, &smbuf);
   smbuf.shm_perm.uid = geteuid();
   smbuf.shm_perm.mode = 0744;	/* read-only */
   shmctl(shmid, IPC_SET, &smbuf);

   sharedMemory = (struct memory *) shmat(shmid, 0, 0);
   if (sharedMemory == (struct memory *) - 1) {
      perror("shm attach");
      exit(1);
   }
   players = sharedMemory->players;
   torps = sharedMemory->torps;
   plasmatorps = sharedMemory->plasmatorps;
   status = sharedMemory->status;
   planets = sharedMemory->planets;
   phasers = sharedMemory->phasers;
   mctl = sharedMemory->mctl;
   messages = sharedMemory->messages;
   teams = sharedMemory->teams;
#ifdef VOTE
   vote = &sharedMemory->vote;
   bzero(vote, sizeof(struct vote));
#endif

   for (i = 0; i < MAXPLAYER; i++) {
      players[i].p_status = PFREE;
      players[i].p_stats.st_tticks = 1;
      players[i].p_no = i;
   }

   plfd = open(PLFILE, O_RDWR, 0744);
   if (plfd < 0) {
      fprintf(stderr, "No planet file.  Restarting galaxy\n");
      fflush(stderr);
      doResources();
   } else {
      if (read(plfd, (char *) planets, sizeof(pdata)) != sizeof(pdata)) {
	 fprintf(stderr, "Planet file wrong size.  Restarting galaxy\n");
	 fflush(stderr);
	 doResources();
      }
   }

   glfd = open(GLOBAL, O_RDWR, 0744);
   if (glfd < 0) {
      fprintf(stderr, "No global file.  Resetting all stats\n");
      fflush(stderr);
   } else {
      if (read(glfd, (char *) status, sizeof(struct status)) !=
	  sizeof(struct status)) {
	 fprintf(stderr, "Global file wrong size.  Resetting all stats\n");
	 fflush(stderr);
	 bzero((char *) status, sizeof(struct status));
      }
   }
   if (status->time == 0) {
      /* Start all stats at 1 to prevent overflow */
      /* 10, now because of division by 10 in socket.c */
      status->time = 10;
      status->timeprod = 10;
      status->planets = 10;
      status->armsbomb = 10;
      status->kills = 10;
      status->losses = 10;
#ifdef BASEPRACTICE
      status->dami = 1.;
      status->damr = 1.;
#endif
   }
   status->active = 0;
   status->wait = 0;
   status->count = 0;
   status->request = 0;
   if (gamedown)
      status->gameup = 0;
   else
      status->gameup = 1;

   (void) signal(SIGCHLD, reaper);

#ifdef BASEPRACTICE
   startbasep();
#endif

   if (stat(MOTD, &mstat) < 0 || (time(NULL) - mstat.st_mtime) > 60 * 60 * 12) {
      if (fork() == 0) {
	 execl("/bin/sh", "sh", "-c", MAKEMOTD, 0);
	 perror(MAKEMOTD);
	 _exit(1);
      }
   }

   (void) signal(SIGALRM, move);
   udt.it_interval.tv_sec = 0;
   udt.it_interval.tv_usec = UPDATE;
   udt.it_value.tv_sec = 0;
   udt.it_value.tv_usec = UPDATE;
   (void) setitimer(ITIMER_REAL, &udt, (struct itimerval *) 0);

   (void) setjmp(env);

   x = 0;

   /* reset the SB construction and surrender countdown immediately */
   /* after a conquer 4/17/92 TC (was prior to setjmp()) */

   for (i = 0; i <= MAXTEAM; i++) {
      teams[i].s_turns = 0;
      teams[i].s_surrender = 0;
   }

   check_load();

   while (1) {
      pause();
      if (debug) {
	 if (!(++x % 50)) {
	    printf("Mark %d\n", x);
	    fflush(stdout);
	 }
      }
   }
}

void
detach()
{
   int             fd;

   fd = open("/dev/tty", O_RDWR, 0);
   if (fd < 0)
      return;
#ifndef SYSV
   (void) ioctl(fd, TIOCNOTTY, (char *) NULL);
#endif
   (void) close(fd);
}

int
tournamentMode()
/*
 * Return 1 if tournament mode, 0 otherwise. This function only used to
 * determine if we should be recording stats.
 */
{
   register int             i;

#ifdef BASEPRACTICE
   return 0;	/* t-mode not used */
#endif
   i = 0;
   if (pcount[FED] >= tournplayers)
      i++;
   if (pcount[ROM] >= tournplayers){
      if(++i == 2) return 1;
   }
   if (pcount[KLI] >= tournplayers){
      if(++i == 2) return 1;
   }
   if (pcount[ORI] >= tournplayers){
      if(++i == 2) return 1;
   }

   return 0;
}

void
move()
{
   static int      oldtourn = 0;
   static int      oldmessage;

   ticks += tickinc;
   if (ticks == dietime) {	/* no player for 1 minute. kill self */
#ifdef SELF_RESET		/* isae -- reset the galaxy when the daemon
				 * dies */
      /*
      fprintf(stderr, "Resetting the galaxy!\n");
      */
      doResources();
#endif
      printf("Daemon exiting from inactivity.\n");
      fflush(stdout);
      freemem(0, 0, NULL, NULL);
   }
   if (fuse(SYSDEFFUSE) && update_sys_defaults( /* daemon calling.. */ 1)) {
      /*
       * This messge does 2 things: First, it tells the players that the
       * system defaults have in fact changed. Secondly, it triggers the
       * ntserv processes to check the new defaults.
       */
      pmessage("Loading new server configuration.", 0, MALL, "GOD->ALL");
      if (timecheck) {
	 time_t            t = time(NULL);
	 timecheck = 0;
	 pmessage(sysdef_comment, 0, MALL, "GOD->ALL");

	 /* log the change */
	 fprintf(stderr, "timecheck: New sysdef file from timecheck on %s",
		 ctime(&t));
	 fprintf(stderr, "timecheck: File: \"%s\", description: \"%s\"\n",
		 sysdef_file, sysdef_comment);
	 fprintf(stderr, "timecheck: This configuration ends on %s",
		 ctime(&timechecknew));
	 fflush(stderr);
      }

      if (gamedown) {
	 register        i;
	 register struct player *j;
	 status->gameup = 0;
	 /* boot everybody out */
	 for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	    if (j->p_status == PALIVE) {
	       j->p_explode = 20 / UPDATE_NORM;
	       j->p_status = PEXPLODE;
	       j->p_whydead = KQUIT;
	    }
	 }
      }
   }
   if (tournamentMode()) {
      if (!oldtourn) {
	 switch (oldmessage = (random() % 9)) {
	 case 0:
	    pmessage("A dark mood settles upon the galaxy", 0, MALL, "GOD->ALL");
	    break;
	 case 1:
	    pmessage("Political pressure to expand is rising", 0, MALL, "GOD->ALL");
	    break;
	 case 2:
	    pmessage("Border disputes break out as political tensions increase!", 0, MALL, "GOD->ALL");
	    break;
	 case 3:
	    pmessage("Galactic superpowers seek to colonize new territory!", 0, MALL, "GOD->ALL");
	    break;
	 case 4:
	    pmessage("'Manifest Destiny' becomes motive in galactic superpower conflict!", 0, MALL, "GOD->ALL");
	    break;
	 case 5:
	    pmessage("Diplomat insults foriegn emperor's mother and fighting breaks out!", 0, MALL, "GOD->ALL");
	    break;
	 case 6:
	    pmessage("Dan Quayle declares self as galactic emperor and chaos breaks out!", 0, MALL, "GOD->ALL");
	    break;
	 case 7:
	    pmessage("Head of state vomits in foriegn diplomat's lap and war is declared!", 0, MALL, "GOD->ALL");
	    break;
	    break;
	 default:
	    pmessage("Peace parties have been demobilized, and fighting ensues.", 0, MALL, "GOD->ALL");
	    break;
	 }
#ifdef BORG
	 do_tourn_start();
#endif
      }
      oldtourn = 1;
      status->tourn = 1;
      status->time += tickinc;
      tourntimestamp = ticks;
   } else {
      if (oldtourn) {
	 tourntimestamp = ticks;
	 switch (oldmessage) {
	 case 0:
	    pmessage("A new day dawns as the oppressive mood lifts", 0, MALL, "GOD->ALL");
	    break;
	 case 1:
	    pmessage("The expansionist pressure subsides", 0, MALL, "GOD->ALL");
	    break;
	 case 2:
	    pmessage("Temporary agreement is reached in local border disputes.", 0, MALL, "GOD->ALL");
	    break;
	 case 3:
	    pmessage("Galactic governments reduce colonization efforts.", 0, MALL, "GOD->ALL");
	    break;
	 case 4:
	    pmessage("'Manifest Destiny is no longer a fad.' says influential philosopher.", 0, MALL, "GOD->ALL");
	    break;
	 case 5:
	    pmessage("Diplomat apologizes to foreign emperor's mother and invasion is stopped!", 0, MALL, "GOD->ALL");
	    break;
	 case 6:
	    pmessage("Dan Quayle is locked up and order returns to the galaxy!", 0, MALL, "GOD->ALL");
	    break;
	 case 7:
	    pmessage("Cleaning crews contain the emergency and all is forgiven!", 0, MALL, "GOD->ALL");
	    break;
	 default:
	    pmessage("The peace party has reformed, and is rallying for peace", 0, MALL, "GOD->ALL");
	    break;
	 }
      }
      oldtourn = 0;
      status->tourn = 0;
   }

   if (fuse(PLAYERFUSE)) {
      udplayers();
   }
   if (fuse(TORPFUSE)) {
      udtorps();
   }
   if (fuse(PLASMAFUSE)) {
      udplasmatorps();
   }
   if (fuse(PHASERFUSE)) {
      udphaser();
   }
   if (fuse(CLOAKFUSE)) {
      udcloak();
   }

#ifdef DS
   /* not sure where's the best place to put this (or even if it matters).  
      If here, we tell the ntserv processes to update now while the daemon 
      putters through the remaining non-essential updates */
   signal_servers();
#endif

   if (fuse(TEAMFUSE)) {
      teamtimers();
   }
   if (fuse(PLFIGHTFUSE)) {
      plfight();
   }
   if (fuse(BEAMFUSE)) {
      beam();
   }
   if (fuse(SYNCFUSE)) {
      save_planets();
   }
   if (fuse(PLANETFUSE)) {
      udplanets();
   }
   if (fuse(CHECKLOADFUSE)) {
      check_load();
   }
   if (fuse(MINUTEFUSE)) {
      udsurrend();
   }
#ifdef ORGANIAN
   if (organian > 1 && fuse(50)) {
      /*
       * they're in the game possibly resetting planets.  once done we need
       * to kick everybody out
       */
      check_organians();
   }
#endif

#ifdef SNAKEPATROL
   if (snakepatrol) {
      if (fuse(snakepatrol/UPDATE_NORM)) {
	 /* may or may not be started from here */
	 checksnake();
      }
   }
#endif

#ifdef nodef
   if (fuse(STATUSFUSE)) {
      udstatus();
   }
#endif

#ifdef VOTE
   if(vote->v_resetgalaxy){
      if(fuse(RESETFUSE))
	 udvotegalaxyreset();
   }
   if(fuse(VOTEFUSE)){
      udvote();
   }
#endif
}

void
udplayers()
{
   register int    i, k;
   register struct player *j;
   float           dist;	/* used by tractor beams */
   int             maxspeed;
   unsigned int    alert_status;

   nplayers = 0;
   tcount[NOBODY] = tcount[FED] = tcount[ROM] = tcount[KLI] = tcount[ORI] = 0;
   tcount[NOBODY] = pcount[FED] = pcount[ROM] = pcount[KLI] = pcount[ORI] = 0;
   for (i = status->active = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      switch (j->p_status) {
      case POUTFIT:
	 if (!(j->p_flags & PFROBOT)) 
	    pcount[j->p_team] ++;
	 if (++(j->p_ghostbuster) > OUTFITTIME) {
	    saveplayer(j);
	    j->p_status = PFREE;
	    j->p_ghostbuster = 0;	/* stop from hosing new players */
#ifdef DS
	    if(j->p_process > 1)/* to risky to let the process hang around */
	       (void)kill(j->p_process, SIGTERM);
#endif
	 }
	 continue;
      case PFREE:
	 nplayers++;
	 j->p_ghostbuster = 0;	/* stop from hosing new players */
	 continue;
      case PDEAD:
	 if (!(j->p_flags & PFROBOT)) 
	    pcount[j->p_team] ++;
	 if (--j->p_explode <= 0) {	/* Ghost Buster */
	    saveplayer(j);
	    j->p_status = POUTFIT;	/* was PFREE -- TC */
	 }
	 continue;
      case PEXPLODE:
	 j->p_updates++;
	 j->p_flags &= ~PFCLOAK;
	 if ((j->p_ship.s_type == STARBASE) &&
	     (j->p_explode == (((2 * SBEXPVIEWS - 3)/UPDATE_NORM) / PLAYERFUSE)))
	    blowup(j);		/* damdiffe everyone else around */
	 else if (j->p_explode == (((10 - 3)/UPDATE_NORM) / PLAYERFUSE))
	    blowup(j);		/* damdiffe everyone else around */

	 if (--j->p_explode <= 0) {
	    j->p_status = PDEAD;
	    j->p_explode = (600/UPDATE_NORM) / PLAYERFUSE;	/* set ghost buster */
	 }
	 /* At this point, docked ships become removed. */
	 if (j->p_flags & PFDOCK) {
	    players[j->p_docked].p_docked--;
#ifdef SB_DOCK
	    players[j->p_docked].p_port[j->p_portdocked] = VACANT;
#else
	    players[j->p_docked].p_port[j->p_port[0]] = VACANT;
#endif
	    j->p_flags &= ~PFDOCK;
	 }
	 /* NEW -- no rebuild time for starbases killed out of t-mode */
	 if (j->p_ship.s_type == STARBASE && status->tourn) {
	    if ((j->p_whydead == KSHIP) ||
		(j->p_whydead == KTORP) ||
		(j->p_whydead == KPHASER) ||
		(j->p_whydead == KPLASMA) ||
		(j->p_whydead == KPLANET) ||
		(j->p_whydead == KGENOCIDE))
	       teams[j->p_team].s_turns = BUILD_SB_TIME;
	    /* 30 minute reconstruction period for new starbase */
	    for (k = 0; k < NUMPORTS; k++)
	       if (j->p_port[k] != VACANT) {
		  players[j->p_port[k]].p_flags &= ~PFDOCK;
		  j->p_port[k] = VACANT;
	       }
	    j->p_docked = 0;
	 }
	 /* And he is ejected from orbit. */
	 if (j->p_flags & PFORBIT)
	    j->p_flags &= ~PFORBIT;

	 /* Fall through to alive so explosions move */

      case PALIVE:
	 /* Move Player in orbit */
	 if ((j->p_flags & PFORBIT) && !(j->p_flags & PFDOCK)) {
	    j->p_dir += 2 * UPDATE_NORM;
	    j->p_desdir = j->p_dir;
	    j->p_x = planets[j->p_planet].pl_x + (int)(ORBDIST
	       * Cos[(unsigned char) (j->p_dir - (unsigned char) 64)]);
	    j->p_y = planets[j->p_planet].pl_y + (int)(ORBDIST
	       * Sin[(unsigned char) (j->p_dir - (unsigned char) 64)]);
	 }
	 /* Move player through space */
	 else if (!(j->p_flags & PFDOCK)) {

	    if ((j->p_dir != j->p_desdir) && (j->p_status != PEXPLODE))
	       changedir(j);

#ifdef SB_TRANSWARP
	    if (j->p_flags & PFTWARP) {
	       if ((j->p_speed *= 2) > (j->p_desspeed))
		  j->p_speed = j->p_desspeed;
	       j->p_fuel -= (int)((j->p_ship.s_warpcost * j->p_speed)*.8);
	       j->p_etemp += j->p_ship.s_maxspeed;
	    } else {		/* not transwarping */
#endif
	       /* Alter speed */
	       if(j->p_ship.s_maxdamage)	/* happens it seems */
		  maxspeed = (j->p_ship.s_maxspeed + 2) -
		   (j->p_ship.s_maxspeed + 1) *
		  ((float) j->p_damage / (float) (j->p_ship.s_maxdamage));
	       else
		  maxspeed = j->p_ship.s_maxspeed;

	       if (j->p_desspeed > maxspeed)
		  j->p_desspeed = maxspeed;
	       if (j->p_flags & PFENG)
		  j->p_desspeed = 1;

	       /* Charge for speed */
	       if (j->p_fuel < (j->p_ship.s_warpcost * j->p_speed)) {
		  /*
		   * Out of fuel ... set speed to one that uses less fuel
		   * than the ship's reactor generates (if necessary).
		   */
		  if (j->p_ship.s_recharge / j->p_ship.s_warpcost <
		      j->p_speed) {
		     j->p_desspeed = j->p_ship.s_recharge / j->p_ship.s_warpcost + 2;
		     /* Above incantation is a magic formula ... (hack). */
		     if (j->p_desspeed > j->p_ship.s_maxspeed)
			j->p_desspeed = j->p_ship.s_maxspeed - 1;
		  } else {
		     j->p_desspeed = j->p_speed;
		  }
		  j->p_fuel = 0;
	       } else {
		  j->p_fuel -= (j->p_ship.s_warpcost * j->p_speed);
		  j->p_etemp += j->p_speed;

		  /* Charge SB's for mass of docked vessels ... */
		  if (j->p_ship.s_type == STARBASE) {
		     for (k = 0; k < NUMPORTS; k++)
			if (j->p_port[k] != VACANT) {
			   j->p_fuel -= players[j->p_port[k]].p_ship.s_warpcost * j->p_speed;
			   j->p_etemp += (int)(.7 * (j->p_speed * j->p_docked));
			}
		  }
	       }
#ifdef SB_TRANSWARP
	    }
#endif

	    if (j->p_desspeed > j->p_speed) {
	       j->p_subspeed += j->p_ship.s_accint;
	    }
	    if (j->p_desspeed < j->p_speed)
	       j->p_subspeed -= j->p_ship.s_decint;

	    if (j->p_subspeed / 1000) {
	       j->p_speed += j->p_subspeed / 1000;
	       j->p_subspeed %= 1000;
/* suggested by Leonard:  j->p_subspeed %= 1000  as /= 1000 */
	       if (j->p_speed < 0)
		  j->p_speed = 0;
	       if (j->p_speed > j->p_ship.s_maxspeed)
		  j->p_speed = j->p_ship.s_maxspeed;
	    }
	    /*
	     * the following grouping of variables converts one floating
	     * point multiplication to an integer multiplication without loss
	     * of precision
	     */
	    j->p_x += (int)((j->p_speed * WARP1) * Cos[j->p_dir]);
	    j->p_y += (int)((j->p_speed * WARP1) * Sin[j->p_dir]);
	    if (!shipwrap) {
	       /* Bounce off the side of the galaxy */
	       if (j->p_x < 0) {
		  j->p_x = -j->p_x;
		  if (j->p_dir == 192)
		     j->p_dir = j->p_desdir = 64;
		  else
		     j->p_dir = j->p_desdir = 64 - (j->p_dir - 192);
	       } else if (j->p_x > GWIDTH) {
		  j->p_x = GWIDTH - (j->p_x - GWIDTH);
		  if (j->p_dir == 64)
		     j->p_dir = j->p_desdir = 192;
		  else
		     j->p_dir = j->p_desdir = 192 - (j->p_dir - 64);
	       } if (j->p_y < 0) {
		  j->p_y = -j->p_y;
		  if (j->p_dir == 0)
		     j->p_dir = j->p_desdir = 128;
		  else
		     j->p_dir = j->p_desdir = 128 - j->p_dir;
	       } else if (j->p_y > GWIDTH) {
		  j->p_y = GWIDTH - (j->p_y - GWIDTH);
		  if (j->p_dir == 128)
		     j->p_dir = j->p_desdir = 0;
		  else
		     j->p_dir = j->p_desdir = 0 - (j->p_dir - 128);
	       }
	    } else {
	       if (j->p_x < 0) {
		  j->p_x = GWIDTH + j->p_x;
	       } else if (j->p_x > GWIDTH) {
		  j->p_x = 0 + (j->p_x - GWIDTH);
	       } if (j->p_y < 0) {
		  j->p_y = GWIDTH + j->p_y;
	       } else if (j->p_y > GWIDTH) {
		  j->p_y = 0 + (j->p_y - GWIDTH);
	       }
	    }
	 }
	 if (!(j->p_flags & PFROBOT)) 
	    pcount[j->p_team] ++;

	 /* If player is actually dead, don't do anything below ... */
	 if (j->p_status == PEXPLODE || j->p_status == PDEAD)
	    break;

#ifdef BASEPRACTICE
	 if(j->p_ship.s_type==STARBASE) {
	    j->p_stats.st_sbticks += tickinc;
	    j->p_stats.st_tticks += tickinc;
	    status->timeprod += tickinc;
	 }
	 else
#endif
	 if (j->p_ship.s_type == STARBASE) {
	    j->p_stats.st_sbticks++;
	 } else if (status->tourn) {
	    j->p_stats.st_tticks += tickinc;
	    status->timeprod += tickinc;
	 } else {
	    j->p_stats.st_ticks += tickinc;
	 }

	 if (++(j->p_ghostbuster) > GHOSTTIME) {
	    j->p_status = PEXPLODE;
	    if (j->p_ship.s_type == STARBASE)
	       j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
	    else
	       j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
	    ghostmess(j);
	    saveplayer(j);
	    j->p_whydead = KGHOST;
	    j->p_whodead = i;
	 }
	 status->active += (1 << i);
	 tcount[j->p_team]++;
	 j->p_updates++;

	 /* Charge for shields */
	 if (j->p_flags & PFSHIELD) {	/* This is a kludge, I know. */
	    switch (j->p_ship.s_type) {
	    case SCOUT:
	       j->p_fuel -= 2 * UPDATE_NORM;
	       break;
	    case DESTROYER:
	    case CRUISER:
	    case BATTLESHIP:
	    case GALAXY:
	    case ASSAULT:
	       j->p_fuel -= 3 * UPDATE_NORM;
	       break;
	    case STARBASE:
	       j->p_fuel -= 6 * UPDATE_NORM;
	       break;
	    default:
	       break;
	    }
	 }
	 /* affect tractor beams */
	 if ((j->p_flags & PFTRACT)) {
	    if ((isAlive(&players[j->p_tractor])) &&
		((j->p_fuel > TRACTCOST) && !(j->p_flags & PFENG)) &&
		((dist = hypot((double) (j->p_x - players[j->p_tractor].p_x),
			     (double) (j->p_y - players[j->p_tractor].p_y)))
		 < ((double) TRACTDIST) * j->p_ship.s_tractrng) &&
		(!(j->p_flags & (PFORBIT | PFDOCK))) &&
		(!(players[j->p_tractor].p_flags & PFDOCK))) {
	       float           cosTheta, sinTheta;	/* Cos and Sin from me
							 * to him */
	       int             halfforce;	/* Half force of tractor */
	       int             dir;	/* -1 for repress, otherwise 1 */

	       if (players[j->p_tractor].p_flags & PFORBIT)
		  players[j->p_tractor].p_flags &= ~PFORBIT;
#ifdef SB_TRANSWARP
	       if (players[j->p_tractor].p_flags & PFTWARP)
		  players[j->p_tractor].p_flags &= ~PFTWARP;
#endif

	       j->p_fuel -= TRACTCOST;
	       j->p_etemp += TRACTEHEAT;


	       cosTheta = players[j->p_tractor].p_x - j->p_x;
	       sinTheta = players[j->p_tractor].p_y - j->p_y;
	       if (dist == 0)
		  dist = 1;	/* prevent divide by zero */
	       cosTheta /= dist;/* normalize sin and cos */
	       sinTheta /= dist;
	       /* force of tractor is WARP2 * tractstr */
	       halfforce = (WARP1 * j->p_ship.s_tractstr);

	       dir = 1;
	       if (j->p_flags & PFPRESS)
		  dir = -1;
	       /* change in position is tractor strength over mass */
	       j->p_x += (int)((dir * halfforce) * cosTheta / j->p_ship.s_mass);
	       j->p_y += (int)((dir * halfforce) * sinTheta / j->p_ship.s_mass);
	       players[j->p_tractor].p_x -= (int) ((dir * halfforce) * cosTheta
		  / players[j->p_tractor].p_ship.s_mass);
	       players[j->p_tractor].p_y -= (int) ((dir * halfforce) * sinTheta
		  / players[j->p_tractor].p_ship.s_mass);
	    } else {
	       j->p_flags &= ~(PFTRACT | PFPRESS);
	    }
	 }
	 /* cool weapons */
	 j->p_wtemp -= j->p_ship.s_wpncoolrate;
	 if (j->p_wtemp < 0)
	    j->p_wtemp = 0;
	 if (j->p_flags & PFWEP) {
	    if (--j->p_wtime <= 0)
	       j->p_flags &= ~PFWEP;
	 } else if (j->p_wtemp > j->p_ship.s_maxwpntemp) {
	    if (!(random() % 40)) {
	       j->p_flags |= PFWEP;
	       j->p_wtime = ((random() % 150) + 100) / PLAYERFUSE;
	    }
	 }
	 /* cool engine */
	 j->p_etemp -= j->p_ship.s_egncoolrate;
	 if (j->p_etemp < 0)
	    j->p_etemp = 0;
	 if (j->p_flags & PFENG) {
	    if (--j->p_etime <= 0)
	       j->p_flags &= ~PFENG;
	 } else if (j->p_etemp > j->p_ship.s_maxegntemp) {
	    if (!(random() % 40)) {
	       j->p_flags |= PFENG;
	       j->p_etime = ((random() % 150) + 100) / PLAYERFUSE;
	       j->p_desspeed = 0;
	    }
	 }
	 /* Charge for cloaking */
	 if (j->p_flags & PFCLOAK) {
	    if (j->p_fuel < j->p_ship.s_cloakcost) {
	       j->p_flags &= ~PFCLOAK;
	    } else {
	       j->p_fuel -= j->p_ship.s_cloakcost;
	    }
	 }
	 /* Add fuel */
	 j->p_fuel += 2 * j->p_ship.s_recharge;
	 if ((j->p_flags & PFORBIT) &&
	     (planets[j->p_planet].pl_flags & PLFUEL) &&
	     (!(planets[j->p_planet].pl_owner
		& (j->p_swar | j->p_hostile)))) {
	    j->p_fuel += 6 * j->p_ship.s_recharge;
	 } else if ((j->p_flags & PFDOCK) &&
		    (j->p_fuel < j->p_ship.s_maxfuel) &&
		    (players[j->p_docked].p_fuel > SBFUELMIN)) {
	    int             fc = MIN(10 * j->p_ship.s_recharge,
				     j->p_ship.s_maxfuel - j->p_fuel);
	    j->p_fuel += fc;
	    players[j->p_docked].p_fuel -= fc;
	 }

	 if (j->p_fuel > j->p_ship.s_maxfuel)
	    j->p_fuel = j->p_ship.s_maxfuel;
	 if (j->p_fuel < 0) {
	    j->p_desspeed = 0;
	    j->p_flags &= ~PFCLOAK;
	 }
	 /* repair shields */
	 if (j->p_shield < j->p_ship.s_maxshield) {
	    if ((j->p_flags & PFREPAIR) && (j->p_speed == 0)) {
	       j->p_subshield += j->p_ship.s_repair * 4;
	       if ((j->p_flags & PFORBIT) &&
		   (planets[j->p_planet].pl_flags & PLREPAIR) &&
		   (!(planets[j->p_planet].pl_owner
		      & (j->p_swar | j->p_hostile)))) {
		  j->p_subshield += j->p_ship.s_repair * 4;
	       }
	       if (j->p_flags & PFDOCK)
		  j->p_subshield += 6 * j->p_ship.s_repair;
	    } else
	       j->p_subshield += j->p_ship.s_repair * 2;
	    if (j->p_subshield / 1000) {
	       j->p_shield += j->p_subshield / 1000;
#ifdef BASEPRACTICE
	       if(isSB(j)){
		  j->p_stats.st_damr += j->p_subshield/1000;
		  status->damr += j->p_subshield/1000;
	       }
#endif

	       j->p_subshield %= 1000;
	    }
	    if (j->p_shield > j->p_ship.s_maxshield) {
	       j->p_shield = j->p_ship.s_maxshield;
	       j->p_subshield = 0;
	    }
	 }
	 /* repair damage */
	 if (j->p_damage && !(j->p_flags & PFSHIELD)) {
	    if ((j->p_flags & PFREPAIR) && (j->p_speed == 0)) {
	       j->p_subdamage += j->p_ship.s_repair * 2;
	       if ((j->p_flags & PFORBIT) &&
		   (planets[j->p_planet].pl_flags & PLREPAIR) &&
		   (!(planets[j->p_planet].pl_owner
		      & (j->p_swar | j->p_hostile)))) {
		  j->p_subdamage += j->p_ship.s_repair * 2;
	       }
	       if (j->p_flags & PFDOCK)
		  j->p_subdamage += j->p_ship.s_repair * 3;
	    } else
	       j->p_subdamage += j->p_ship.s_repair;
	    if (j->p_subdamage / 1000) {
	       j->p_damage -= j->p_subdamage / 1000;
#ifdef BASEPRACTICE
	       if(isSB(j)){
		  j->p_stats.st_damr += j->p_subdamage / 1000;
		  status->damr += j->p_subdamage / 1000;
	       }
#endif
	       j->p_subdamage %= 1000;
	    }
	    if (j->p_damage < 0) {
	       j->p_damage = 0;
	       j->p_subdamage = 0;
	    }
	 }
	 /* Move Player in dock */
	 if (j->p_flags & PFDOCK) {
#ifdef SB_DOCK
	    int index = (j->p_portdocked * 90 + 45) * 255 / 360;
#else
	    int index = (j->p_port[0] * 90 + 45) * 255 / 360;
#endif
	    j->p_x = players[j->p_docked].p_x + (int)(DOCKDIST * Cos[index]);
	    j->p_y = players[j->p_docked].p_y + (int)(DOCKDIST * Sin[index]);
	 }
	 /* Set player's alert status */
#define YRANGE ((GWIDTH)/7)
#define RRANGE ((GWIDTH)/10)

	 alert_status = PFGREEN;
	 for (k = 0; k < MAXPLAYER; k++) {
	    int             dx, dy, dist;
	    if ((players[k].p_status != PALIVE) ||
		((!((j->p_swar | j->p_hostile) & players[k].p_team)) &&
		 (!((players[k].p_swar | players[k].p_hostile) &
		    j->p_team)))) {
	       continue;
	    } else if (j == &players[k]) {
	       continue;
	    } else {
	       dx = j->p_x - players[k].p_x;
	       dy = j->p_y - players[k].p_y;
	       if (ABS(dx) > YRANGE || ABS(dy) > YRANGE)
		  continue;
	       dist = dx * dx + dy * dy;
	       if (dist < RRANGE * RRANGE) {
		  alert_status = PFRED;
		  break;	/* we don't need to go further here */
	       } else if (dist < YRANGE * YRANGE) {
		  alert_status = PFYELLOW;
	       }
	    }
	 }
	 j->p_flags &= ~(PFGREEN | PFRED | PFYELLOW);
	 j->p_flags |= alert_status;
	 break;
      }				/* end switch */
   }
   if (nplayers == MAXPLAYER) {
      if (dietime == -1)
	 dietime = ticks + 600 / PLAYERFUSE;
   } else {
      dietime = -1;
   }
}

void
changedir(sp)
   struct player  *sp;
{
   unsigned int    ticks;
   unsigned int    min, max;

   if (sp->p_speed == 0) {
      sp->p_dir = sp->p_desdir;
      sp->p_subdir = 0;
   } else {
      sp->p_subdir += sp->p_ship.s_turns / ((sp->p_speed < 30) ? (1 << sp->p_speed) : 1000000000);
      ticks = sp->p_subdir / 1000;
      if (ticks) {
	 if (sp->p_dir > sp->p_desdir) {
	    min = sp->p_desdir;
	    max = sp->p_dir;
	 } else {
	    min = sp->p_dir;
	    max = sp->p_desdir;
	 }
	 if ((ticks > max - min) || (ticks > 256 - max + min))
	    sp->p_dir = sp->p_desdir;
	 else if ((unsigned char) ((int) sp->p_dir - (int) sp->p_desdir) > 127)
	    sp->p_dir += ticks;
	 else
	    sp->p_dir -= ticks;
	 sp->p_subdir %= 1000;
      }
   }
}

void
udcloak()
{
   register int    i;
   register struct player *j;	/* avoid indirect calc -- tsh */

   for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
      if (isAlive(j))
	 if ((j->p_flags & PFCLOAK) &&
	     (j->p_cloakphase < (CLOAK_PHASES - 1))) {
	    j->p_cloakphase ++;
	 } else if ((j->p_cloakphase > 0) &&
		   !(j->p_flags & PFCLOAK)){
	    j->p_cloakphase --;
	 }
   }
}

void
udtorps()
{
   register int    i, turn = 0, heading = 0;
   register struct torp *j;

   for (i = 0, j = &torps[i]; i < MAXPLAYER * MAXTORP; i++, j++) {
      switch (j->t_status) {
      case TFREE:
	 continue;
      case TMOVE:
      case TSTRAIGHT:
#ifdef MINES
	 /*
	  * if player dies the fuse count goes to zero and the mine torps
	  * vanish
	  */
	 if (j->t_mine && players[j->t_owner].p_status != PDEAD &&
	     players[j->t_owner].p_ship.s_type == STARBASE) {
	    if (fuse(minerot))
	       rotmine(j);
	    if (near(j)) {
	       explode(j);
	       if (torps[j->t_pair].t_status != TEXPLODE)
		  explode(&torps[j->t_pair]);
	    }
	    break;
	 }
#endif

	 if (j->t_turns > 0) {
	    turn = torp_track_target(j);
	    if (turn < 0) {
	       heading = ((int) j->t_dir) - j->t_turns;
	       j->t_dir = ((heading < 0) ? ((unsigned char) (256 + heading)) :
			   ((unsigned char) heading));
	    } else if (turn > 0) {
	       heading = ((int) j->t_dir) + j->t_turns;
	       j->t_dir = ((heading > 255) ? ((unsigned char) (heading - 256)) :
			   ((unsigned char) heading));
	    }
	 }
	 j->t_x += (int)((j->t_speed * WARP1) * Cos[j->t_dir]);
	 if (j->t_x < 0) {
	    j->t_x = 0;
	    j->t_status = TDET; /* no wall kills 8/31/91 TC */
	    j->t_whodet = j->t_owner; /* 8/31/91 TC */
	    explode(j);
	    break;
	 } else if (j->t_x > GWIDTH) {
	    j->t_x = GWIDTH;
	    j->t_status = TDET; /* no wall kills 8/31/91 TC */
	    j->t_whodet = j->t_owner; /* 8/31/91 TC */
	    explode(j);
	    break;
	 }
	 j->t_y += (int)((j->t_speed * WARP1) * Sin[j->t_dir]);
	 if (j->t_y < 0) {
	    j->t_y = 0;
            j->t_status = TDET; /* no wall kills 8/31/91 TC */
            j->t_whodet = j->t_owner; /* 8/31/91 TC */
	    explode(j);
	    break;
	 } else if (j->t_y > GWIDTH) {
	    j->t_y = GWIDTH;
            j->t_status = TDET; /* no wall kills 8/31/91 TC */
            j->t_whodet = j->t_owner; /* 8/31/91 TC */
	    explode(j);
	    break;
	 }
	 /* Make sure that player torps wobble */
	 if (j->t_status == TMOVE)
	    j->t_dir += (random() % 3) - 1;

	 if (j->t_fuse-- <= 0) {
	    if (lastexplode)	/* jfy -- exploding torps */
	       explode(j);
	    else {
	       j->t_status = TFREE;
	       players[j->t_owner].p_ntorp--;
	    }
	 } else if (near(j)) {
	    explode(j);
	 }
	 break;
      case TDET:
	 explode(j);
	 break;
      case TEXPLODE:
	 if (j->t_fuse-- <= 0) {
	    j->t_status = TFREE;
	    players[j->t_owner].p_ntorp--;
	 }
	 break;
      case TOFF:
	 /* jfy -- exploding torps */
	 if (expltorps) {
	    if (players[j->t_owner].p_ntorp == 1)
	       explode(j);
	    else {
	       j->t_status = TFREE;
	       players[j->t_owner].p_ntorp--;
	    }
	 } else {
	    j->t_status = TFREE;
	    players[j->t_owner].p_ntorp--;
	 }
	 break;
      default:			/* Shouldn't happen */
	 j->t_status = TFREE;
	 break;
      }
   }
}

void
udplasmatorps()
{
   register int    i, turn = 0, heading;
   register struct plasmatorp *j;

   for (i = 0, j = &plasmatorps[i]; i < MAXPLAYER * MAXPLASMA; i++, j++) {

      switch (j->pt_status) {
      case PTFREE:
	 continue;
      case PTMOVE:
	 turn = plasma_track_target(j);
	 if (turn < 0) {
	    heading = ((int) j->pt_dir) - j->pt_turns;
	    j->pt_dir = ((heading < 0) ? ((unsigned char) (256 + heading)) :
			 ((unsigned char) heading));
	 } else if (turn > 0) {
	    heading = ((int) j->pt_dir) + j->pt_turns;
	    j->pt_dir = ((heading > 255) ? ((unsigned char) (heading - 256)) :
			 ((unsigned char) heading));
	 }
	 j->pt_x += (int)((j->pt_speed * WARP1) * Cos[j->pt_dir]);
	 if (j->pt_x < 0) {
	    j->pt_x = 0;
	    pexplode(j);
	    break;
	 } else if (j->pt_x > GWIDTH) {
	    j->pt_x = GWIDTH;
	    pexplode(j);
	    break;
	 }
	 j->pt_y += (int)((j->pt_speed * WARP1) * Sin[j->pt_dir]);
	 if (j->pt_y < 0) {
	    j->pt_y = 0;
	    pexplode(j);
	    break;
	 } else if (j->pt_y > GWIDTH) {
	    j->pt_y = GWIDTH;
	    pexplode(j);
	    break;
	 }
	 if (j->pt_fuse-- <= 0) {
	    j->pt_status = PTFREE;
	    players[j->pt_owner].p_nplasmatorp--;
	 } else if (pnear(j)) {
	    pexplode(j);
	 }
	 break;
      case PTDET:
	 pexplode(j);
	 break;
      case PTEXPLODE:
	 if (j->pt_fuse-- <= 0) {
	    j->pt_status = PTFREE;
	    players[j->pt_owner].p_nplasmatorp--;
	 }
	 break;
      default:			/* Shouldn't happen */
	 j->pt_status = PTFREE;
	 break;
      }
   }
}

/* See if there is someone close enough to explode for */
int
near(torp)
   struct torp    *torp;
{
   register int    i;
   register int    dx, dy;
   register struct player *j;

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (!(j->p_status == PALIVE))
	 continue;
      if (torp->t_owner == j->p_no)
	 continue;
      /*
       * This is tricky.  If you aren't at war with the team, and that team
       * is not at war with your team, continue.
       */
      if (((!(torp->t_war & j->p_team)) &&
	   (!(torp->t_team & (j->p_swar | j->p_hostile)))))
	 continue;
      dx = torp->t_x - j->p_x;
      dy = torp->t_y - j->p_y;
#ifdef MINES
      if (torp->t_mine) {
	 if (ABS(dx) > minedet || ABS(dy) > minedet)
	    continue;
	 if (dx * dx + dy * dy < minedet * minedet)
	    return 1;
      } else
#endif
      {
	 if (ABS(dx) > EXPDIST || ABS(dy) > EXPDIST)
	    continue;
	 if (dx * dx + dy * dy < (EXPDIST * EXPDIST))
	    return 1;
      }
   }
   return 0;
}


/* See if there is someone close enough to explode for */
int
pnear(plasmatorp)
   struct plasmatorp *plasmatorp;
{
   register int    i, expdist = EXPDIST;
   int             dx, dy;
   register struct player *j;

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (!(j->p_status == PALIVE))
	 continue;
      if (plasmatorp->pt_owner == j->p_no)
	 continue;
      /*
       * This is tricky.  If you aren't at war with the team, and that team
       * is not at war with your team, continue.
       */
      if ((!(plasmatorp->pt_war & j->p_team)) &&
	  (!(plasmatorp->pt_team & (j->p_swar | j->p_hostile))))
	 continue;
      dx = plasmatorp->pt_x - j->p_x;
      dy = plasmatorp->pt_y - j->p_y;
      if (ABS(dx) > expdist || ABS(dy) > expdist)
	 continue;
      if (dx * dx + dy * dy < (expdist) * (expdist))
	 return 1;
   }
   return 0;
}



/* Do damage to all surrounding players */

void
explode(torp)
   struct torp    *torp;
{
   register int    i;
   int             dx, dy, dist;
   int             damage;
   register struct player *j;
#if defined(CHAIN_REACTION) || defined(BASEPRACTICE)
   register struct player *killer;
#endif
#ifdef BASEPRACTICE
   double	   d;
#endif

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (!(j->p_status == PALIVE))
	 continue;
      if (torp->t_owner == j->p_no
#ifdef MINES
	  && !torp->t_mine	/* mine will damage owner also */
#endif
	 )			/* This isn't realistic */
	 continue;
      /*
       * Now, only the person who detonated the torpedo and people on one of
       * the other teams takes damage. This prevents hosers from killing
       * teammates with the 'd' key
       */
      if (torp->t_status == TDET &&
#if defined(CHAIN_REACTION) || defined(BASEPRACTICE)  /* XXX shouldn't happen
							 but does. */
          torp->t_whodet != NODET &&
#endif
	  j->p_no != torp->t_whodet &&
	  j->p_team == players[torp->t_whodet].p_team) {
	 continue;
      }
      dx = torp->t_x - j->p_x;
      dy = torp->t_y - j->p_y;

#ifdef MINES			/* slightly different calculations for
				 * minedet */
      damage = 0;

      if (torp->t_mine) {

	 /*
	  * small threshold to make sure both torps do damage to the same
	  * player
	  */
	 if (ABS(dx) > minedet + 500 || ABS(dy) > minedet + 500)
	    continue;

	 dist = dx * dx + dy * dy;
	 if (dist <= (minedet + 500) * (minedet + 500)) {
	    /* everyone within range gets full damage */
	    damage = torp->t_damage;
	 } else
	    continue;
      } else
#endif
      {
	 if (ABS(dx) > DAMDIST || ABS(dy) > DAMDIST)	/* XXX */
	    continue;
	 dist = dx * dx + dy * dy;
	 if (dist > DAMDIST * DAMDIST)
	    continue;
	 if (dist > EXPDIST * EXPDIST) {
	    damage = (int)(torp->t_damage * (DAMDIST - sqrt((double)dist)) /
		     (DAMDIST - EXPDIST));
	    MATH_ERR_CHECK(dist);
	 } else {
	    damage = torp->t_damage;
	 }
      }

      if (damage > 0) {
	 /* First, check to see if torp owner has started a war */
	 if (players[torp->t_owner].p_hostile & j->p_team) {
	    players[torp->t_owner].p_swar |= j->p_team;
	 }
	 /*
	  * Note that if a player is at peace with the victim, then the torp
	  * has caused damage either accidently, or because the victim was at
	  * war with, or hostile to, the player. In either case, we don't
	  * consider the damage to be an act of war
	  */
#ifdef BASEPRACTICE
	 killer = ((torp->t_whodet != NODET)
		   && (torp->t_whodet != j->p_no)
		   && (players[torp->t_whodet].p_team != j->p_team)) ?
	    &players[torp->t_whodet] : &players[torp->t_owner];

	    /* figure in j's defense to damage */
	    if(isSB(killer) && (j->p_flags & PFBPROBOT)){
	       d = def_factor(j, damage, killer);
	       killer->p_stats.st_dami += d; 
	       status->dami += d;
	    }
#ifdef nodef
	    if(isSB(j) && (killer->p_flags & PFBPROBOT)){
	       /* figure in killer's offense to damage */
	       d=off_factor(killer, damage, j);
	       j->p_stats.st_damr += d; 
	       status->damr += d;
	    }
#endif
#endif
	 if (j->p_flags & PFSHIELD) {
	    j->p_shield -= damage;
	    if (j->p_shield < 0) {
	       j->p_damage -= j->p_shield;
	       j->p_shield = 0;
	    }
	 } else {
	    j->p_damage += damage;
	 }
	 if (j->p_damage >= j->p_ship.s_maxdamage) {
	    j->p_status = PEXPLODE;
	    if (j->p_ship.s_type == STARBASE) {
	       j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
	    } else {
	       j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
	    }
#if defined(CHAIN_REACTION) && !defined(BASEPRACTICE)
	    killer = ((torp->t_whodet != NODET)
		      && (torp->t_whodet != j->p_no)
		      && (players[torp->t_whodet].p_team != j->p_team)) ?
	       &players[torp->t_whodet] : &players[torp->t_owner];
#endif
#ifdef CHAIN_REACTION
	    if ((killer->p_team != j->p_team) ||
		((killer->p_team == j->p_team) && (j->p_flags & PFPRACTR))) {
	       killer->p_kills += 1.0 +
		  j->p_armies * 0.1 + j->p_kills * 0.1;
	       checkmaxkills(killer->p_no);
	    }
	    killerstats(killer->p_no, torp->t_owner, j);
	    loserstats(i);
	    j->p_whydead = KTORP;
	    j->p_whodead = killer->p_no;
	    killmess(j, killer, &players[torp->t_owner],
		     (killer->p_no != torp->t_owner) ? KTORP2 : KTORP);
#else

	    killerstats(torp->t_owner, torp->t_owner, j);
	    players[torp->t_owner].p_kills += 1.0 +
	       j->p_armies * 0.1 + j->p_kills * 0.1;
	    checkmaxkills(torp->t_owner);
	    loserstats(i);
	    j->p_whydead = KTORP;
	    j->p_whodead = torp->t_owner;
	    killmess(j, &players[torp->t_owner], &players[torp->t_owner], 
		     KTORP);
#endif				/* CHAIN_REACTION */
	 }
      }
   }
   torp->t_status = TEXPLODE;
   torp->t_fuse = 10 / TORPFUSE;
}

/* Do damage to all surrounding players */

void
pexplode(plasmatorp)
   struct plasmatorp *plasmatorp;
{
   register int    i;
   int             dx, dy, dist;
   int             damage;
   register struct player *j;
#if defined(CHAIN_REACTION) || defined(BASEPRACTICE)
   register struct player	*killer;
#endif
#ifdef BASEPRACTICE
   double	   d;
#endif

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (!(j->p_status == PALIVE))
	 continue;

      /*
       * Testing to see if this helps make them risky ... (Scott: 12/10/88)
       * if (plasmatorp->pt_owner == j->p_no)	/ This isn't realistic /
       * continue;
       */

      dx = plasmatorp->pt_x - j->p_x;
      dy = plasmatorp->pt_y - j->p_y;
      if (ABS(dx) > PLASDAMDIST || ABS(dy) > PLASDAMDIST)	/* XXX */
	 continue;
      dist = dx * dx + dy * dy;
      if (dist > PLASDAMDIST * PLASDAMDIST)
	 continue;
      if (dist > (EXPDIST) * (EXPDIST)) {
	 damage = (int)(plasmatorp->pt_damage * 
	    (PLASDAMDIST - sqrt((double) dist)) / (PLASDAMDIST - (EXPDIST)));
	 MATH_ERR_CHECK(dist);
      } else {
	 damage = plasmatorp->pt_damage;
      }
      if (damage > 0) {
	 /* First, check to see if torp owner has started a war */
	 if (players[plasmatorp->pt_owner].p_hostile & j->p_team) {
	    players[plasmatorp->pt_owner].p_swar |= j->p_team;
	 }
	 /*
	  * Note that if a player is at peace with the victim, then the torp
	  * has caused damage either accidently, or because the victim was at
	  * war with, or hostile to, the player. In either case, we don't
	  * consider the damage to be an act of war
	  */
#ifdef BASEPRACTICE
	 killer = &players[plasmatorp->pt_owner];
	 if(isSB(killer) && (j->p_flags & PFBPROBOT)){
	    d=def_factor(j, damage, killer);
	    killer->p_stats.st_dami += d; 
	    status->dami += d;
	 }
#ifdef nodef
	 if(isSB(j) && (killer->p_flags & PFBPROBOT)){
	    d=off_factor(killer,damage, j);
	    j->p_stats.st_damr += d; 
	    status->damr += d;
	 }
#endif
#endif
	 if (j->p_flags & PFSHIELD) {
	    j->p_shield -= damage;
	    if (j->p_shield < 0) {
	       j->p_damage -= j->p_shield;
	       j->p_shield = 0;
	    }
	 } else {
	    j->p_damage += damage;
	 }
	 if (j->p_damage >= j->p_ship.s_maxdamage) {
	    j->p_status = PEXPLODE;
	    if (j->p_ship.s_type == STARBASE)
	       j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
	    else
	       j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
#if defined(CHAIN_REACTION) && !defined(BASEPRACTICE)
	    /* XXX: need pt_whodet field to do this correctly */
	    killer = &players[plasmatorp->pt_owner];
#endif
#ifdef CHAIN_REACTION
	    if(killer->p_team != j->p_team ||
	       ((killer->p_team == j->p_team) && (j->p_flags & PFPRACTR))){
	       killer->p_kills += 1.0 + j->p_armies * 0.1 + j->p_kills*0.1;
	       checkmaxkills(killer->p_no);
	    }
	    killerstats(killer->p_no, plasmatorp->pt_owner, j);
	    loserstats(i);
	    j->p_whydead = KPLASMA;
	    j->p_whodead = killer->p_no;
	    killmess(j, killer, &players[plasmatorp->pt_owner], KPLASMA);
#else
	    killerstats(plasmatorp->pt_owner, plasmatorp->pt_owner, j);
	    players[plasmatorp->pt_owner].p_kills += 1.0 +
	       j->p_armies * 0.1 + j->p_kills * 0.1;
	    checkmaxkills(plasmatorp->pt_owner);
	    killmess(j, &players[plasmatorp->pt_owner], 
			&players[plasmatorp->pt_owner], KPLASMA);
	    loserstats(i);
	    j->p_whydead = KPLASMA;
	    j->p_whodead = plasmatorp->pt_owner;
#endif
	 }
      }
   }
   plasmatorp->pt_status = PTEXPLODE;
   plasmatorp->pt_fuse = 10 / PLASMAFUSE;
}

void
loserstats(pl)
   int             pl;
{
   struct player  *dude;

   dude = &players[pl];

   if (dude->p_ship.s_type == STARBASE) {
      dude->p_stats.st_sblosses++;
#ifdef BASEPRACTICE
      dude->p_stats.st_tlosses++;
      status->losses++;
#endif
   } else {
      if (status->tourn) {
	 dude->p_stats.st_tlosses++;
	 status->losses++;
      } else {
	 dude->p_stats.st_losses++;
      }
   }
}

void
killerstats(cl, pl, victim)
   int             cl, pl;
   struct player  *victim;
{
   struct player  *dude, *credit_killer;

   credit_killer = &players[cl];
   dude = &players[pl];

   if(credit_killer->p_team == dude->p_team)
      credit_killer = dude;

   if (credit_killer->p_ship.s_type == STARBASE) {
      credit_killer->p_stats.st_sbkills++;
      credit_killer->p_stats.st_armsbomb += 5 * victim->p_armies;
#ifdef BASEPRACTICE
      credit_killer->p_stats.st_tkills++;
      status->kills++;
#endif
   } else {
      if (status->tourn) {
	 status->kills++;
	 if(credit_killer->p_team != victim->p_team){	/* NO CREDIT */
	    credit_killer->p_stats.st_tkills++;
	    credit_killer->p_stats.st_tarmsbomb += 5 * victim->p_armies;
	    status->armsbomb += 5 * victim->p_armies;
	 }
      } else {
	 if(credit_killer->p_team != victim->p_team){	/* NO CREDIT */
	    credit_killer->p_stats.st_kills++;
	    credit_killer->p_stats.st_armsbomb += 5 * victim->p_armies;
	 }
      }
   }
}

void
checkmaxkills(pl)
   int             pl;
{
   struct stats   *stats;
   struct player  *dude;

   dude = &(players[pl]);
   stats = &(dude->p_stats);

   if (dude->p_ship.s_type == STARBASE) {
      if (stats->st_sbmaxkills < dude->p_kills) {
	 stats->st_sbmaxkills = dude->p_kills;
      }
   } else if (stats->st_maxkills < dude->p_kills) {
      stats->st_maxkills = dude->p_kills;
   }
}

#ifdef INL_POP
/*
 * isae -- Note that I rewrote the whole thing for optimization I merged all
 * the functions to udplanets() and used my own scheme to randomize the
 * planet pop order
 */
/* updates planets, grows armies on them, etc */
static int      pl_poporder[MAXPLANETS];        /* planet number to pop next */
void
udplanets()
{
   register struct planet *l;
   register int    i;
   int             plnum;
   static int	   init;
   static int      lastpop;     /* isae -- moved here */

   /*
    * reorder order in which planets will be populated set lastpop to
    * MAXPLANETS
    */
   if (--lastpop < 0) {
      register	tmp, j;
      if (debug)
         printf("Randomizing planetary pop order!\n");
#ifdef nodef
      for (i = 0; i < MAXPLANETS; i++)
         picked_planets[i] = 0;
      for (i = 0; i < MAXPLANETS; i++) {
         temp = random() % MAXPLANETS;
         while (picked_planets[temp] > 0)
            temp = random() % MAXPLANETS;
         picked_planets[temp] = 1;
         pl_poporder[i] = temp;
      }
#endif
      if(!init){
	 for(i=0; i< MAXPLANETS; i++)
	    pl_poporder[i] = i;
	 init = 1;
      }

      for(i=0; i< MAXPLANETS; i++){
	 tmp = pl_poporder[i];
	 j = random()%40;
	 pl_poporder[i] = pl_poporder[j];
	 pl_poporder[j] = tmp;
      }
	 

      if (debug) {
         printf("Planet pop order:");
         for (i = 0; i < MAXPLANETS; i++)
            printf(" %d", pl_poporder[i]);
         printf("\n");
      }
      lastpop = MAXPLANETS - 1;
   }
   plnum = pl_poporder[lastpop];
   if (debug)
      printf("populating planet %d\n", plnum);

   if (plnum < 0 || plnum >= MAXPLANETS)
      return;                   /* arg sanity */
   l = &planets[plnum];

   if (l->pl_armies == 0)
      return;

   if ((random() % 3000) < l->pl_armies) {      /* plague! */
      l->pl_armies -= (random() % l->pl_armies);
      if (l->pl_armies < 5)
         l->pl_flags |= PLREDRAW;
   }
   if (l->pl_armies < 4) {
      if ((random() % 20) == 0) {       /* planets< 4 grow faster */
         l->pl_armies++;
      }
      if (l->pl_flags & PLAGRI) {       /* Always grow 1 if Agricultural. */
         if (++l->pl_armies > 4)
            l->pl_flags |= PLREDRAW;
      }
   }
   if ((random() % 10) == 0) {  /* Chance for extra armies. */
      if (l->pl_armies < 5)
         l->pl_flags |= PLREDRAW;
      l->pl_armies += (random() % 3) + 1;
   }
   /* Argicultural worlds have extra chance for army growth. */
   if (((random() % 5) == 0) && (l->pl_flags & PLAGRI)) {
      if (++l->pl_armies > 4)
         l->pl_flags |= PLREDRAW;
   }
}

#else		/* INL_POP */

void
udplanets()
{
   register int    i;
   register struct planet *l;

   for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++) {
      if (l->pl_couptime)	/* Decrement coup counter one minute */
	 l->pl_couptime--;
      if (l->pl_armies == 0)
	 continue;
      if ((random() % 3000) < l->pl_armies) {
	 l->pl_armies -= (random() % l->pl_armies);
	 if (l->pl_armies < 5)
	    l->pl_flags |= PLREDRAW;
      }
      if (l->pl_armies < 4) {
	 if ((random() % 20) == 0) {
	    l->pl_armies++;
	 }
	 if (l->pl_flags & PLAGRI) {	/* Always grow 1 if Agricultural. */
	    if (++l->pl_armies > 4)
	       l->pl_flags |= PLREDRAW;
	 }
      }
      if ((random() % 10) == 0) {	/* Chance for extra armies. */
	 if (l->pl_armies < 5)
	    l->pl_flags |= PLREDRAW;
	 l->pl_armies += (random() % 3) + 1;
      }
      /* Argicultural worlds have extra chance for army growth. */
      if (((random() % 5) == 0) && (l->pl_flags & PLAGRI)) {
	 if (++l->pl_armies > 4)
	    l->pl_flags |= PLREDRAW;
      }
   }
}

#endif	/* INL_POP */

/* new code TC */
void
udsurrend()
{
   register int    i, t;
   char            buf[80];
   struct planet  *l;
   struct player  *j, *win;

   /* Decrement SB reconstruction timer code moved here (from udplanets()) */

   for (i = 0; i <= MAXTEAM; i++) {
      (teams[i].s_turns--);
   }
   /* Coup timer moved here (from udplanets()) */

   for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++) {
      if (l->pl_couptime)	/* Decrement coup counter one minute */
	 l->pl_couptime--;
   }

   /* All tmode-only code must follow this statement. */

   if (!status->tourn)
      return;

#if 0
   /* Terminate non-warring humans 9/1/91 TC */

   for (i = 0, j = &players[0]; i < MAXPLAYER; i++, j++) {
      if ((j->p_status == PALIVE) &&
	  !(j->p_flags & PFROBOT) &&
	  (j->p_team != NOBODY) &&
	  (pcount[j->p_team] < tournplayers) &&
	  (random() % 5 == 0))
	 rescue(j->p_team, j);
   }
#endif

#ifdef BIRD
   /* put terminator-style check here */
   if (status->tourn) {
      struct player  *j;
      int             bc = 0, c = -1;
      for (i = 0, j = &players[0]; i < MAXPLAYER; i++, j++) {
	 if (j->p_flags & PFBIRD) {
	    bc++;
	    continue;
	 }
	 if ((j->p_status == PALIVE) &&
	     !(j->p_flags & PFROBOT) &&
	     (j->p_team != NOBODY) &&
	     (pcount[j->p_team] < tournplayers)) {

	    c = (int) shipnos[j->p_no];
	 }
      }
      if (c > -1 && bc < 2) {
	 start_bird(0, (char) c);
      }
   }
#endif

   for (t = 0; t <= MAXTEAM; t++) {	/* maint: was "<" 6/22/92 TC */
      /* "suspend" countdown if less than Tmode players */
      if ((teams[t].s_surrender == 0) || (pcount[t] < tournplayers))
	 continue;

      /* also if team has more than SURRSTART planets */
      if (teams[t].s_plcount > SURRSTART)
	 continue;

      teams[t].s_surrender--;
      if (teams[t].s_surrender == 0) {

	 /* terminate race t */
	 sprintf(buf, "The %s %s surrendered.", teamNames[t],
		 teamVerbage[t]);
	 pmessage("", 0, MALL, "");
	 pmessage(buf, 0, MALL, "GOD->ALL");
	 pmessage("", 0, MALL, "");
	 surrenderMessage(t);

	 /* neutralize race t's planets  */

	 for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++) {
	    if (((l->pl_flags & ALLTEAM) == t) && (l->pl_flags & PLHOME))
	       l->pl_couptime = MIN_COUP_TIME +
		  (random() % (MAX_COUP_TIME - MIN_COUP_TIME));

	    /* removed above dependency on PLANETFUSE and hardcoded */
	    /* constants (now in defs.h).  30-60 min for a coup. */
	    /* 4/15/92 TC */

	    if (l->pl_owner == t) {
	       l->pl_owner = 0;
	       l->pl_armies = 0;
	    }
	 }

	 win = NULL;	/* shut up compiler */
	 /* kick out all the players on that team */
	 /* makes it easy for them to come in as a different team */
	 for (i = 0, j = &players[0]; i < MAXPLAYER; i++, j++) {
	    if (j->p_team == t) {
	       j->p_genoplanets = 0;
	       j->p_genoarmsbomb = 0;
	       j->p_armsbomb = 0;
	       j->p_planets = 0;
	       if (j->p_status == PALIVE){
		  j->p_status = PEXPLODE;
		  if (j->p_ship.s_type == STARBASE)
		     j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
		  else
		     j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
		  j->p_whydead = KPROVIDENCE;
		  j->p_whodead = 0;
	       }
	    }
	    else if(j->p_status == PALIVE)
	       win = j;	/* arbitrary */
	 }			/* end for */

	 checkgen(t, win);

      }
      /* end if */
      else {			/* surrender countdown */
	 if ((teams[t].s_surrender % 5) == 0) {
	    sprintf(buf, "The %s %s %d minutes remaining.",
		    teamNames[t], teamVerbage[t],
		    teams[t].s_surrender);
	    pmessage("", 0, MALL, "");
	    pmessage(buf, 0, MALL, "GOD->ALL");

	    sprintf(buf, "%d planets will sustain the empire.  %d suspends countdown.",
		    SURREND, SURRSTART + 1);
	    pmessage(buf, 0, MALL, "GOD->ALL");
	    pmessage("", 0, MALL, "");
	 }
      }
   }				/* end for team */
}

void
udphaser()
{
   register int    i;
   register struct phaser *j;
   register struct player *victim;
#ifdef BASEPRACTICE
   double		  d;
#endif

   for (i = 0, j = &phasers[i]; i < MAXPLAYER; i++, j++) {
      switch (j->ph_status) {
      case PHFREE:
	 continue;
      case PHMISS:
      case PHHIT2:
	 if (j->ph_fuse-- <= 1)		/* just to be safe -- TSH */
	    j->ph_status = PHFREE;
	 break;
      case PHHIT:
	 if (j->ph_fuse-- == players[i].p_ship.s_phaserfuse) {
	    victim = &players[j->ph_target];
	    /* start a war, if necessary */
	    if (players[i].p_hostile & victim->p_team) {
	       players[i].p_swar |= victim->p_team;
	    }
	    if (victim->p_status == PALIVE) {
#ifdef BASEPRACTICE
	       if(isSB((&players[i])) && (victim->p_flags & PFBPROBOT)){
		  d=def_factor(victim, j->ph_damage, &players[i]);
		  players[i].p_stats.st_dami += d; 
		  status->dami += d;
	       }
#ifdef nodef
	       if(isSB(victim) && (players[i].p_flags & PFBPROBOT)){
		  d=off_factor(&players[i], j->ph_damage, victim);
		  victim->p_stats.st_damr += d; 
		  status->damr += d;
	       }
#endif
#endif
	       if (victim->p_flags & PFSHIELD) {
		  victim->p_shield -= j->ph_damage;
		  if (victim->p_shield < 0) {
		     victim->p_damage -= victim->p_shield;
		     victim->p_shield = 0;
		  }
	       } else {
		  victim->p_damage += j->ph_damage;
	       }
	       if (victim->p_damage >= victim->p_ship.s_maxdamage) {
		  victim->p_status = PEXPLODE;
		  if (victim->p_ship.s_type == STARBASE)
		     victim->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
		  else
		     victim->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;

		  players[i].p_kills += 1.0 +
		     victim->p_armies * 0.1 +
		     victim->p_kills * 0.1;
		  killerstats(i, i, victim);
		  checkmaxkills(i);
		  killmess(victim, &players[i], &players[i], KPHASER);
		  loserstats(j->ph_target);
		  victim->p_whydead = KPHASER;
		  victim->p_whodead = i;
	       }
	    }
	 }
	 if (j->ph_fuse <= 0)		/* just to be safe -- TSH */
	    j->ph_status = PHFREE;
	 break;
      }
   }
}

int             pl_warning[MAXPLANETS];	/* To keep planets shut up for awhile */
int             tm_robots[MAXTEAM + 1];	/* To limit the number of robots */
int             tm_coup[MAXTEAM + 1];	/* To allow a coup */

void
teamtimers()
{
   register int    i;
   for (i = 0; i <= MAXTEAM; i++) {
      if (tm_robots[i] > 0)
	 tm_robots[i]--;
      if (tm_coup[i] > 0)
	 tm_coup[i]--;
   }
}

void
plfight()
{
   register int    h, i;
   register struct player *j;
   register struct planet *l;
   register struct player *pl2;
   int             seen;
   int             dx, dy;
   int             damage;
   unsigned
   int             dist;
   int             rnd;
   char            buf[90];
   char            buf1[80];
   char            addrbuf[10];

   for (h = 0, l = &planets[h]; h < MAXPLANETS; h++, l++) {
      if (l->pl_flags & PLCOUP) {
	 l->pl_flags &= ~PLCOUP;
	 sprintf(addrbuf, "%-3s->%s ", l->pl_name, teamshort[l->pl_owner]);
	 pmessage("Planet lost in political struggle",
		  l->pl_owner, MTEAM | MCOUP1, addrbuf);
	 l->pl_owner = (l->pl_flags & ALLTEAM);
	 l->pl_armies = 4;
	 sprintf(addrbuf, "%-3s->%s ", l->pl_name, teamshort[l->pl_owner]);
	 pmessage("Previous dictatorship overthrown in coup d'etat",
		  l->pl_owner, MTEAM | MCOUP2, addrbuf);
      }
      l->pl_flags &= ~PLREDRAW;
      if (pl_warning[h] > 0)
	 pl_warning[h]--;
   }
   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (j->p_status == PALIVE) {

	 seen = 0;
	 for (h = 0, pl2 = &players[0]; h < MAXPLAYER; h++, pl2++) {
	    if (pl2->p_status == PALIVE) {
	       if (pl2->p_flags & PFROBOT)
		  continue;
	       if (pl2->p_team != j->p_team) {
		  dx = ABS(pl2->p_x - j->p_x);
		  dy = ABS(pl2->p_y - j->p_y);
		  if (dx > GWIDTH / 3 || dy > GWIDTH / 3)
		     continue;
		  /*
		   * experiments show that hypot() is a good deal slower
		   * (20x) then two multiplications.
		   */
		  /*
		   * NOTE: If the const expressions are not grouped properly
		   * they may overflow a 32 bit integer.
		   */
		  dist = dx * dx + dy * dy;
		  if ((dist < ((GWIDTH / 3) * (GWIDTH / 3))	/* -O should calc int
								 * result */
		       &&((j->p_flags & PFCLOAK) == 0))
		      || (dist < ((GWIDTH / 7) * (GWIDTH / 7)))	/* "    */
		      ||((j->p_flags & PFORBIT) &&
			 (planets[j->p_planet].pl_owner != j->p_team))) {
		     seen = 1;
		     break;
		  }
	       }
	    }
	 }
	 if (seen) {
	    j->p_flags |= PFSEEN;
	 } else {
	    j->p_flags &= ~PFSEEN;
	 }
	 /*
	  * Have planets do damage to nearby players.  Note that this is the
	  * most inefficient code in the game.  If one needs cycles, simply
	  * change the code to have planets only attack ships that are in
	  * orbit.   Thus you get rid of the following for statement
	  */
	 for (h = 0, l = &planets[h]; h < MAXPLANETS; h++, l++) {
	    dx = ABS(l->pl_x - j->p_x);
	    dy = ABS(l->pl_y - j->p_y);
	    if (dx < 3 * PFIREDIST && dy < 3 * PFIREDIST)
	       l->pl_flags |= PLREDRAW;
	    if (dx > PFIREDIST || dy > PFIREDIST)	/* XXX */
	       continue;

	    /*
	     * experiments show that hypot() is a good deal slower (20x) then
	     * two multiplications.
	     */
	    if (dx * dx + dy * dy > PFIREDIST * PFIREDIST)
	       continue;

	    if ((j->p_swar | j->p_hostile) & l->pl_owner) {
	       if (l->pl_armies > 0)
		  damage = l->pl_armies / 10 + 2;
	       else
		  damage = 0;
	       if (damage > 0) {
		  if (j->p_flags & PFSHIELD) {
		     j->p_shield -= damage;
		     if (j->p_shield < 0) {
			j->p_damage -= j->p_shield;
			j->p_shield = 0;
		     }
		  } else {
		     j->p_damage += damage;
		  }
		  if (j->p_damage >= j->p_ship.s_maxdamage) {
		     if (j->p_ship.s_type == STARBASE)
			j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
		     else
			j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
		     j->p_status = PEXPLODE;
		     loserstats(i);
		     (void) sprintf(buf, "%s (%c%c) killed by %s (%c)",
				    j->p_name,
				    teamlet[j->p_team],
				    shipnos[j->p_no],
				    l->pl_name,
				    teamlet[l->pl_owner]);
#ifdef SHORT_PACKETS
		     m_args[0] = DMKILLP;
		     m_args[1] = j->p_no;
		     m_args[2] = l->pl_no;
		     m_args[3] = j->p_armies;
#endif
		     pmessage(buf, 0, MALL | MKILLP, "GOD->ALL");
		     j->p_whydead = KPLANET;
		     j->p_whodead = h;
		  }
	       }
	    }
	 }			/* End planet damage */

	 /* do bombing */
	 if ((!(j->p_flags & PFORBIT)) || (!(j->p_flags & PFBOMB)))
	    continue;
	 l = &planets[j->p_planet];

	 if (j->p_team == l->pl_owner)
	    continue;
	 if (!((j->p_swar | j->p_hostile) & l->pl_owner))
	    continue;
	 if (l->pl_armies < 5)
	    continue;

	 if (!(status->tourn) && (notmodebomb == 0)) {
	    strcpy(buf, "You may not bomb when not in tmode.");
	    sprintf(buf1, "GOD->%c%c ", teamlet[j->p_team], shipnos[j->p_no]);
	    pmessage2(buf, j->p_no, MINDIV, buf1, 255);
	    continue;
	 }
	 /* Warn owning team */
	 if (pl_warning[j->p_planet] <= 0) {
	    pl_warning[j->p_planet] = 50 / PLFIGHTFUSE;
	    (void) sprintf(buf, "We are being attacked by %s %c%c who is %d%% damaged.",
			   j->p_name,
			   teamlet[j->p_team],
			   shipnos[j->p_no],
			   (100 * j->p_damage) / (j->p_ship.s_maxdamage));
	    (void) sprintf(buf1, "%-3s->%-3s",
			   l->pl_name, teamshort[l->pl_owner]);
#ifdef SHORT_PACKETS
	    m_args[0] = DMBOMB;
	    m_args[1] = j->p_no;
	    m_args[2] = (100*j->p_damage)/(j->p_ship.s_maxdamage);
	    m_args[3] = l->pl_no;
#endif
	    pmessage(buf, l->pl_owner, MTEAM | MBOMB,buf1);
	 }
	 /* start the war (if necessary) */
	 j->p_swar |= l->pl_owner;

	 rnd = random() % 100;
	 if (rnd < 50) {
	    continue;
	 } else if (rnd < 80) {
	    l->pl_armies -= 1;
	    if (l->pl_armies < 5)
	       l->pl_flags |= PLREDRAW;
	    j->p_kills += 0.02;
	    j->p_armsbomb += 1;
	    j->p_genoarmsbomb += 1;
	    checkmaxkills(i);
	    /* Give him bombing stats if he is bombing a team with 3+ */
	    if (status->tourn && pcount[l->pl_owner] >= 3) {
	       status->armsbomb++;
	       j->p_stats.st_tarmsbomb++;
	    } else {
	       j->p_stats.st_armsbomb++;
	    }
	 } else if (rnd < 90) {
	    l->pl_armies -= 2;
	    if (l->pl_armies < 5)
	       l->pl_flags |= PLREDRAW;
	    j->p_kills += 0.04;
	    j->p_armsbomb += 2;
	    j->p_genoarmsbomb += 2;
	    checkmaxkills(i);
	    if (status->tourn && pcount[l->pl_owner] >= 3) {
	       status->armsbomb += 2;
	       j->p_stats.st_tarmsbomb += 2;
	    } else {
	       j->p_stats.st_armsbomb += 2;
	    }
	 } else {
	    l->pl_armies -= 3;
	    if (l->pl_armies < 5)
	       l->pl_flags |= PLREDRAW;
	    j->p_kills += 0.06;
	    j->p_armsbomb += 3;
	    j->p_genoarmsbomb += 3;
	    checkmaxkills(i);
	    if (status->tourn && pcount[l->pl_owner] >= 3) {
	       status->armsbomb += 3;
	       j->p_stats.st_tarmsbomb += 3;
	    } else {
	       j->p_stats.st_armsbomb += 3;
	    }
	 }

	 if ((j->p_ship.s_type == ASSAULT) && (rnd > 20)) {
	    l->pl_armies -= 1;
	    if (l->pl_armies < 5)
	       l->pl_flags |= PLREDRAW;
	    j->p_kills += 0.02;
	    j->p_armsbomb += 1;
	    j->p_genoarmsbomb += 1;
	    checkmaxkills(i);
	    if (status->tourn && pcount[l->pl_owner] >= 3) {
	       status->armsbomb += 1;
	       j->p_stats.st_tarmsbomb += 1;
	    } else {
	       j->p_stats.st_armsbomb += 1;
	    }
	 }
	 /*
	  * Send in a robot if there are no other defenders and the planet is
	  * in the team's home space
	  */

	 if ((tcount[l->pl_owner] == 0) &&
	     (l->pl_flags & l->pl_owner) &&
	     tm_robots[l->pl_owner] == 0) {
	    rescue(l->pl_owner, j);
	    /* 15 seconds, timer not essential */
	    tm_robots[l->pl_owner] = 150 / TEAMFUSE;
#if 0
	    tm_robots[l->pl_owner] = (1800 +
				      (random() % 1800)) /
	       TEAMFUSE;
#endif
	 }
      } else {
	 j->p_flags |= PFSEEN;
      }
   }
}

void
beam()
{
   register int    i;
   register struct player *j;
   register struct planet *l = NULL;
   char            buf[80];
   char            buf1[80];

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if ((j->p_status != PALIVE) || !(j->p_flags & (PFORBIT | PFDOCK)))
	 continue;
      if (j->p_flags & PFORBIT)
	 l = &planets[j->p_planet];
      if (j->p_flags & PFBEAMUP) {
	 if (j->p_flags & PFORBIT)
	    if (l->pl_armies < 5)
	       continue;
	 if (j->p_flags & PFDOCK)
	    if (players[j->p_docked].p_armies == 0)
	       continue;
	 if (j->p_armies == j->p_ship.s_maxarmies)
	    continue;
	 /* XXX */
	 if (j->p_ship.s_type == ASSAULT) {
	    if (j->p_armies == (int) (j->p_kills * 3.0))
	       continue;
	 } else if (j->p_ship.s_type != STARBASE)
	    if (j->p_armies == (int) (j->p_kills * 2.0))
	       continue;
	 if (j->p_flags & PFORBIT) {
	    if (j->p_team != l->pl_owner)
	       continue;
	    if ((j->p_swar | j->p_hostile) & l->pl_owner)
	       continue;
	 }
	 j->p_armies++;
	 if (j->p_flags & PFORBIT)
	    l->pl_armies--;
	 else if (j->p_flags & PFDOCK)
	    players[j->p_docked].p_armies--;
	 continue;
      }
      if (j->p_flags & PFBEAMDOWN) {
	 if (j->p_armies == 0)
	    continue;
	 if (j->p_flags & PFORBIT) {
	    if ((!((j->p_swar | j->p_hostile) & l->pl_owner))
		&& (j->p_team != l->pl_owner) && (l->pl_owner != NOBODY))
	       continue;
	    if (j->p_team != l->pl_owner) {
	       j->p_armies--;
	       if (l->pl_armies) {	/* If there are armies on the planet */
		  int             oldowner = l->pl_owner;

		  /* start the war (if necessary) */
		  j->p_swar |= l->pl_owner;

		  l->pl_armies--;
		  j->p_kills += 0.02;
		  j->p_armsbomb += 1;
		  j->p_genoarmsbomb += 1;
		  checkmaxkills(i);
		  if (status->tourn && pcount[oldowner] >= 3) {
		     j->p_stats.st_tarmsbomb++;
		     status->armsbomb++;
		  } else {
		     j->p_stats.st_armsbomb++;
		  }
		  if (l->pl_armies == 0) {	/* planet is indep now */
		     /* Give planet to nobody */
		     (void) sprintf(buf, "%s destroyed by %s (%c%c)",
				    l->pl_name,
				    j->p_name,
				    teamlet[j->p_team],
				    shipnos[j->p_no]);
		     (void) sprintf(buf1, "%-3s->%-3s",
				    l->pl_name, teamshort[l->pl_owner]);
#ifdef SHORT_PACKETS
		     m_args[0] = DMDEST;
		     m_args[1] = l->pl_no;
		     m_args[2] = j->p_no;
#endif
		     pmessage(buf, l->pl_owner, MTEAM | MDEST ,buf1);
		     if (status->tourn &&
			 pcount[l->pl_owner] < 2 &&
			 pcount[l->pl_flags &
				      (FED | ROM | ORI | KLI)] < 2) {
			l->pl_flags |= PLCHEAP;
			/* Next person to take this planet will */
			/* get limited credit */
		     }
		     l->pl_owner = NOBODY;
#ifdef ORGANIAN
		     if (organian <= 1)
#endif
			checkgen(oldowner, j);
		  }
	       } else {		/* planet taken over */
		  l->pl_armies++;
		  j->p_planets++;
		  j->p_genoplanets++;
		  if (status->tourn && !(l->pl_flags & PLCHEAP)) {
		     j->p_stats.st_tplanets++;
		     status->planets++;
		     if ((l->pl_flags & PLCORE) &&
			 !(j->p_team & l->pl_flags) &&
			 (pcount[ALLTEAM & l->pl_flags] > 2)) {
			j->p_stats.st_tplanets++;
			status->planets++;
		     }
		  } else {
		     j->p_stats.st_planets++;
		  }
		  l->pl_flags &= ~PLCHEAP;

		  j->p_kills += 0.25;
		  checkmaxkills(i);
		  (void) sprintf(buf, "%s taken over by %s (%c%c)",
				 l->pl_name,
				 j->p_name,
				 teamlet[j->p_team],
				 shipnos[j->p_no]);
		  l->pl_owner = j->p_team;
		  l->pl_info = j->p_team;
#ifdef ORGANIAN
		  if (!organian)
#endif
		     checkwin(j);
		  /* now tell new team */
		  (void) sprintf(buf1, "%-3s->%-3s",
				 l->pl_name, teamshort[l->pl_owner]);
#ifdef SHORT_PACKETS
		  m_args[0] = DMTAKE;
		  m_args[1] = l->pl_no;
		  m_args[2] = j->p_no;
#endif
		  pmessage(buf, l->pl_owner, MTEAM | MTAKE, buf1);
	       }
	    } else {		/* bombing */
	       if (status->tourn && l->pl_armies < 4 &&
		   me->p_ship.s_type != STARBASE) {
		  j->p_stats.st_tarmsbomb += (4 - l->pl_armies);
		  status->armsbomb += (4 - l->pl_armies);
	       }
	       j->p_armies--;
	       l->pl_armies++;
	    }
	 } else if (j->p_flags & PFDOCK) {
	    if (players[j->p_docked].p_team != j->p_team)
	       continue;
	    if (players[j->p_docked].p_armies == players[j->p_docked].p_ship.s_maxarmies) {
	       continue;
	    } else {
	       j->p_armies--;
	       players[j->p_docked].p_armies++;
	    }
	 }
      }
   }
}

void
blowup(sh)
   struct player  *sh;
{
   register int    i;
   int             dx, dy, dist;
   int             damage;
   register struct player *j;
#if defined(CHAIN_REACTION) || defined(BASEPRACTICE)
   register struct player *killer;
#endif
#ifdef BASEPRACTICE
   double	   	  d;
#endif
   
   if(sh->p_whydead == KGHOST) return;

   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
      if (j->p_status != PALIVE)
	 continue;
      if (sh == j)
	 continue;
      /* the following keeps people from blowing up on others */
      me = sh;
      if ((me->p_whydead == KQUIT) && (friendlyPlayer(j)))
	 continue;
      dx = sh->p_x - j->p_x;
      dy = sh->p_y - j->p_y;
      if (ABS(dx) > SHIPDAMDIST || ABS(dy) > SHIPDAMDIST)
	 continue;
      dist = dx * dx + dy * dy;
      if (dist > SHIPDAMDIST * SHIPDAMDIST)
	 continue;
      if (dist > EXPDIST * EXPDIST) {
	 if (sh->p_ship.s_type == STARBASE){
	    damage = (int)(200 * (SHIPDAMDIST - sqrt((double) dist)) /
	       (SHIPDAMDIST - EXPDIST));
	    MATH_ERR_CHECK(dist);
	 }
	 else if (sh->p_ship.s_type == SCOUT){
	    damage = (int)(75 * (SHIPDAMDIST - sqrt((double) dist)) /
	       (SHIPDAMDIST - EXPDIST));
	    MATH_ERR_CHECK(dist);
	 }
	 else{
	    damage = (int)(100 * (SHIPDAMDIST - sqrt((double) dist)) /
	       (SHIPDAMDIST - EXPDIST));
	    MATH_ERR_CHECK(dist);
	 }
      } else {
	 if (sh->p_ship.s_type == STARBASE)
	    damage = 200;
	 else if (sh->p_ship.s_type == SCOUT)
	    damage = 75;
	 else
	    damage = 100;
      }
      if (damage > 0) {
#ifdef BASEPRACTICE
	 if ((j->p_no != sh->p_whodead) && (j->p_team != 
	     players[sh->p_whodead].p_team) && 
	     (sh->p_whydead == KSHIP || sh->p_whydead == KTORP ||
		 sh->p_whydead == KPHASER || sh->p_whydead == KPLASMA))
	    killer = &players[sh->p_whodead];
	 else
	    killer = sh;
	 if(isSB(killer) && (j->p_flags & PFBPROBOT)){
	    d=def_factor(j, damage, killer);
	    killer->p_stats.st_dami += d; 
	    status->dami += d;
	 }
#ifdef nodef
	 if(isSB(j) && (killer->p_flags & PFBPROBOT)){
	    d=off_factor(killer, damage, j);
	    j->p_stats.st_damr += d; 
	    status->damr += d;
	 }
#endif
#endif
	 if (j->p_flags & PFSHIELD) {
	    j->p_shield -= damage;
	    if (j->p_shield < 0) {
	       j->p_damage -= j->p_shield;
	       j->p_shield = 0;
	    }
	 } else {
	    j->p_damage += damage;
	 }
	 if (j->p_damage >= j->p_ship.s_maxdamage) {
	    j->p_status = PEXPLODE;
	    if (j->p_ship.s_type == STARBASE)
	       j->p_explode = 2 * SBEXPVIEWS / UPDATE_NORM;
	    else
	       j->p_explode = 10 / UPDATE_NORM;
#if defined(CHAIN_REACTION) && !defined(BASEPRACTICE)
	    if ((j->p_no != sh->p_whodead) && (j->p_team != 
		players[sh->p_whodead].p_team) && 
		(sh->p_whydead == KSHIP || sh->p_whydead == KTORP ||
		    sh->p_whydead == KPHASER || sh->p_whydead == KPLASMA))
	       killer = &players[sh->p_whodead];
	    else
	       killer = sh;
#endif
#ifdef CHAIN_REACTION
	    if ((killer->p_team != j->p_team) ||
	      ((killer->p_team == j->p_team) && (j->p_flags & PFPRACTR))) {
	       killer->p_kills += 1.0 +
		  j->p_armies * 0.1 + j->p_kills * 0.1;
	       checkmaxkills(killer->p_no);
	    }

	    killerstats(killer->p_no, sh->p_no, j);
	    loserstats(i);
	    j->p_whydead = KSHIP;
	    j->p_whodead = killer->p_no;

	    killmess(j, killer, sh, 
		     (killer->p_no == sh->p_no? KSHIP : KSHIP2));
#else

	    sh->p_kills += 1.0 +
	       j->p_armies * 0.1 + j->p_kills * 0.1;
	    killerstats(sh->p_no, sh->p_no, j);
	    checkmaxkills(sh->p_no);
	    loserstats(i);
	    j->p_whydead = KSHIP;
	    j->p_whodead = sh->p_no;
	    killmess(j, sh, sh, KSHIP);
#endif				/* CHAIN REACTION */
	 }
      }
   }
}

void
freemem(sig, code, scp, addr)
   int             sig, code;
   struct sigcontext *scp;
   char           *addr;
{
   register int    i;
   register struct player *j;

   if(sig){
      fprintf(stderr, "got signal %d\n", sig);
      fflush(stderr);
   }

#ifndef SYSV
   if (sig == SIGFPE)
      return;			/* Too many errors occur ... ignore them. */
#endif
   /* otherwise we get infinite loop */

   /* some signals can be ignored */
   if (sig == SIGCHLD || sig == SIGCONT)
      return;

   /* added by TSH */

   (void) signal(SIGALRM, SIG_IGN);	/* ignore timer signal */
   if (sig) {
      fprintf(stderr, "daemonII going down on signal %d\n", sig);
      fflush(stderr);
   }
   /* if problem occured before shared memory setup */
   if (!players)
      exit(0);

   /* Blow players out of the game */
   for (i = 0, j = &players[i]; i < MAXPLAYER; i++, j++) {
#ifdef DS
      if(j->p_process > 1)
	 (void)kill(j->p_process, SIGTERM);
#endif
      j->p_status = PEXPLODE;
      j->p_whydead = KDAEMON;
      j->p_ntorp = 0;
      j->p_nplasmatorp = 0;
      j->p_explode = (600/UPDATE_NORM) / PLAYERFUSE;	/* ghost buster was leaving players
					 * in */
   }
   /* Kill waiting players */
   status->count = 0;
   save_planets();
   my_sleep(2);
   shmctl(shmid, IPC_RMID, (struct shmid_ds *) 0);

   switch(sig){
      case SIGSEGV:
      case SIGBUS:
      case SIGILL:
      case SIGFPE:
      case SIGSYS:
	 (void) signal(SIGABRT, SIG_DFL);
	 abort();	/* get the coredump */
      default:
	 break;
   }

   exit(0);
}

void
save_planets()
{
   if (plfd >= 0) {
      (void) lseek(plfd, (long) 0, 0);
      (void) write(plfd, (char *) planets, sizeof(pdata));
   }
   if (glfd >= 0) {
      (void) lseek(glfd, (long) 0, 0);
      (void) write(glfd, (char *) status, sizeof(struct status));
   }
}

void
check_load()
{
   FILE           *fp,
#ifdef MYPOPEN               
		  *mypopen();
#else				
		  *popen();
#endif	
   char            buf[100];
   char            *s;
   float           load;

   if (gamedown)
      return;

   (void) signal(SIGALRM, SIG_IGN);

#ifdef MYPOPEN
   fp = mypopen(UPTIME, "uptime", "r");
#else
   fp = popen(UPTIME, "r");
#endif
   if (fp == NULL) {
      status->gameup = 0;
      fprintf(stderr, "Game down because of non-existance UPTIME %s\n",
	 UPTIME);
      fflush(stderr);
      (void) signal(SIGALRM, move);
      return;
   }
   fgets(buf, 99, fp);
   s = strrchr(buf, ':');
   if (s == NULL) {
      status->gameup = 0;
      pclose(fp);
      fprintf(stderr, "Game down because of unparseable line from UPTIME\n");
      fprintf(stderr, "%s\n", buf);
      fflush(stderr);
      (void) signal(SIGALRM, move);
      return;
   }
   if (sscanf(s + 1, " %f", &load) == 1) {
      if (load >= maxload && status->gameup == 1) {
	 status->gameup = 0;
	 fprintf(stderr, "Game down because load average %f\n", load);
	 fflush(stderr);
	 sprintf(buf, "The load is %f, this game is going down", load);
	 pmessage(buf, 0, MALL, "GOD->ALL");
      } else if (load < maxload && status->gameup == 0) {
	 status->gameup = 1;
	 sprintf(buf, "The load is %f, this game is coming up", load);
	 pmessage(buf, 0, MALL, "GOD->ALL");
      }
   } else {
      status->gameup = 0;
      fprintf(stderr, 
	 "Game down because of unparseable load average from UPTIME\n");
      fprintf(stderr, "%s\n", buf);
      fflush(stderr);
   }
   pclose(fp);
   (void) signal(SIGALRM, move);
}

int
CountPlanets(who)
   int             who;
{
   register int    i, count = 0;
   register struct planet *l;

   for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++)
      if (l->pl_owner == who)
	 count++;
   return count;
}

/*
 * This function checks to see if a team has been genocided -- their last
 * planet has been beamed down to zero.  It will set a timer on their home
 * planet that will prevent them from couping for a random number of minutes.
 */
void
checkgen(loser, winner)
   int             loser;
   struct player  *winner;
{
   register int    i;
   register struct planet *l;
   struct planet  *homep = NULL;
   struct player  *j;

   for (i = 0; i <= MAXTEAM; i++) /* maint: was "<" 6/22/92 TC */
      teams[i].s_plcount = 0;

   for (i = 0, l = &planets[i]; i < MAXPLANETS; i++, l++) {
      teams[l->pl_owner].s_plcount++; /* for now, recompute here */
/*
      if (l->pl_owner == loser)
	 return;
*/
      if (((l->pl_flags & ALLTEAM) == loser) && (l->pl_flags & PLHOME))
	 homep = l;          /* Keep track of his home planet for marking */
   }

   /* check to initiate surrender */

   if ((teams[loser].s_surrender == 0) &&
       (teams[loser].s_plcount == SURRSTART) &&
       (pcount[loser] >= tournplayers)) {
      char            buf[80];

      sprintf(buf, "The %s %s %d minutes before the empire collapses.",
	      teamNames[loser], teamVerbage[loser],
	      binconfirm ? SURRLENGTH : SURRLENGTH * 2 / 3);
      pmessage("", 0, MALL, "");
      pmessage(buf, 0, MALL, "GOD->ALL");
      sprintf(buf, "%d planets are needed to sustain the empire.",
	      SURREND);
      pmessage(buf, 0, MALL, "GOD->ALL");
      pmessage("", 0, MALL, "");

      teams[loser].s_surrender = binconfirm ?	/* shorter for borgs? */
	 SURRLENGTH : SURRLENGTH * 2 / 3;	/* start the clock 1/27/92 TC */
   }
   if (teams[loser].s_plcount > 0)
      return;

   if(teams[loser].s_surrender){
      /* Give extra credit for neutralizing last planet if team didn't
         surrender */
      winner->p_genoplanets++;
      winner->p_planets++;
      if (status->tourn) {
	 winner->p_stats.st_tplanets++;
      } else {
	 winner->p_stats.st_planets++;
      }
   }
   teams[loser].s_surrender = 0;/* stop the clock */


   checkwin(winner);		/* conquer is higher priority, but genocide
				 * is a necessary condition 3/25/92 TC */


   _winner = winner;
   /* Tell winning team what wonderful people they are (pat on the back!) */
   genocideMessage(loser, winner->p_team);
   /* Give more troops to winning team? */
   addTroops(loser, winner->p_team);

   /* kick out all the players on that team */
   /* makes it easy for them to come in as a different team */
   for (i = 0, j = &players[0]; i < MAXPLAYER; i++, j++) {
      if (j->p_team == loser) {
	 j->p_genoplanets = 0;
	 j->p_genoarmsbomb = 0;
	 j->p_armsbomb = 0;
	 j->p_planets = 0;
      }
      if (j->p_status == PALIVE && j->p_team == loser) {
	 j->p_status = PEXPLODE;
	 if (j->p_ship.s_type == STARBASE)
	    j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
	 else
	    j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
	 j->p_whydead = KGENOCIDE;
	 j->p_whodead = winner->p_no;
      }
   }

   /* well, I guess they are losers.  Hose them now */
   homep->pl_couptime = MIN_COUP_TIME +
      (random() % (MAX_COUP_TIME - MIN_COUP_TIME));

#ifdef ORGANIAN
   if (organian)
      organian_start(winner);
#endif

#ifdef VOTE
   /* set up reset vote */
   vote_resetgalaxy_on();
#endif
}


/*
 * This function is called when a planet has been taken over. It checks all
 * the planets to see if the victory conditions are right.  If so, it blows
 * everyone out of the game and resets the galaxy
 */
void
checkwin(winner)
   struct player  *winner;
{
   register int    i, h;
   register struct planet *l;
   int             team[MAXTEAM + 1];

   int             teamtemp[MAXTEAM + 1];

   if(!winner) winner = &players[0]; 	/* failsafe */

   teams[0].s_plcount = 0;
   for (i = 0; i < 4; i++) {
      team[1 << i] = 0;
      teams[1 << i].s_plcount = 0;
   }

   /* count the number of home systems owned by one team */
   for (i = 0; i < 4; i++) {
      teamtemp[NOBODY] = teamtemp[FED] = teamtemp[ROM] = teamtemp[KLI] =
	 teamtemp[ORI] = 0;

      for (h = 0, l = &planets[i * 10]; h < 10; h++, l++) {
	 teams[l->pl_owner].s_plcount++;
	 teamtemp[l->pl_owner]++;
      }

      /* count neutral planets towards conquer 3/24/92 TC */

      if ((teamtemp[FED] + teamtemp[ROM] + teamtemp[KLI]) == 0)
	 team[ORI]++;
      else if ((teamtemp[FED] + teamtemp[ROM] + teamtemp[ORI]) == 0)
	 team[KLI]++;
      else if ((teamtemp[FED] + teamtemp[KLI] + teamtemp[ORI]) == 0)
	 team[ROM]++;
      else if ((teamtemp[ROM] + teamtemp[KLI] + teamtemp[ORI]) == 0)
	 team[FED]++;
   }

   /* check to end surrender */

   if ((teams[winner->p_team].s_surrender > 0) &&
       (teams[winner->p_team].s_plcount == SURREND)) {
      char            buf[80];
      pmessage("", 0, MALL, "");
      sprintf(buf, "The %s %s prevented collapse of the empire.",
	      teamNames[winner->p_team], teamVerbage[winner->p_team]);
      pmessage(buf, 0, MALL, "GOD->ALL");
      pmessage("", 0, MALL, "");
      teams[winner->p_team].s_surrender = 0;	/* stop the clock */
   }
   for (i = 0; i < 4; i++) {
      if (team[1 << i] >= VICTORY) {
	 _winner = winner;	/* save */
	 resetGalaxy(winner);
      }
   }
}

void
resetGalaxy(winner)

   struct player	*winner;
{
   register int			h;
   register struct player	*j;

   /* We have a winning team */
   conquerMessage(winner->p_team);
   for (h = 0, j = &players[0]; h < MAXPLAYER; h++, j++) {
      if (j->p_status == PFREE)
	 continue;
      j->p_status = PEXPLODE;
      j->p_whydead = KWINNER;
      j->p_whodead = winner->p_no;
      if (j->p_ship.s_type == STARBASE)
	 j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
      else
	 j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
      j->p_ntorp = 0;
      j->p_nplasmatorp = 0;
   }
   doResources();
   longjmp(env, 0);
}

void
pmessage(str, recip, group, address)
   char           *str;
   int             recip;
   int             group;
   char           *address;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = 255;		/* From god */

   copy80(cur->m_data, "%s ", address, str);
   cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
   if(m_args[0] != DINVALID){
      /* only 6 out of 8 used now */
      bcopy(m_args, cur->args, 6 * sizeof(int));
   }
   else if(strstr(address, "GOD->")!=NULL)
      cur->args[0] = SVALID;	/* It has a real header */
   else
      cur->args[0] = SINVALID;	/* marked as invalid to send over */
   m_args[0] = DINVALID;
#endif
}

void
pmessage2(str, recip, group, address, from)
   char           *str;
   int             recip;
   int             group;
   char           *address;
   unsigned char   from;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = from;
   copy80(cur->m_data, "%-9s ", address, str);
   cur->m_flags |= MVALID;
}

void
ghostmess(victim)
   struct player  *victim;
{
   char            buf[128];
   static float    ghostkills = 0.0;
   int             i, k;

   ghostkills += 1.0 + victim->p_armies * 0.1 + victim->p_kills * 0.1;
   (void) sprintf(buf, "%s (%c%c) was kill %0.2f for the GhostBusters",
	     victim->p_name, teamlet[victim->p_team], shipnos[victim->p_no],
		  ghostkills);
#ifdef SHORT_PACKETS
   m_args[0] = DGHOSTKILL;
   m_args[1] = victim->p_no;
   m_args[2] = 100 * ghostkills;
#endif
   pmessage(buf, 0, MALL | MGHOST, "GOD->ALL");
   if (victim->p_armies > 0) {
      k = 10 * (remap[victim->p_team] - 1);
      if (k >= 0 && k <= 30)
	 for (i = 0; i < 10; i++) {
	    if (planets[i + k].pl_owner == victim->p_team) {
	       planets[i + k].pl_armies += victim->p_armies;
	       (void) sprintf(buf, "%s's %d armies placed on %s",
		  victim->p_name, victim->p_armies, planets[k + i].pl_name);
	       pmessage(buf, 0, MALL | MGHOST, "GOD->ALL");
	       break;
	    }
	 }
   }
}

/* straight out of X11/Intrinsic.h */
#if defined(CRAY) || defined(__arm)
#ifdef CRAY2

#define _Offset(p_type,field) \
        (sizeof(int)*((unsigned int)&(((p_type)NULL)->field)))

#else   /* !CRAY2 */

#define _Offset(p_type,field) ((unsigned int)&(((p_type)NULL)->field))

#endif  /* !CRAY2 */
#else   /* ! (CRAY || __arm) */

#define _Offset(p_type,field) \
        ((unsigned int) (((char *) (&(((p_type)NULL)->field))) - ((char *) NULL)))

#endif /* !CRAY */

void
saveplayer(victim)
   struct player  *victim;
{
   int             fd;

   if (victim->p_pos < 0)
      return;
   if (victim->p_stats.st_lastlogin == 0)
      return;
#if 0
   if (victim->p_flags & PFROBOT)
      return;
#endif

   fd = open(PLAYERFILE, O_WRONLY, 0644);
   if (fd >= 0) {
      if(lseek(fd, 32 + victim->p_pos * sizeof(struct statentry), 0) < 0){
	 perror("daemonII'lseek");
	 close(fd);
	 return;
      }
      write(fd, (char *) &victim->p_stats, sizeof(struct stats));
      close(fd);
   }
}

/* isae - death info */
static char    *killmsgs[] = {"", "", "[photon]", "[phaser]", "", "[explosion]",
"", "", "", "", "", "[plasma]", "[detted photon]", "[chain explosion]"};

void
killmess(victim, credit_killer, killer, whydead)
   struct player  *victim, *killer, *credit_killer;
   int             whydead;
{
   char            buf[256];
   char		   *why = NULL;
   char		   kills[20];
#ifdef SHORT_PACKETS
   int		   reason;
#endif
   
   if(credit_killer->p_team == killer->p_team)
      credit_killer = killer;
   
   sprintf(kills, "%0.2f", killer->p_kills);

#ifdef SHORT_PACKETS
   reason = whydead;
#endif
   why = killmsgs[whydead];

   if ((victim->p_team == killer->p_team) && !(victim->p_flags & PFPRACTR)) {
      strcpy(kills, "NO CREDIT");
   }
   if(victim->p_armies == 0){
      (void) sprintf(buf, "%s (%c%c) was kill %s for %s (%c%c)",
		     victim->p_name,
		     teamlet[victim->p_team],
		     shipnos[victim->p_no],
		     kills,
		     killer->p_name,
		     teamlet[killer->p_team],
		     shipnos[killer->p_no]);

#ifdef SHORT_PACKETS
      m_args[0] = DMKILL;	/* or INLDMKILL */
      m_args[1] = victim->p_no;
      m_args[2] = killer->p_no;
      m_args[3] = 100 * killer->p_kills;
      m_args[4] = victim->p_armies;
      m_args[5] = reason;
#endif
      sprintf(buf, "%-50s %s", buf, why);
      pmessage(buf, 0, MALL | MKILL, "GOD->ALL");

      if(killer != credit_killer){
	 (void) sprintf(buf, "Credit for %c%c awarded to %s (%c%c), kill %0.2f",
			      teamlet[victim->p_team],
			      shipnos[victim->p_no],
			      credit_killer->p_name,
			      teamlet[credit_killer->p_team],
			      shipnos[credit_killer->p_no],
			      credit_killer->p_kills);
	 pmessage(buf, 0, MALL | MKILL, "GOD->ALL");
      }
   } else {
      (void) sprintf(buf, "%s (%c%c+%d arm%s) was kill %s for %s (%c%c)",
			  victim->p_name,
			  teamlet[victim->p_team],
			  shipnos[victim->p_no],
			  victim->p_armies,
			  victim->p_armies > 1?"ies":"y",
			  kills,
			  killer->p_name,
			  teamlet[killer->p_team],
			  shipnos[killer->p_no]);
#ifdef SHORT_PACKETS
      m_args[0] = DMKILL;	/* or INLDMKILL */
      m_args[1] = victim->p_no;
      m_args[2] = killer->p_no;
      m_args[3] = 100 * killer->p_kills;
      m_args[4] = victim->p_armies;
      m_args[5] = reason;
#endif
      sprintf(buf, "%-50s %s", buf, why);

      pmessage(buf, 0, MALL | MKILLA, "GOD->ALL");

      if(killer != credit_killer){
	 (void) sprintf(buf, "Credit for %c%c+%d arm%s awarded to %s (%c%c), kill %0.2f",
			     teamlet[victim->p_team],
			     shipnos[victim->p_no],
			     victim->p_armies,
			     victim->p_armies > 1?"ies":"y",
			     credit_killer->p_name,
			     teamlet[credit_killer->p_team],
			     shipnos[credit_killer->p_no],
			     credit_killer->p_kills);
	 pmessage(buf, 0, MALL | MKILLA, "GOD->ALL");
      }
   }
}

#ifndef USE_EROBOT	/* erobot.c */
/* Send in a robot to avenge the aggrieved team */
void
rescue(team, p)
   int             	team;
   struct player	*p;
{
   char           *arg1 = NULL;

   if (fork() == 0) {
      (void) close(0);
      (void) close(1);
      (void) close(2);
      (void) signal(SIGALRM, SIG_DFL);
      switch (team) {
      case FED:
	 arg1 = "-Tf";
	 break;
      case ROM:
	 arg1 = "-Tr";
	 break;
      case KLI:
	 arg1 = "-Tk";
	 break;
      case ORI:
	 arg1 = "-To";
	 break;
      }
      execl(ROBOT, "robot", arg1, 0);
      /* If we get here, we are hosed anyway */
      _exit(1);
   }
}
#endif

/* Don't fear the ... */
void
reaper(sig)
   int sig;
{
   while (wait3(NULL, WNOHANG, NULL) > 0);
}

unsigned char
getcourse(x, y, xme, yme)
   int             x, y, xme, yme;
{
   unsigned char	d;
   d = ((unsigned char) (atan2((double) (x - xme),
				  (double) (yme - y)) / 3.14159 * 128.));
   MATH_ERR_CHECK(x-xme+yme-y);
   return d;
}


unsigned char
get_bearing(xme, yme, x, y, dir)
   int		   xme, yme, dir;
   int             x, y;
{
   int             phi = 0;

   phi = ((int) (atan2((double) (x - xme), (double) (yme - y)) / 3.14159 * 128.));
   MATH_ERR_CHECK(x-xme+yme-y);
   if (phi < 0)
      phi = 256 + phi;
   if (phi >= dir)
      return ((unsigned char) (phi - dir));
   else
      return ((unsigned char) (256 + phi - dir));
}

/*
 * Find nearest hostile vessel and indicate if it is on the right or the left
 * of the current heading.  A return value of -1 means left and +1 means
 * right.
 */
int
torp_track_target(torp)
   struct torp    *torp;
{
   int             i, closest = -1, min_dist = GWIDTH * 2, dist = 0, 
		   war_mask, x, y, owner;
   unsigned char   bearing = 0, dir = 0;

   x = torp->t_x;
   y = torp->t_y;
   dir = torp->t_dir;
   owner = torp->t_owner;
   war_mask = torp->t_war;
   for (i = 0; i < MAXPLAYER; i++) {
      if ((isAlive(&players[i])) && (owner != i)) {
	 if (((war_mask & players[i].p_team) ||
			     (players[owner].p_team & (players[i].p_hostile | players[i].p_swar)))) {
	    bearing = get_bearing(x, y, players[i].p_x, players[i].p_y, dir);
	    /*
	     * An approximation to prevent the torpedo from tracking
	     * unreasonable targets: If target is in an arc that the torp
	     * might reach, then it is valid.
	     */
	    /*
	     * fprintf(stderr, "Target %d bearing [%d]:  heading %d,  fuse
	     * %d\n", i, (int) bearing, (int) dir, torp->t_fuse);
	     */
	    if ((torp->t_turns * torp->t_fuse > 127) ||
		(bearing < ((unsigned char) torp->t_turns * torp->t_fuse)) ||
		(bearing > ((unsigned char) (256 - torp->t_turns * torp->t_fuse)))) {
	       dist = ((int) hypot((double) (x - players[i].p_x),
				   (double) (y - players[i].p_y)));
	       if (dist < min_dist) {
		  min_dist = dist;
		  closest = i;
	       }
	    }
	 }
      }
   }
   if (closest >= 0) {
      if (get_bearing(x, y, players[closest].p_x, players[closest].p_y, dir) > 128)
	 return (-1);		/* Target is on the left. */
      else
	 return (1);		/* Target is on the right. */
   }
   return (0);			/* No target ... go straight. */
}				/* torp_track_target()
				 * -------------------------------------------
				 * ------ */

/*
 * Find nearest hostile vessel and indicate if it is on the right or the left
 * of the current heading.  A return value of -1 means left and +1 means
 * right.
 */
int
plasma_track_target(torp)
   struct plasmatorp *torp;
{
   int             i, closest = -1, min_dist = GWIDTH * 2, dist = 0, 
		   war_mask, x, y, owner;
   unsigned char   bearing = 0, dir = 0;

   x = torp->pt_x;
   y = torp->pt_y;
   dir = torp->pt_dir;
   owner = torp->pt_owner;
   war_mask = torp->pt_war;
   for (i = 0; i < MAXPLAYER; i++) {
      if ((isAlive(&players[i])) && (owner != i)) {
	 if (((war_mask & players[i].p_team) ||
			     (players[owner].p_team & (players[i].p_hostile | players[i].p_swar)))) {

	    bearing = get_bearing(x, y, players[i].p_x, players[i].p_y, dir);
	    /*
	     * An approximation to prevent the torpedo from tracking
	     * unreasonable targets: If target is in an arc that the torp
	     * might reach, then it is valid.
	     */
	    /*
	     * fprintf(stderr, "Target %d bearing [%d]:  heading %d,  fuse
	     * %d\n", i, (int) bearing, (int) dir, torp->pt_fuse);
	     */
	    if ((torp->pt_turns * torp->pt_fuse > 127) ||
	     (bearing < ((unsigned char) torp->pt_turns * torp->pt_fuse)) ||
		(bearing > ((unsigned char) (256 - torp->pt_turns * torp->pt_fuse)))) {
	       dist = ((int) hypot((double) (x - players[i].p_x),
				   (double) (y - players[i].p_y)));
	       if (dist < min_dist) {
		  min_dist = dist;
		  closest = i;
	       }
	    }
	 }
      }
   }

   if (closest >= 0) {
      if (get_bearing(x, y, players[closest].p_x, players[closest].p_y, dir) > 128)
	 return (-1);		/* Target is on the left. */
      else
	 return (1);		/* Target is on the right. */
   }
   return (0);			/* No target ... go straight. */
}				/* plasma_track_target()
				 * -------------------------------------------
				 * ------ */


/* Give more troops to winning team in case they need them. */
void
addTroops(loser, winner)
   int             loser, winner;
{
   int             count = 0;
   int             i;
   char            addrbuf[10];

   for (i = 0; i < MAXPLAYER; i++) {
      if (players[i].p_status != PFREE && players[i].p_team == loser)
	 count++;
   }
   /* Never give armies if 0-2 defenders */
   if (count < 3)
      return;
   /* Sometimes give armies if 3-5 defenders */
   if (count < 6 && (random() % count) == 0)
      return;
   /* Always give armies if 6+ defenders */
   sprintf(addrbuf, "GOD->%s ", teamshort[winner]);
   pmessage("The recent victory has boosted enrollment!", winner, MTEAM, addrbuf);
   for (i = 0; i < MAXPLANETS; i++) {
      if (planets[i].pl_owner == winner) {
	 planets[i].pl_armies += random() % (2 * count);
      }
   }
}

void
surrenderMessage(loser)
   int             loser;
{
   char            buf[80];
   FILE           *conqfile;
   time_t          curtime;

   conqfile = fopen(CONQFILE, "a");

   /* TC 10/90 */
   time(&curtime);
   strcpy(buf, "\nSurrender! ");
   strcat(buf, ctime(&curtime));
   fprintf(conqfile, "  %s\n", buf);

   sprintf(buf, "The %s %s surrendered.", teamNames[loser],
	   teamVerbage[loser]);

   fprintf(conqfile, "  %s\n", buf);

#ifdef nodef
   sprintf(buf, "The %s:", teamNames[winner]); 
   pmessage(buf, 0, MALL | MGENO, " "); 
   fprintf(conqfile, "  %s\n", buf); 
   displayBest(conqfile, winner, KGENOCIDE);

   sprintf(buf, "The %s:", teamNames[loser]); 
   pmessage(buf, 0, MALL | MGENO, " "); 
   fprintf(conqfile, "  %s\n", buf); 
   displayBest(conqfile, loser, KGENOCIDE);
#endif

   fprintf(conqfile, "\n");
   fclose(conqfile);
}

void
genocideMessage(loser, winner)
   int             loser, winner;
{
   char            buf[80];
   FILE           *conqfile;
   time_t	   curtime;

   conqfile = fopen(CONQFILE, "a");

/* TC 10/90 */
   time(&curtime);
   strcpy(buf,"\nGenocide! ");
   strcat(buf, ctime(&curtime));
   fprintf(conqfile,"  %s\n",buf);

   pmessage("***********************************************************",
	    0, MALL|MGENO, " ");
   sprintf(buf, "The %s %s been genocided by the %s.", teamNames[loser],
	   teamVerbage[loser], teamNames[winner]);
   pmessage(buf, 0, MALL|MGENO, " ");

   fprintf(conqfile, "  %s\n", buf);
   sprintf(buf, "The %s:", teamNames[winner]);
   pmessage(buf, 0, MALL|MGENO, " ");
   fprintf(conqfile, "  %s\n", buf);
   displayBest(conqfile, winner, KGENOCIDE);
   sprintf(buf, "The %s:", teamNames[loser]);
   pmessage(buf, 0, MALL|MGENO, " ");
   fprintf(conqfile, "  %s\n", buf);
   displayBest(conqfile, loser, KGENOCIDE);
   pmessage("***********************************************************",
	    0, MALL|MGENO, " ");
   fprintf(conqfile, "\n");
   fclose(conqfile);
}

void
conquerMessage(winner)
   int             winner;
{
   char            buf[80];
   FILE           *conqfile;
   time_t          curtime;

   conqfile = fopen(CONQFILE, "a");

   /* TC 10/90 */
   time(&curtime);
   strcpy(buf, "\nConquer! ");
   strcat(buf, ctime(&curtime));
   fprintf(conqfile, "  %s\n", buf);

   pmessage("***********************************************************",
	    0, MALL | MCONQ, " ");
   sprintf(buf, "The galaxy has been conquered by the %s:", teamNames[winner]);
   pmessage(buf, 0, MALL | MCONQ, " ");
   fprintf(conqfile, "  %s\n", buf);
   sprintf(buf, "The %s:", teamNames[winner]);
   pmessage(buf, 0, MALL | MCONQ, " ");
   fprintf(conqfile, "  %s\n", buf);
   displayBest(conqfile, winner, KWINNER);
   pmessage("***********************************************************",
	    0, MALL | MCONQ, " ");
   fprintf(conqfile, "\n");
   fclose(conqfile);
}

typedef struct playerlist {
   char            name[16];
   char            mapchars[2];
   int             planets, armies;
}               Players;

void
displayBest(conqfile, team, type)
   FILE           *conqfile;
   int             team, type;
{
   register int    i, k, l;
   register struct player *j;
   int             planets, armies;
   Players         winners[MAXPLAYER + 1];
   char            buf[80];
   int             number;

   number = 0;
   for (i = 0, j = &players[0]; i < MAXPLAYER; i++, j++) {
      if (j->p_team != team || j->p_status == PFREE)
	 continue;
      if (type == KGENOCIDE) {
	 planets = j->p_genoplanets;
	 armies = j->p_genoarmsbomb;
	 j->p_genoplanets = 0;
	 j->p_genoarmsbomb = 0;
      } else {
	 planets = j->p_planets;
	 armies = j->p_armsbomb;
      }
      for (k = 0; k < number; k++) {
	 if (30 * winners[k].planets + winners[k].armies <
	     30 * planets + armies) {
	    /* insert person at location k */
	    break;
	 }
      }
      for (l = number; l >= k; l--) {
	 winners[l + 1] = winners[l];
      }
      number++;
      winners[k].planets = planets;
      winners[k].armies = armies;
      strncpy(winners[k].mapchars, j->p_mapchars, 2);
      strncpy(winners[k].name, j->p_name, 16);
      winners[k].name[15] = 0;	/* `Just in case' paranoia */
   }
   for (k = 0; k < number; k++) {
      if (winners[k].planets != 0 || winners[k].armies != 0) {
	 sprintf(buf, "  %16s (%2.2s) with %d planets and %d armies.",
		 winners[k].name, winners[k].mapchars, winners[k].planets, winners[k].armies);
	 if(type == KWINNER)
	    pmessage(buf, 0, MALL|MCONQ, " ");
	 else
	    pmessage(buf, 0, MALL|MGENO, " ");
	 fprintf(conqfile, "  %s\n", buf);
      }
   }
   return;
}


#ifdef SNAKEPATROL
/*
 * Our check works this way: If no snake existing, snake is started to patrol
 * a neutral area -- if robot guarding 1, snake patrols other, if no robot,
 * snake patrols both, if 2 robots snake patrols both (roughly).  Check the
 * snake source to see how it handles bombers, robots, etc.. If not t-mode
 * snake patrols all 4.
 */

void
checksnake()
{
   register        i, n, r;
   struct player  *j;
   int             t1 = 0, t2 = 0, min = MAXPLAYER;

   /* first check to see if there is a snake in the game */
   for (i = 0, j = players, n = 0, r = 0; i < MAXPLAYER; i++, j++) {
      if (j->p_status != PFREE) {
	 if (j->p_flags & PFROBOT) {
	    r++;
	    if (j->p_flags & PFSNAKE)
	       return;
	 } else
	    n++;
      }
   }

   /* room in game? */
   if (n + r > MAXPLAYER - 3)	/* note: leaving 1 slot open always */
      return;
   if (n == 0)			/* no human players */
      return;

   if (!status->tourn) {
      /* patrol galaxy */
      start_snake(0, 0);
      return;
   }
   /* no snake.  Now check tcount to find minimum race(s) */
   for (i = 0; i < NUMTEAM; i++) {
      if (tcount[1 << i] >= tournplayers)
	 continue;
      if (tcount[1 << i] < min) {
	 min = tcount[1 << i];
	 t1 = 1 << i;
      }
   }
   min = MAXPLAYER;

   for (i = 0; i < NUMTEAM; i++) {
      if (tcount[1 << i] >= tournplayers)
	 continue;
      if (t1 == 1 << i)
	 continue;

      if (tcount[1 << i] < min) {
	 min = tcount[1 << i];
	 t2 = 1 << i;
      }
   }

   printf("snake patrolling %s, %s\n", teamshort[t1], teamshort[t2]);
   fflush(stdout);

   /* random swap to let snake belong to all teams */
   if ((random() % 2) == 0) {
      min = t1;
      t1 = t2;
      t2 = min;
   }
   start_snake(t1, t2);
}

/* note snake doesn't patrol 3 races */
void
start_snake(pt1, pt2)
   int             pt1, pt2;
{
   char            team1[4], team2[4], *pt;

   /*
    * snake interprets these flags accoording to patrol rules described above
    */
   sprintf(team1, "-T%c", teamlet[pt1]);
   sprintf(team2, "-T%c", teamlet[pt2]);

   pt = "-p";			/* patrol */

   if (fork() == 0) {
      (void) signal(SIGALRM, SIG_DFL);
      /*
       * hmm, safe? (void) close(0); (void) close(1); (void) close(2);
       */

      execl(SNAKEF, "snake", team1, team2, pt, 0);
      perror(SNAKEF);
      _exit(1);
   }
}
#endif				/* SNAKEPATROL */

#ifdef BIRD
void
start_bird(t, target)
   int             t;
   char            target;
{
   char            team[4], targ[4];

   sprintf(team, "-T%c", teamlet[t]);
   sprintf(targ, "-t%c", target);

   if (fork() == 0) {
      (void) signal(SIGALRM, SIG_DFL);
      /*
       * hmm, safe? (void) close(0); (void) close(1); (void) close(2);
       */

      execl(BIRDF, "bird", team, targ, 0);
      perror(BIRDF);
      _exit(1);
   }
}
#endif				/* BIRD */

#ifdef MINES
/*
 * initially:		t - torp, X - 20 space (1 pixel), M - me o - torp
 * origin.
 * 
 * tttXXXX totXXXX tttXXXX XXXMXXX XXXXttt XXXXtot XXXXttt
 */

void
rotmine(j)
   struct torp    *j;
{
   /* other in pair */
   struct torp    *k = &torps[j->t_pair];

   if (j->t_status == TEXPLODE || k->t_status == TEXPLODE)
      return;

   /* we move both at once here */
   if (j->t_pair > j->t_no)
      return;

   if (j->t_x < k->t_x && j->t_y < k->t_y) {
      j->t_x += 40;
      k->t_x -= 40;
   } else if (j->t_x == k->t_x && j->t_y < k->t_y) {
      j->t_x += 40;
      k->t_x -= 40;
   } else if (j->t_x > k->t_x && j->t_y < k->t_y) {
      j->t_y += 40;
      k->t_y -= 40;
   } else if (j->t_x > k->t_x && j->t_y == k->t_y) {
      j->t_y += 40;
      k->t_y -= 40;
   } else if (j->t_x > k->t_x && j->t_y > k->t_y) {
      j->t_x -= 40;
      k->t_x += 40;
   } else if (j->t_x == k->t_x && j->t_y > k->t_y) {
      j->t_x -= 40;
      k->t_x += 40;
   } else if (j->t_x < k->t_x && j->t_y > k->t_y) {
      j->t_y -= 40;
      k->t_y += 40;
   } else if (j->t_x < k->t_x && j->t_y == k->t_y) {
      j->t_y -= 40;
      k->t_y += 40;
   } else
      fprintf(stderr, "missed one j:%d,%d, k:%d,%d\n",
	      j->t_x, j->t_y, k->t_x, k->t_y);

}
#endif				/* MINES */

#ifdef BASEPRACTICE
void
startbasep()
{
   if (fork() == 0) {
      (void) signal(SIGALRM, SIG_DFL);
      execl(BASEP, "basep", 0);
      perror(BASEP);
      _exit(1);
   }
}

#ifdef nodef	/* NOT USED */
void
udstatus()
{
   register struct player	*j;
   register			i;

   /* here's the rap: since the only stats meaningful are damage inflicted
      and damage received and of course they don't get transmitted to the
      client, we have to kludge them somehow into tkills and tlosses */

   status->kills = (int) status->dami/100;
   status->losses = (int) status->damr/100;

   for(i=0,j=players; i< MAXPLAYER; i++,j++){
      if(j->p_status != PALIVE) continue;
      if(!isSB(j)) continue;
      /* the higher this number the better */
      j->p_stats.st_tkills = (int) j->p_stats.st_dami/100;
      /* the lower this number the better */
      j->p_stats.st_tlosses = (int) j->p_stats.st_damr/100;
   }
}
#endif

#endif

#ifdef ORGANIAN
void
organian_start(winner)
   struct player  *winner;
{
   register int    i, h, k, count;
   register struct planet *l;
   int             team[MAXTEAM + 1];
   int             conq = 0;

   for (i = 0; i < 4; i++)
      team[1 << i] = 0;


   for (k = 0; k < 4; k++) {	/* all teams */
      for (i = 0; i < 4; i++) {	/* all race planet systems */
	 count = 0;
	 for (h = 0, l = &planets[i * 10]; h < 10; h++, l++) {
	    if (l->pl_owner == (1 << k) || l->pl_owner == NOBODY) {
	       if (++count == 10) {
		  team[(1 << k)]++;
	       }
	    }
	 }
      }
   }

   for (i = 0; i < 4; i++) {
      if (team[1 << i] >= VICTORY) {
	 /* We have a winning team */
	 conquerMessage(winner->p_team);
	 conq = 1;
	 /* conquerer is same as genocider */
	 whoconq = winner->p_no;
	 break;
      }
   }
   if (!conq)
      return;

   organian = 2;
   start_organians();
}

void
start_organians()
{
   pmessage("The Organians will reset the galaxy.",
	    0, MALL, "GOD->ALL");

   if (fork() == 0) {
      (void) signal(SIGALRM, SIG_DFL);
      execl(ORGANIANP, "organian", "-p", "-q", 0);
      perror(ORGANIANP);
      _exit(1);
   }
}

void
check_organians()
{
   register        i;
   register struct player *j;
   register struct planet *k;

   if (organian == 2) {		/* organians started */

      for (i = MAXPLAYER - 1, j = &players[i]; i >= 0; i--, j--) {
	 if (j->p_flags & PFORGANIAN)
	    /* they're still in the game */
	    return;
      }
      /*
       * no organians in the game .. we must assume the program failed or the
       * galaxy has been reset
       */
      organian = 3;
   } else {

      /* double check -- gives at least 5 seconds for program to start */
      for (i = MAXPLAYER - 1, j = &players[i]; i >= 0; i--, j--) {
	 if (j->p_flags & PFORGANIAN) {
	    /* they're still in the game */
	    organian = 2;
	    return;
	 }
      }

      /* otherwise kick everybody out like checkwin */
      pmessage("Bye...", 0, MALL, "GOD->ALL");

      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if (j->p_status == PFREE)
	    continue;
	 j->p_status = PEXPLODE;
	 j->p_whydead = KWINNER;
	 j->p_whodead = whoconq;
	 if (j->p_ship.s_type == STARBASE)
	    j->p_explode = (2 * SBEXPVIEWS/UPDATE_NORM) / PLAYERFUSE;
	 else
	    j->p_explode = (10/UPDATE_NORM) / PLAYERFUSE;
	 j->p_ntorp = 0;
	 j->p_nplasmatorp = 0;
      }
      organian = 1;

      /* and to be safe, a final planet check */

      for (i = 0, k = planets; i < MAXPLANETS; i++, k++) {
	 if(jmn_planets){
	    k->pl_owner = pdata_jmn[i].pl_owner;
	    k->pl_info = pdata_jmn[i].pl_info;
	 }
	 else{
	    k->pl_owner = pdata[i].pl_owner;
	    k->pl_info = pdata[i].pl_info;
	 }
	 k->pl_armies = 30;
      }
   }

   /* don't need longjmp here do we? */
}
#endif

/* maximum number of agris that can exist in one quadrant */

#define AGRI_LIMIT 3

/* the four close planets to the home planet */
static int      core_planets[4][4] =
{{7, 6, 5, 8,},
{12, 19, 15, 16,},
{24, 29, 25, 26,},
{34, 39, 38, 37,},
};
/* the outside edge, going around in order */
static int      front_planets[4][5] =
{{1, 2, 4, 6, 3,},
{14, 18, 13, 17, 11,},
{22, 28, 23, 21, 27,},
{31, 32, 33, 35, 36,},
};

void
doResources()
{
   int             i, j, k, which;

   if(jmn_planets)
      bcopy(pdata_jmn, planets, sizeof(pdata_jmn));
   else
      bcopy(pdata, planets, sizeof(pdata));
   for (i = 0; i < 4; i++) {
      /* one core AGRI */
      planets[core_planets[i][random() % 4]].pl_flags |= PLAGRI;

      /* one front AGRI */
      which = random() % 2;
      if (which) {
	 planets[front_planets[i][random() % 2]].pl_flags |= PLAGRI;

	 /* place one repair on the other front */
	 planets[front_planets[i][(random() % 3) + 2]].pl_flags |= PLREPAIR;

	 /* place 2 FUEL on the other front */
	 for (j = 0; j < 2; j++) {
	    do {
	       k = random() % 3;
	    } while (planets[front_planets[i][k + 2]].pl_flags & PLFUEL);
	    planets[front_planets[i][k + 2]].pl_flags |= PLFUEL;
	 }
      } else {
	 planets[front_planets[i][(random() % 2) + 3]].pl_flags |= PLAGRI;

	 /* place one repair on the other front */
	 planets[front_planets[i][random() % 3]].pl_flags |= PLREPAIR;

	 /* place 2 FUEL on the other front */
	 for (j = 0; j < 2; j++) {
	    do {
	       k = random() % 3;
	    } while (planets[front_planets[i][k]].pl_flags & PLFUEL);
	    planets[front_planets[i][k]].pl_flags |= PLFUEL;
	 }
      }

      /*
       * drop one more repair in the core (home + 1 front + 1 core = 3
       * Repair)
       */
      planets[core_planets[i][random() % 4]].pl_flags |= PLREPAIR;

      /* now we need to put down 2 fuel (home + 2 front + 2 = 5 fuel) */
      for (j = 0; j < 2; j++) {
	 do {
	    k = random() % 4;
	 } while (planets[core_planets[i][k]].pl_flags & PLFUEL);
	 planets[core_planets[i][k]].pl_flags |= PLFUEL;
      }
   }

#ifdef BASEPRACTICE
   {
      register struct planet	*l;

      for (i = 0, l = planets; i < MAXPLANETS; i++, l++)
	 l->pl_flags |= (PLREPAIR | PLFUEL);
   }
#endif
}

/* base inflicted 'damage' on 'victim'.  Compensate for victim's
   defense rating -- high defense returns higher damage, low defense returns
   lower damage */

double
def_factor(victim, damage, base)
   
   struct player	*victim;
   int			damage;
   struct player	*base;
{
#ifdef BASEPRACTICE

   double		d;
   if(!victim->p_df)		/* shouldn't happen */
      d = (double)damage;
   else
      d = (double)(victim->p_df * damage)/100.;

   /* if you aren't the target you don't get as much credit */
   if(victim->p_tg && victim->p_tg-1 != base->p_no)
      d = d/4;

#ifdef DEBUG
   printf("base gets %f damage inflicted out of %d (victim %c%c, def: %d)\n",
      (double)d, damage, victim->p_mapchars[0], victim->p_mapchars[1],
      victim->p_df);
   fflush(stdout);
#endif

   return d;
#endif
}

#ifdef nodef
/* base received 'damage' from 'killer'.  Compensate for killer's
   offense rating -- high offense rating returns lower damage, lower offense
   rating returns higher damage */

double
off_factor(killer, damage, base)

   struct player	*killer;
   int			damage;
   struct player	*base;
{
   double	d;

   if(!killer->p_of)		/* shouldn't happen */
      d = (double)damage;
   else
      d = (double) damage + ((100-killer->p_of)*damage)/100;

#ifdef DEBUG
   printf("base gets %f damage received out of %d (killer %c%c, off: %d)\n",
      (double)d, damage, killer->p_mapchars[0], killer->p_mapchars[1],
      killer->p_of);
   fflush(stdout);
#endif

   return d;
}
#endif

#ifdef DS

/*
 * The obvious problem here is that the only client update speeds that 
 * make sense are 10, 5, 3, 2, 1 at 100000 u/s per update.
 */

void
signal_servers()
{
   register			i, t;
   register struct player	*j;

   for(i=0,j=players; i< MAXPLAYER; i++, j++){
      if(!j->p_status) continue;
      if(j->p_process <= 1) continue;

      t = j->p_timerdelay;
      if(!t)				/* paranoia */
	 t = 5;

      /* adding 'i' to 'ticks' allows us to "interleave" update signals to
	 the ntserv processes: at 5 u/s, 1/2 the processes are signalled on
	 every update, at 2 u/s, 1/5 are signalled, etc.  (Of course, if 
	 everyone is at 10 u/s we have to signal them all every time) */

      if(t==1 /*skip mod*/|| (((ticks+ i*(int)UPDATE_NORM) % t) == 0)){
	 if(kill(j->p_process, SIGALRM) < 0){
	    /* if the ntserv process died what are we doing here? */
	    if(errno == ESRCH)
	       j->p_status = PFREE;
	 }
      }
   }
}

#endif

#ifdef VOTE
void
udvotegalaxyreset()
{
   if(vote->v_resetgalaxy_votes){
      register			i;
      register struct player	*j;
      int				team[MAXTEAM + 1], teams=0;

      bzero(&team, sizeof(team));

      /* v_resetgalaxy was set when a race was genocided but a third race
	 still remains. */
   
      for(i=0,j=players; i< MAXPLAYER; i++,j++){
	 if(!j->p_status || (j->p_flags & PFROBOT)) continue;
	 if(voted_yes(vote->v_resetgalaxy_votes, i)){
	    team[j->p_team]++;
	 }
      }
      for(i=0; i< NUMTEAM; i++)
	 if(team[1 << i] > 1)	/* NEW: requires 2 votes minimum each team */
	    teams++;
      
      if(teams > 1){
	 /* reset has been voted down. */
	 vote_resetgalaxy_off();
	 pmessage("RESET has been disabled.", 0, MALL, "GOD->ALL");
	 return;
      }
   }
   /* how much time do we have left? */
   if(vote->v_resetgalaxy_timer > 0){
      char	buf[80];

      if((vote->v_resetgalaxy_timer % 5)==0){

	 sprintf(buf, "RESET will occur in %d seconds", 
	    vote->v_resetgalaxy_timer);
	 pmessage(buf, 0, MALL, "GOD->ALL");
	 if(vote->v_resetgalaxy_timer == 20)
	    pmessage("Send NORESET to yourself to disable.", 
		     0, MALL, "GOD->ALL");
      }
      vote->v_resetgalaxy_timer --;
   }
   else {
      /* no vote, timer expired.  Do reset */
      vote_resetgalaxy_off();
      resetGalaxy(_winner);
   }
}

void 
vote_resetgalaxy_off()
{
   vote->v_resetgalaxy_timer = 0;
   vote->v_resetgalaxy = 0;
   vote->v_resetgalaxy_votes = 0;
}

void
vote_resetgalaxy_on()
{
   vote->v_resetgalaxy = 1;
   vote->v_resetgalaxy_timer = 20;
   vote->v_resetgalaxy_votes = 0;
}

/* general voting */
void
udvote()
{

   if(vote->v_resetplanet_votes){
      register			i;
      register struct player	*j;
      int			team[MAXTEAM + 1], numteams=0, teams=0;

      bzero(&team, sizeof(team));

      for(i=0,j=players; i< MAXPLAYER; i++,j++){
	 if(!j->p_status || (j->p_flags & PFROBOT)) continue;
	 if(voted_yes(vote->v_resetplanet_votes, i)){
	    team[j->p_team]++;
	 }
      }
      /* votes are in */
      for(i=0; i< NUMTEAM; i++){
	 if(pcount[1<<i] > 0)
	    numteams++;
	 /* majority rules */
	 if(team[1<<i] > pcount[1<<i]/2)
	    teams ++;
      }
      if(teams > 1 || (teams > 0 && numteams == 1)){
	 /* do it */
	 pmessage(" ", 0, MALL, "GOD->ALL");
	 pmessage("PLANET RESET has been requested by popular vote.", 
	    0, MALL, "GOD->ALL");
	 pmessage(" ", 0, MALL, "GOD->ALL");
	 doResources();
	 vote->v_resetplanet_votes = 0;
	 for(i=0,j=players; i< MAXPLAYER; i++,j++){
	    if(j->p_ship.s_type != STARBASE)
	       j->p_kills = 0.0;
	    j->p_armies = 0;
	 }
      }
   }
}

int
voted_yes(v, i)

   unsigned int	v;
   int		i;
{
   return (v & (1 << i));
}

#endif /* VOTE */

#ifdef BORG
void
do_tourn_start()
{
   register			i,j;
   register struct planet	*l;
   char				*nukepl[MAXPLANETS];
   char				*argv[MAXPLANETS];
   int				team1= -1, team2 = -1;

   bzero(nukepl, sizeof(nukepl));

   /* which are the active teams? */
   for(i=0; i< NUMTEAM; i++){
      if(pcount[1 << i] >= tournplayers){
	 if(team1 < 0) {
	    team1 = i;
	 }
	 else {
	    team2 = i;
	    break;
	 }
      }
   }
   if(team1 < 0 || team2 < 0) {
      return;
   }

   /* which planets on team1 are not owned by team1 or team2? */
   for(j=0, l=&planets[team1*10]; j<10; j++,l++){
      if(l->pl_owner != (1 << team1) && l->pl_owner != (1 << team2)){
	 nukepl[team1*10+j] = l->pl_name;
      }
   }
   /* which planets on team2 are not owned by team1 or team2? */
   for(j=0, l=&planets[team2*10]; j<10; j++,l++){
      if(l->pl_owner != (1 << team1) && l->pl_owner != (1 << team2)){
	 nukepl[team2*10+j] = l->pl_name;
      }
   }
   /* create an argument vector */
   for(i=0,j=1; i< MAXPLANETS; i++){
      if(nukepl[i]){
	 argv[j] = nukepl[i];
	 j++;
      }
   }
   argv[0] =  "borg";
   argv[j] = NULL;
   if(j == 1) {
      return;
   }
      
   if (fork() == 0) {
      (void) signal(SIGALRM, SIG_DFL);
      execv(BORG_BOT, argv);
      perror(BORG_BOT);
      _exit(1);
   }
}
#endif
