/*
 * rsa_key.c
 * 
 * Mike Polek   11/92
 */
#include "copyright2.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/file.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern long                   random P_((void));
extern void                   perror P_((const char *));
extern int                    getsockname P_((int s, struct sockaddr *name, int *namelen));
extern void                   bcopy P_((const void *, void *, int));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    read P_((int, void *, unsigned int));
extern int                    bcmp P_((const void *, const void *, int));
extern int                    close P_((int));
extern int                    fflush P_((FILE *));
extern time_t                 time P_((time_t *));
extern int                    fclose P_((FILE *));

#undef P_
#endif

void
makeRSAPacket(packet)
   struct rsa_key_spacket *packet;
{
   int             i;

   for (i = 0; i < KEY_SIZE; i++)
      packet->data[i] = random() % 256;
   /* make sure it is less than the global key */
   for (i = KEY_SIZE - 10; i < KEY_SIZE; i++)
      packet->data[i] = 0;
   packet->type = SP_RSA_KEY;
}

/* returns 1 if the user verifies incorrectly */
int
decryptRSAPacket(spacket, cpacket, serverName, key)
   struct rsa_key_spacket *spacket;
   struct rsa_key_cpacket *cpacket;
   char           *serverName;
   struct rsa_key *key;
{
   struct sockaddr_in saddr;
   unsigned char   temp[KEY_SIZE], *data;
   int             fd;
   int             done, found, len;

   len = sizeof(saddr);
   if (getsockname(sock, (struct sockaddr *)&saddr, &len) < 0) {
      perror("getsockname(sock)");
      exit(1);
   }
   /* replace the first few bytes of the message */
   /* will be the low order bytes of the number */
   data = spacket->data;
   /* NEW, ignore the socket address and use the serverName passed in */
   saddr.sin_addr.s_addr = inet_addr(serverName);
   /*
   fprintf(stderr, "testing %s (%d)\n", serverName, saddr.sin_port);
   */

   bcopy(&saddr.sin_addr.s_addr, data, sizeof(saddr.sin_addr.s_addr));
   data += sizeof(saddr.sin_addr.s_addr);
   bcopy(&saddr.sin_port, data, sizeof(saddr.sin_port));

   fd = open(RSA_KEY_FILE, O_RDONLY);
   if (fd < 0) {
      fprintf(stderr, "[RSA] Can't open the RSA key file %s\n", RSA_KEY_FILE);
      /* exit(1); */
      return 1;
   }
   done = found = 0;
   do {
      if (read(fd, key, sizeof(struct rsa_key)) != sizeof(struct rsa_key))
	 done = 1;
      if (!(bcmp(key->global, cpacket->global, KEY_SIZE) ||
	    bcmp(key->public, cpacket->public, KEY_SIZE)))
	 done = found = 1;
   } while (!done);

   close(fd);

   /* If he wasn't in the file, kick him out */
   if (!found) {
      *key->key_name = 0;	/* not found */
      return 1;
   }

   rsa_encode(temp, cpacket->resp, key->public, key->global, KEY_SIZE);

   /* If we don't get the right answer, kick him out */
   if (bcmp(temp, spacket->data, KEY_SIZE)) {
      return 1;
   }

   return 0;
}
