/*
 * rmove2.c
 */
#include "copyright.h"

#include <stdio.h>
#include <signal.h>
#include <values.h>
#include <math.h>
#include <time.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#ifndef _NO_DEPEND
#include "snake_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern long                   random P_((void));
extern char *                 strcat P_((char *, const char *));
extern int                    printf P_((const char *, ...));
extern int                    fflush P_((FILE *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void                   bzero P_((void *, size_t));
extern void                   perror P_((const char *));

#undef P_
#endif

#define MAXFUSE			32767

/* distance on either side of center */
#define EYEWIDTH		100	/* 100 */
/* which torp the eyes are positioned alongside */
#define EYETORP			0	/* 0 */
/* angle from EYETORP */
#define EYEANGLE		64	/* 64 */

/* threshold to avoid galactic boundaries */
#define MD		4500
/* overlap threshold on team boundaries */
#define TMD		2250

/* how often to check for bombers in cyles (1/10th second) */
#define BOMBCHECK	20

/* how often to check for players in the game in cyles (1/10th second) */
#define PLAYERCHECK	100

#define WARHOSTILE(p)		((((p)->p_swar | (p)->p_hostile)& \
				perfs[0]->p_team) || ((perfs[0]->p_swar | \
				perfs[0]->p_hostile) & (p)->p_team))

#define SNAKETORPDAMAGE		30
#define SNAKEPLASMADAMAGE	100

static struct torp *thead, *eyet, *eyetp;
static int      lx, ly;
static struct plasmatorp *fl, *fr;
static int      explode, explodetorps, tfuse;

extern struct player *perfs[2];
extern int      num_perfs;
extern int      debug;
extern int      lastm;
extern int      tno;
extern int      target;
extern int      berserk;
extern int      patrol;
extern int      team1, team2;

static int      s_clock;
static          _move();
static int      defenders[ALLTEAM];

struct player  *whokilledme();

unsigned char   getacourse();

void
snakemove()
{
   s_clock++;

   _move();
}

static int
_move()
{
   if (!perfs[0] || !perfs[1])
      exitSnake(0);

   if(perfs[0]->p_status == PFREE ||
      perfs[1]->p_status == PFREE)
      exitSnake(0);

   /* galaxy conquered, etc.. */
   if(perfs[0]->p_status == PEXPLODE ||
      perfs[1]->p_status == PEXPLODE)
      exitSnake(0);

   /* keep ghostbuster away */
   perfs[0]->p_ghostbuster = 0;
   perfs[1]->p_ghostbuster = 0;

   if (s_clock == 5)
      startsnake();
   else if (s_clock > 5) {
      check_explode();
      if (!explode) {
	 movesnake();
      } else if ((perfs[0]->p_ntorp == 0 && perfs[1]->p_ntorp == 0 &&
	    perfs[0]->p_nplasmatorp == 0 && perfs[1]->p_nplasmatorp == 0) ||
	 /* xx -- sometimes above doesn't work? */
		 s_clock - explode > 56)
	 exitSnake(0);
   }
   return 1;
}

void
startsnake()
{
   register        i, l;
   register struct player *j;
   register struct torp *k;
   struct player  *p1 = perfs[0], *p2 = perfs[1];
   int             first = 1;
   int             px, py;

   px = (random() % 10000) - 5000;
   py = (random() % 10000) - 5000;

   p1->p_ntorp = p2->p_ntorp = 0;
   p1->p_nplasmatorp = p2->p_nplasmatorp = 0;

   /* body */

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {

      for (l = 0, i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++, l++) {

	 if (l == EYETORP && j == perfs[0]) {
	    /* eye location */
	    eyet = k;
	 }
	 if (l == ((EYETORP == 0) ? 1 : (EYETORP - 1)) && j == perfs[0]) {
	    /* perpendicular torp offset */
	    eyetp = k;
	 }
	 /* torps are free here */
	 snake_torp(k, i, j);
	 /* note: have to be same team for this to work */
	 k->t_x = planets[tno * 10].pl_x + px;
	 k->t_y = planets[tno * 10].pl_y + py - l * 200 - (j == perfs[1] ? 1600 : 0);
	 /*
	  * if(debug) fprintf(stderr, "k->t_x %d, k->t_y %d\n", k->t_x,
	  * k->t_y);
	  */
	 if (first) {
	    thead = k;
	    first = 0;
	 }
      }
   }

   /* eyes */
   fl = &plasmatorps[perfs[0]->p_no * MAXPLASMA];
   fl->pt_no = perfs[0]->p_no * MAXPLASMA;
   fl->pt_status = PTMOVE;
   fl->pt_owner = perfs[0]->p_no;
   fl->pt_team = perfs[0]->p_team;
   fl->pt_x = eyet->t_x - EYEWIDTH;
   fl->pt_y = eyet->t_y;
   fl->pt_damage = SNAKEPLASMADAMAGE;
   fl->pt_speed = 0;
   fl->pt_war = 0;
   fl->pt_fuse = MAXFUSE;
   fl->pt_turns = 0;

   (perfs[0])->p_nplasmatorp++;

   fr = &plasmatorps[perfs[1]->p_no * MAXPLASMA];
   fr->pt_no = perfs[1]->p_no * MAXPLASMA;
   fr->pt_status = PTMOVE;
   fr->pt_owner = perfs[1]->p_no;
   fr->pt_team = perfs[1]->p_team;	/* doesn't work */
   fr->pt_x = eyet->t_x + EYEWIDTH;
   fr->pt_y = eyet->t_y;
   fr->pt_damage = SNAKEPLASMADAMAGE;
   fr->pt_speed = 0;
   fr->pt_war = 0;
   fr->pt_fuse = MAXFUSE;
   fr->pt_turns = 0;

   (perfs[1])->p_nplasmatorp++;

   if (debug)
      fprintf(stderr, "started\n");
}

void
movesnake()
{
   register        i, l, px, py;
   register struct player *j;
   register struct torp *k;
   unsigned char   tc;
   struct player  *tr;
   static
   int             dir = 8;
   int             ok;

   if ((s_clock % PLAYERCHECK) == 0) {
      /*
       * every x seconds make sure there's people in the game
       */
      ok = 0;
      defenders[FED] = defenders[ROM] = defenders[KLI] = defenders[ORI] = 0;
      for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
	 if ((j->p_status != PFREE) && !(j->p_flags & PFROBOT)) {
	    ok = 1;
	    defenders[j->p_team]++;
	 }
      }
      if (!ok)
	 exitSnake(0);
   }
   if (patrol && (target == -1) && (s_clock % BOMBCHECK) == 0) {
      target = bombcheck(team1, team2);
   }
   if ((s_clock % (4 + (random() % 4))) == 0)
      dir = -dir;

   thead->t_dir += dir + rrnd(8);

   if (target > -1) {
      tr = &players[target];
      if (tr->p_status == PALIVE) {
	 int             nd, td;
	 tc = getacourse(tr->p_x, tr->p_y, thead->t_x, thead->t_y);
	 nd = angdist(thead->t_dir, tc);
	 if (nd > 8) {
	    td = tc + nd;
	    if (td == thead->t_dir)
	       thead->t_dir -= 8;
	    else
	       thead->t_dir += 8;
	 }
      }
   }
   if (target == -1) {
      if (!patrol)
	 check_tboundary(ALLTEAM, &thead->t_dir, 8, thead->t_x, thead->t_y);
      else
	 check_tboundary(team1 | team2, &thead->t_dir, 8, thead->t_x, thead->t_y);
   }
   lx = thead->t_x;
   ly = thead->t_y;

   /* NOTE: we aren't letting the daemon move the torp head */
   thead->t_x += (double) (12 * WARP1) * Cos[thead->t_dir];
   thead->t_y += (double) (12 * WARP1) * Sin[thead->t_dir];
   thead->t_fuse = MAXFUSE;

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {

      for (l = 0, i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++, l++) {

	 if (k->t_status == TFREE)
	    /* got exploded.  x & y location will remain however */
	    snake_torp(k, i, j);

	 if (k == thead)
	    continue;

	 px = k->t_x;
	 py = k->t_y;

	 k->t_x = lx;
	 k->t_y = ly;
	 k->t_fuse = MAXFUSE;

	 lx = px;
	 ly = py;
      }
   }
   lx = thead->t_x;
   ly = thead->t_y;

   doeyes();
}

void
check_explode()
{
   register int    i, l;
   register struct player *j;
   register struct torp *k;
   static struct player *killer;

   if (fl->pt_status == PTDET || fr->pt_status == PTDET ||
       fl->pt_status == PTEXPLODE || fr->pt_status == PTEXPLODE || explode) {
      if (debug)
	 fprintf(stderr, "snake exploding\n");
      if (!explode) {
	 /* do once */
	 explode = s_clock;
	 tfuse = 0;
	 if (fl->pt_status == PTDET || fl->pt_status == PTEXPLODE) {
	    killer = whokilledme(fl);
	    if (killer)
	       award(killer);
	 }
	 if (fr->pt_status == PTDET || fr->pt_status == PTEXPLODE) {
	    killer = whokilledme(fr);
	    if (killer)
	       award(killer);
	 }
      }
      if (fl->pt_status != PTFREE && fl->pt_status != PTEXPLODE) {
	 fl->pt_war = FED | ROM | KLI | ORI;	/* make sure its at war when
						 * it explodes */
	 fl->pt_status = PTEXPLODE;
	 fl->pt_fuse = 10;
      }
      if (fl->pt_status != PTFREE && fr->pt_status != PTEXPLODE) {
	 fl->pt_war = FED | ROM | KLI | ORI;
	 fr->pt_status = PTEXPLODE;
	 fr->pt_fuse = 10;
      }
      /*
       * now for some fancy stuff. If killer is our target and is hostile or
       * at war with our team then snake torp head makes a beeline for him.
       * This lasts until killer dies or torps have been chasing killer for
       * 40 cycles
       */

      if (killer && WARHOSTILE(killer) && (killer->p_status == PALIVE) &&
	  tfuse < 40 && killer->p_no == target) {
	 crash_killer(killer);
      } else {
	 /* explode all torps in sequence, 1 per cycle until all gone */
	 for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
	    for (l = (j == perfs[0]) ? 0 : 8, i = j->p_no * MAXTORP, k = &torps[i];
		 i < j->p_no * MAXTORP + MAXTORP; i++, k++, l++) {
	       if (l == explodetorps && k->t_status != TEXPLODE &&
		   k->t_status != TFREE) {
		  k->t_status = TEXPLODE;
		  k->t_fuse = 10;
		  explodetorps++;
		  return;
	       } else if (l == explodetorps)
		  explodetorps++;
	    }
	 }
      }
   }
}

int
crash_killer(p)
   struct player  *p;
{
   register int    i, px, py;
   register struct player *j;
   register struct torp *k;
   unsigned char   tc;
   int             active = 0;

   tfuse++;

   tc = getacourse(p->p_x, p->p_y, thead->t_x, thead->t_y);
   thead->t_dir = tc;

   lx = thead->t_x;
   ly = thead->t_y;

   thead->t_x += (double) (12 * WARP1) * Cos[thead->t_dir];
   thead->t_y += (double) (12 * WARP1) * Sin[thead->t_dir];

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {

	 /* move the head up if it exploded */
	 if (k != thead && thead->t_status == TFREE)
	    thead = k;
	 else if (k == thead)
	    continue;

	 if (k->t_status != TFREE)
	    active++;

	 px = k->t_x;
	 py = k->t_y;

	 k->t_x = lx;
	 k->t_y = ly;

	 lx = px;
	 ly = py;
      }
   }
   return active;
}


void
doeyes()
{
   unsigned char   c = getacourse(eyetp->t_x, eyetp->t_y, eyet->t_x,
				  eyet->t_y);
   int             ew = EYEWIDTH, war_ok;

   /*
    * c + 64 -- fl c - 64 -- fr
    */

   if (fl->pt_war)
      ew += 15;

   fl->pt_x = eyet->t_x + (double) (ew) * Cos[(unsigned char) (c - EYEANGLE)];
   fl->pt_y = eyet->t_y + (double) (ew) * Sin[(unsigned char) (c - EYEANGLE)];
   fl->pt_fuse = MAXFUSE;

#ifdef nodef
   if (fl->pt_x < 0)
      fl->pt_x = 0;
   else if (fl->pt_x > GWIDTH)
      fl->pt_x = GWIDTH;

   if (fl->pt_y < 0)
      fl->pt_y = 0;
   else if (fl->pt_y > GWIDTH)
      fl->pt_y = GWIDTH;
#endif

   if (fr->pt_war)
      ew += 15;

   fr->pt_x = eyet->t_x + (double) (ew) * Cos[(unsigned char) (c + EYEANGLE)];
   fr->pt_y = eyet->t_y + (double) (ew) * Sin[(unsigned char) (c + EYEANGLE)];
   fr->pt_fuse = MAXFUSE;

#ifdef nodef
   if (fr->pt_x < 0)
      fr->pt_x = 0;
   else if (fl->pt_x > GWIDTH)
      fr->pt_x = GWIDTH;

   if (fr->pt_y < 0)
      fr->pt_y = 0;
   else if (fl->pt_y > GWIDTH)
      fr->pt_y = GWIDTH;
#endif

   /* toggle war */
   if ((s_clock % 6) == 0) {
      /*
       * your home planet is always safe from your own snake unless you are a
       * target.
       */
      war_ok = !sp_area(thead->t_x, thead->t_y);

      if (!fr->pt_war && war_ok)
	 fr->pt_war = FED | ROM | KLI | ORI;
      else
	 fr->pt_war = 0;

      if (!fl->pt_war && war_ok)
	 fl->pt_war = FED | ROM | KLI | ORI;
      else
	 fl->pt_war = 0;
   }
}

void
exitSnake(sig)
   int             sig;
{
   register int    i;
   register struct player *j;
   register struct torp *k;
   register struct plasmatorp *f;

   signal(SIGALRM, SIG_IGN);

   if (debug)
      fprintf(stderr, "snake exiting\n");

   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {
	 /* torps are free here */
	 if (k->t_status != TFREE)
	    k->t_status = TOFF;
      }
      f = &plasmatorps[j->p_no * MAXPLASMA];
      f->pt_status = PTFREE;
      /* plasma */
   }

   bzero(perfs[0], sizeof(struct player));
   bzero(perfs[1], sizeof(struct player));
   perfs[0]->p_stats.st_tticks = 1;
   perfs[1]->p_stats.st_tticks = 1;

   /*
    * note: if we had a segmentation violation or bus error we want a core
    * file to debug
    */
   if (sig == SIGBUS || sig == SIGSEGV)
      abort();
   else
      exit(0);
}

void
pmessage(str, recip, group, address)
   char           *str;
   int             recip;
   int             group;
   char           *address;
{
   pmessage2(str, recip, group, address, 255);
}

void
pmessage2(str, recip, group, address, from)
   char           *str;
   int             recip;
   int             group;
   char           *address;
   unsigned char   from;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = from;
   (void) sprintf(cur->m_data, "%-9s %s", address, str);
   cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
   cur->args[0] = DINVALID; /* to prevent false sending with SPS_S_WARNING*/
#endif
}

/* get course from (mx,my) to (x,y) */
unsigned char 
getacourse(x, y, mx, my)
   int             x, y, mx, my;
{
   unsigned char	d;
   if (x == mx && y == my)
      return 0;

   d = (unsigned char) (atan2((double) (x - mx),
				 (double) (my - y)) / 3.14159 * 128.);
   MATH_ERR_CHECK(x-mx+my-y);
   return d;
}

/*
 * FED 1 ROM 2 KLI 4 ORI 8
 */

static struct {
   int             lx, rx, ty, by;
}               boundaries[ALLTEAM + 1] = {
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 0    */
   {
      MD, GWIDTH / 2 + TMD, GWIDTH / 2 - TMD, GWIDTH - MD
   },				/* 1 FED */
   {
      MD, GWIDTH / 2 + TMD, MD, GWIDTH / 2 + TMD
   },				/* 2 ROM */
   {
      MD, GWIDTH / 2 + TMD, MD, GWIDTH - MD
   },				/* 3 FED|ROM */
   {
      GWIDTH / 2 - TMD, GWIDTH - MD, MD, GWIDTH / 2 + TMD
   },				/* 4 KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 5 FED|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH / 2 + TMD
   },				/* 6 ROM|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 7 */
   {
      GWIDTH / 2 - TMD, MD, GWIDTH / 2 - TMD, MD
   },				/* 8 ORI */
   {
      MD, GWIDTH - MD, GWIDTH / 2 - TMD, GWIDTH - MD
   },				/* 9 ORI|FED */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 10 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 11 */
   {
      GWIDTH / 2 - TMD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 12 ORI|KLI */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 13 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 14 */
   {
      MD, GWIDTH - MD, MD, GWIDTH - MD
   },				/* 15 */
};

/*
 * 256/0 192  +   64 128 */

void
check_tboundary(teams, mycrs, range, x, y)
   int             teams;
   int             x, y;
   unsigned char  *mycrs;
   int             range;
{
   int             lx = boundaries[teams].lx, rx = boundaries[teams].rx, ty = boundaries[teams].ty, by = boundaries[teams].by;
   int             r = 0, l = 0;
   unsigned char   crs = *mycrs;

   if (x < lx && crs >= 128) {
      if (crs >= 192)
	 *mycrs += range;
      else
	 *mycrs -= range;
      l = 1;
   } else if (x > rx && crs < 128) {
      if (crs > 64)
	 *mycrs += range;
      else
	 *mycrs -= range;
      r = 1;
   }
   if (y < ty && (crs >= 192 || crs < 64)) {
      if (crs >= 192 && !l)
	 *mycrs -= range;
      else
	 *mycrs += range;
   } else if (y > by && (crs < 192 && crs >= 64)) {
      if (crs < 128 && !r)
	 *mycrs -= range;
      else
	 *mycrs += range;
   }
}

/* return rnd between -range & range */
int
rrnd(range)
   int             range;
{
   return random() % (2 * range) - range;
}

#ifdef nodef
/* NOTUSED */
int
contrast(t)
   int             t;
{
   return ROM;			/* experiment */
   switch (t) {
   case FED:
      return ORI;
   case ORI:
      return FED;
   case KLI:
      return ROM;
   case ROM:
      return KLI;
   default:
      return FED;
   }
}
#endif

void
snake_torp(t, n, p)
   struct torp    *t;
   int             n;
   struct player  *p;
{
   t->t_no = n;
   t->t_status = TMOVE;
   t->t_owner = p->p_no;
   t->t_team = p->p_team;
   t->t_dir = 128;
   t->t_damage = SNAKETORPDAMAGE;
   t->t_speed = 0;
   if (berserk)
      t->t_war = FED | ROM | ORI | KLI;
   else if (target == -1)
      t->t_war = 0;
   else
      t->t_war = (perfs[0])->p_swar | (perfs[0])->p_hostile;
   t->t_fuse = MAXFUSE;
   t->t_turns = 0;
   p->p_ntorp++;
}

struct player  *
whokilledme(pt)
   struct plasmatorp *pt;
{
   register        i;
   register struct phaser *j;

   for (i = 0, j = &phasers[i]; i < MAXPLAYER; i++, j++) {
      if (j->ph_status == PHHIT2) {
	 if (debug) {
	    fprintf(stderr, "found PHHIT2 from %d at %d,%d\n", players[i].p_no,
		    j->ph_x, j->ph_y);
	    fprintf(stderr, "plasma is at %d,%d\n", pt->pt_x, pt->pt_y);
	    fprintf(stderr, "fl is at %d,%d\n", fl->pt_x, fl->pt_y);
	    fprintf(stderr, "fr is at %d,%d\n", fr->pt_x, fr->pt_y);
	 }
	 if (j->ph_x == pt->pt_x && j->ph_y == pt->pt_y) {
	    return &players[i];
	 }
      }
   }
   return NULL;
}

/*
 * NOTE: the procedure could be writing shared memory variables at the same
 * time as the daemon.  This may produce unpredicatable results.  A better
 * implementation would mark the "snake plasma" and have the daemon do the
 * awarding.
 */

void
award(win)
   struct player  *win;
{
   char            buf[80];
   char            addrbuf[10];

   if (!win)
      return;

   if (target == -1 && !(win->p_flags & PFROBOT) && !WARHOSTILE(win)) {
      strcpy(buf, "Snake eyes!");

      /* what do we have for our big winner today, fred? */

      if (((100 * win->p_damage) / win->p_ship.s_maxdamage) > 50 ||
	  ((100 * win->p_shield) / win->p_ship.s_maxshield) < 50) {
	 win->p_damage = 0;
	 win->p_shield = win->p_ship.s_maxshield;
	 strcat(buf, " You win free repairs!");
      } else if (((100 * win->p_fuel) / win->p_ship.s_maxfuel) < 50) {
	 win->p_fuel = win->p_ship.s_maxfuel;
	 strcat(buf, " You win free fuel!");
      } else if (((100 * win->p_etemp) / win->p_ship.s_maxegntemp) > 60) {
	 win->p_etemp = 0;
	 strcat(buf, " You win free engine cooling!");
      } else if (((100 * win->p_wtemp) / win->p_ship.s_maxwpntemp) > 60) {
	 win->p_wtemp = 0;
	 strcat(buf, " You win free weapons cooling!");
      } else {
	 win->p_damage = 0;
	 win->p_shield = win->p_ship.s_maxshield;
	 win->p_fuel = win->p_ship.s_maxfuel;
	 win->p_etemp = 0;
	 win->p_wtemp = 0;
	 strcat(buf, " You feel healthy!");
      }

      /* ... */

      sprintf(addrbuf, "GOD->%c%c",
	      teamlet[win->p_team], shipnos[win->p_no]);
      pmessage2(buf, win->p_no, MINDIV, addrbuf, 255);
   }
   /* and get .5 kills */

   win->p_kills += 0.5;
   if (win->p_ship.s_type == STARBASE) {
      if (win->p_stats.st_sbmaxkills < win->p_kills) {
	 win->p_stats.st_sbmaxkills = win->p_kills;
      }
   } else if (win->p_stats.st_maxkills < win->p_kills) {
      win->p_stats.st_maxkills = win->p_kills;
   }
}

int
bombcheck(team1, team2)
   int             team1, team2;
{
   register        i;
   register struct player *j;
   struct planet  *p;

   for (i = 0, j = players; i < MAXPLAYER; i++, j++) {
      if (j->p_status == PALIVE) {
	 if ((j->p_flags & PFBOMB) && (j->p_flags & PFORBIT)) {
	    p = &planets[j->p_planet];
	    if (!((j->p_swar | j->p_hostile) & p->pl_owner))
	       continue;

	    if (defenders[p->pl_owner] >= tournplayers)
	       continue;

	    /* ignore planets that aren't within team boundaries */
	    if (p->pl_x > boundaries[p->pl_owner].rx ||
		p->pl_x < boundaries[p->pl_owner].lx ||
		p->pl_y > boundaries[p->pl_owner].by ||
		p->pl_y < boundaries[p->pl_owner].ty)
	       continue;

	    if (!team1 && !team2) {
	       /* any planet */
	       printf("snake found bomber: targeting %c%c\n",
		      teamlet[j->p_team], shipnos[j->p_no]);
	       fflush(stdout);
	       make_war(j, p->pl_owner);
	       return j->p_no;
	    } else {
	       if (team1) {
		  if (p->pl_owner == team1) {
		     printf("found bomber: targeting %c%c\n",
			    teamlet[j->p_team], shipnos[j->p_no]);
		     fflush(stdout);
		     make_war(j, p->pl_owner);
		     return j->p_no;
		  }
	       }
	       if (team2) {
		  if (p->pl_owner == team2) {
		     printf("found bomber: targeting %c%c\n",
			    teamlet[j->p_team], shipnos[j->p_no]);
		     fflush(stdout);
		     make_war(j, p->pl_owner);
		     return j->p_no;
		  }
	       }
	    }
	 }
      }
   }
   return -1;
}

void
make_war(p, plteam)
   struct player  *p;
   int             plteam;
{
   register int    i;
   register struct player *j;
   register struct torp *k;

   perfs[0]->p_team = plteam;
   perfs[0]->p_swar = 0;
   perfs[0]->p_hostile = 0;
   perfs[1]->p_team = plteam;
   perfs[1]->p_swar = 0;
   perfs[1]->p_hostile = 0;

   perfs[0]->p_swar |= p->p_team;
   perfs[0]->p_hostile |= p->p_team;
   perfs[1]->p_swar |= p->p_team;
   perfs[1]->p_hostile |= p->p_team;

   /* update our torps war status */
   for (j = perfs[0]; j; j = (j == perfs[1] ? NULL : perfs[1])) {
      for (i = j->p_no * MAXTORP, k = &torps[i];
	   i < j->p_no * MAXTORP + MAXTORP; i++, k++) {

	 k->t_war = perfs[0]->p_swar | perfs[0]->p_hostile;
      }
   }
}



/*
 * start planet areas are places where people will likely enter the game
 * (only considers 1 start planet at this time)
 */

int
sp_area(x, y)
   int             x, y;
{
   register        i, j, px, py;
   for (i = 0; i < NUMTEAM; i++) {
      for (j = 0; j < 10; j++)	/* default executes only once */
	 if (startplanets[i * j])
	    break;
      px = planets[i * 10 + j].pl_x;
      py = planets[i * 10 + j].pl_y;
      /*
       * printf("checking %s (%d,%d)\n", planets[i*10+j].pl_name, i*10,j);
       */

      if (!(x < px - 5300 || x > px + 5300 || y < py - 5300 || y > py + 5300)) {
#ifdef maybe			/* not used. */
	 if (target != -1) {
	    struct player  *p = &players[target];
	    /*
	     * if target is also in same home area, he should at least be
	     * able to phaser the snake since the torps are already deadly
	     */
	    if (!(p->p_x < px - 5300 || p->p_x > px + 5300 ||
		  p->p_y < py - 5300 || p->p_y > py + 5300))
	       return 0;
	    else
	       return 1;
	 }
#endif
	 return 1;
      }
   }
   return 0;
}
