/*
 * $Log:	sysdefaults.c,v $ Revision 2.0 hacked by dank 1/28/92
 * 
 * Revision 1.2  92/01/08  10:34:14  explorer Added support for new options like
 * wrap-around galaxy and ksuchaos
 * 
 * Revision 1.1  92/01/07  08:57:50  explorer Initial revision
 * 
 */

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>
#include <values.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "copyright2.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern void                   perror P_((const char *));
extern int                    fflush P_((FILE *));
extern int		      strncasecmp P_((char *s1, char *s2, int n));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    atoi P_((const char *));
extern time_t                 time P_((time_t *));
extern int                    fclose P_((FILE *));
extern int                    sscanf P_((const char *, const char *, ...));
extern void                   bcopy P_((const void *, void *, int));

#undef P_
#endif

static struct stat oldstat;

char           *weapontypes[WP_MAX] = {"PLASMA", "TRACTOR"};
char           *planettypes[MAXPLANETS] = {
   "00", "01", "02", "03", "04", "05", "06", "07", "08", "09",
   "10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
   "20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
   "30", "31", "32", "33", "34", "35", "36", "37", "38", "39"
};

typedef struct {
   char           *key;
   int             token;
   int            *changeVar;
   int             defaultValue;
}               TOKEN;

enum {
   T_COMMENT = 0,
   T_TOURNAMENT,
   T_SHIPS,
   T_WEAPONS,
   T_PLASMAKILLS,
   T_STARBASERANK,
   T_CHAOS,
   T_HIDDENENEMY,
   T_MAXLOAD,
   T_GALAXYWRAP,
   T_KSUCHAOS,
   T_LASTEXPLODE,
   T_EXPLODETORPS,
   T_STARBASEWARP,
   T_STRAIGHTTORPS,
   T_PLANETS,
   T_CONFIRMBINARY,
   T_NOTMODEBOMB,
   T_ANNTOPTEN,
   T_BACKTORPHEAT,
   T_GAMEUP,
   T_GODSNAME,

   T_UDP,			/* UDP protocol */

   T_VECTORTORPS,		/* vector torps */

   T_ALLWAR,			/* kill 'em all */

#ifdef SNAKEPATROL
   T_SNAKEPATROL,		/* snake patrols neutral territory */
#endif

#ifdef MINES
   T_MINE,			/* mines or not */
   T_MINEDET,			/* mine detonate range */
   T_MINEROT,			/* "rotation" velocity in cyles */
   T_MINEPOW,			/* power in value * s_torpdamage */
#endif

   T_CHECKMESG,
   T_LOGALL,
   T_LOGGOD,

   T_CLOAKDELAY,

   T_GAMEDOWN,

#ifdef BASEPRACTICE
   T_BASEPRACTICE,
#endif

#ifdef ORGANIAN
   T_ORGANIAN,
#endif

#ifdef SB_DOCK
   T_SBDOCK,
#endif

   T_PING_FREQ,
   T_PING_ILOSS_INTERVAL,
   T_PING_GHOSTBUST,
   T_PING_GHOSTBUST_INTERVAL,

#ifdef SB_TRANSWARP
   T_TRANSWARP,
   T_WARPSPEED,
#endif

   T_JMN_PLANETS,

   NUM_KEYWORDS
};

/* NOTE: bug if name is an extension of any other name */

TOKEN           keywords[NUM_KEYWORDS] = {
 { "#", T_COMMENT, NULL, 0 },	/* comment character */
 { "tournplayers", T_TOURNAMENT, &tournplayers, 3 },	/* players on each side
							 * in a tournament */
 { "ships", T_SHIPS, NULL, 0 },	/* ship types allowed */
 { "weapons", T_WEAPONS, NULL, 0 },	/* weapon types allowed */
 { "plasmakills", T_PLASMAKILLS, &plkills, 2 },	/* kills needed for plasma
						 * torps */
 { "starbaserank", T_STARBASERANK, &sbrank, 3 },/* rank needed for starbase */
 { "chaos", T_CHAOS, &chaos, 0 },	/* chaos mode */
 { "hidden", T_HIDDENENEMY, &hiddenenemy, 1 },	/* Whether or not to hide
						 * players at long range */
 { "maxload", T_MAXLOAD, NULL, 0 },	/* maximum load before limiting
					 * players */
 { "galaxywrap", T_GALAXYWRAP, &shipwrap, 0 },	/* wraparound galaxy */
 { "ksuchaos", T_KSUCHAOS, &ksuchaos, 0 },	/* ksu chaos mode */
 { "lastexplode", T_LASTEXPLODE, &lastexplode, 0 },	/* */
 { "explodetorps", T_EXPLODETORPS, &expltorps, 0 },	/* */
 { "starbasewarp", T_STARBASEWARP, &sbwarp, 2 },/* starbase warp factor in
						 * ksuchaos mode */
 { "straighttorps", T_STRAIGHTTORPS, &straightorps, 0 },/* player torps fly
							 * straight */
 { "planets", T_PLANETS, NULL, 0 },	/* starting planet numbers */
 { "confirm", T_CONFIRMBINARY, &binconfirm, 0 },/* Check for "blessed"
						 * binaries */
 { "notmodebomb", T_NOTMODEBOMB, &notmodebomb, 0 },	/* Whether or not to
							 * allow bombing out of
							 * tmode */
 { "announcetopten", T_ANNTOPTEN, &announceTop10, 0 },	/* Announce the players
							 * listed in ".top10" */
 { "backtorpheat", T_BACKTORPHEAT, &backTorpHeat, 1 },	/* Whether or not to
							 * heat up runnerscum
							 * for firing backwards */
 { "gameup", T_GAMEUP, NULL, 1 },
 { "godsname", T_GODSNAME, NULL, 0 },	/* The name of the local deity */

 { "udp", T_UDP, &udpAllowed, 1 },	/* UDP option */

 { "vectortorps", T_VECTORTORPS, &vectortorps, 0 },	/* Vector torps */

 { "allwar", T_ALLWAR, &allwar, 0 },	/* at war with all */

#ifdef SNAKEPATROL
 { "snakepatrol", T_SNAKEPATROL, &snakepatrol, 0 },	/* snake patrol */
#endif

#ifdef MINES
 { "mines", T_MINE, &mines, 0 },	/* starbase mines */
 { "minedet", T_MINEDET, &minedet, SHIPDAMDIST },	/* det range */
 { "minerot", T_MINEROT, &minerot, 5 },	/* rotation freq */
 { "minepow", T_MINEPOW, &minepow, 4 },	/* power (must be > 1) */
#endif

 { "checkmessage", T_CHECKMESG, &checkmessage, 0 },	/* examine messages */
   /* these two require checkmessage to be true */
 { "logall", T_LOGALL, &logall, 0 },	/* logall messages */
 { "loggod", T_LOGGOD, &loggod, 0 },    /* log messages to god */

 { "cloakdelay", T_CLOAKDELAY, &cloakdelay, CLOAK_PHASES - 1 },	/* cloak */

 { "gamedown", T_GAMEDOWN, &gamedown, 0 },

#ifdef BASEPRACTICE
 { "basepractice", T_BASEPRACTICE, &basepractice, 0 },
#endif

#ifdef ORGANIAN
 { "organian", T_ORGANIAN, &organian, 0 },
#endif

#ifdef SB_DOCK
 { "sbdock", T_SBDOCK, &sb_dock, 0 },	/* sb dock */
#endif

 { "ping_freq", T_PING_FREQ, &ping_freq, 1 },
 { "ping_iloss_interval", T_PING_ILOSS_INTERVAL, &ping_iloss_interval, 10 },
 { "ping_allow_ghostbust", T_PING_GHOSTBUST, &ping_allow_ghostbust, 0 },
 { "ping_ghostbust_interval", T_PING_GHOSTBUST_INTERVAL, &ping_ghostbust_interval, 5 },

#ifdef SB_TRANSWARP
 { "twarpmode", T_TRANSWARP, &twarpMode, 0 },
 { "twarpspeed", T_WARPSPEED, &twarpSpeed, 60 },
#endif

 { "jmnplanets", T_JMN_PLANETS, &jmn_planets, 0 },
};

int
readsysdefaults()
{
   int             i, j;	/* scratch variables */
   int             line = 0;	/* current line number in config file */
   FILE           *f;		/* used to read the file */
   char            buf[100];	/* temporary line buffer */
   int             token;	/* token we found in the config file */
   char           *params=NULL; /* pointer to the parameter portion of the
				 * config string */
   int             options_read = 0;

   /* Setup defaults. */
   for (i = 0; i < MAXPLANETS; i++)	/* reset starting planets */
      startplanets[i] = 0;
   startplanets[0] = startplanets[10] = startplanets[20] = startplanets[30] = 1;
   for (i = 0; i < NUM_TYPES; i++)	/* allow all ship types */
      shipsallowed[i] = 1;
   for (i = 0; i < WP_MAX; i++)	/* allow all weapon types */
      weaponsallowed[i] = 1;
   if(testtime == 0)	/* only test people who've already passed the test */
      testtime = -1;

   for (i = 0; i < NUM_KEYWORDS; i++)	/* re-initialize the default
					 * values... */
      if (keywords[i].changeVar != NULL)
	 *(keywords[i].changeVar) = keywords[i].defaultValue;

   /* Initialize variables to default values here */
   /* (If not done above) */

   /*
    * read the master .server file to find out what our current sysdef file
    * is
    */
   timecheck = dotimecheck();
   f = fopen(sysdef_file, "r");
   if (!f) {
      perror(sysdef_file);
      fflush(stderr);
      return 0;
   }
   stat(sysdef_file, &oldstat);

   line = 0;
   /* fprintf(stderr,"numkeywords=%d",NUM_KEYWORDS); */
   errno = 0;
   while (fgets(buf, 199, f) != NULL) {
      line++;
      if ((*buf == '\0') || (*buf == '\n') || (*buf == '#'))
	 continue;
      if (buf[strlen(buf) - 1] == '\n')
	 buf[strlen(buf) - 1] = '\0';

      /*
       * fprintf(stderr, "option=%s\n", buf);
       */
      token = -1;		/* didn't find the token */
      for (i = 0; i < NUM_KEYWORDS; i++) {
	 /* fprintf(stderr,"i=%d keyword[i]=%s\n",i,keywords[i].key); */
	 if (strncasecmp(keywords[i].key, buf, strlen(keywords[i].key)) == 0) {
	    /*
	     * fprintf(stderr,"\ngot one:
	     * i=%d,token=%d,key=%s\n",i,keywords[i].token,keywords[i].key);
	     */
	    token = keywords[i].token;
	    params = buf + strlen(keywords[i].key) + 1;	/* skip the key */
	    break;
	 }
      }
      if (token == -1) {
	 fprintf(stderr, "Unknown option: %s\n", buf);
	 fflush(stderr);
	 continue;
      }
      options_read++;

      /* printf("token %d, param = %s\n", token, params); */
      if (keywords[token].changeVar != NULL) {
	 *(keywords[token].changeVar) = atoi(params);
      } else
	 switch (token) {
	 case T_COMMENT:	/* we simply forget about this one */
	    break;
	 case T_SHIPS:
	    for (i = 0; i < NUM_TYPES; i++)
	       shipsallowed[i] = 0;
	    readstrings("Ship", params, shiptypes, shipsallowed, NUM_TYPES);
	    break;
	 case T_WEAPONS:
	    for (i = 0; i < WP_MAX; i++)
	       weaponsallowed[i] = 0;
	    readstrings("Weapon", params, weapontypes, weaponsallowed, WP_MAX);
	    break;
	 case T_MAXLOAD:
	    maxload = atof(params);
	    break;
	 case T_PLANETS:
	    for (i = 0; i < MAXPLANETS; i++)
	       startplanets[i] = 0;
	    readstrings("Planet", params, planettypes, startplanets, MAXPLANETS);
	    break;
	 case T_GODSNAME:
	    strncpy(nombreDios, params, 31);
	    if (nombreDios[strlen(nombreDios) - 1] == '\n')
	       nombreDios[strlen(nombreDios) - 1] = '\0';
	    break;
	 case T_GAMEUP:
	    if (status)		/* when this is first read status will not be
				 * defined */
	       status->gameup = atoi(params);
	    break;
	 default:
	    fprintf(stderr, "Unknown option on config line %d (option=%s).\n", line, buf);
	    fflush(stderr);
	    break;
	 }
   }

   /*
    * debug.  Note that sometimes this will print 'read 0 lines' when called
    * from the daemon for some inexplicable reason.
    */
   /*
    * fprintf(stderr, "read %d lines\n", line); fflush(stdout);
    */

   /* Make sure every team can start at some planet! */
   for (i = 0; i < 4; i++) {
      for (j = 0; j < 10; j++) {
	 if (startplanets[i * 10 + j])
	    goto foundone;
      }
      fprintf(stderr, "No starting planets specified for team %d\n", i);
      fflush(stderr);
      startplanets[i * 10] = 1;
foundone:;
   }

   fclose(f);

   return options_read;
}

int 
YesNo(p)
   char           *p;
{
   if ((*p == '1') || (*p == 'y') || (*p == 'Y') || (*p == 't') || (*p == 'T'))
      return (1);
   else
      return (0);
}

void
readstrings(type, string, keys, array, max)
   char           *type;
   char           *string;
   char          **keys;
   int            *array;
   int             max;
{
   int             i;

   while (*string != '\0') {
      while (*string == '\n' || *string == ' ' || *string == ',' ||
	     *string == '\t')
	 string++;
      if (*string == '\0')
	 break;
      for (i = 0; i < max; i++) {
	 if (strncasecmp(keys[i], string, strlen(keys[i])) == 0) {
	    string += strlen(keys[i]);
	    array[i] = 1;
	    break;
	 }
      }
      if (i == max) {
	 fprintf(stderr, "%s type %s unknown!\n", type, string);
	 fflush(stderr);
	 string++;
      }
   }
}

/*
 * This function will update all of the defaults if the default file changed.
 * It is called often, and assuming the OS caches the file inode info well,
 * this isn't a problem.
 */
int
update_sys_defaults(daemon)
   int             daemon;	/* tell us whether daemon or ntserv is
				 * calling */
{
   struct stat     newstat;

   time_t          now;

   /*
    * with time check, stat is not good enough.  The file won't change but
    * time will, which may mean that a new sysdef file is now needed.
    * However, we don't want to call anything expensive here because this
    * routine is used a lot. The daemon calls it 10 times per second, but
    * ntserv calls it (roughly) whenever a message is sent
    */

   if (daemon) {
      /* daemon does a time check via time() every minute */
      if (totimecheck++ >= (1000000 / UPDATE) * 60) {
	 time(&now);
	 totimecheck = 0;

	 if (timechecknew <= now) {
	    /* whoops, time for a new sysdef file */
	    return readsysdefaults();
	 }
      }
   } else {
      /* xx: this gets called for every message sent. */
      time(&now);

      if (timechecknew <= now) {
	 /* whoops, time for a new sysdef file */
	 return readsysdefaults();
      }
   }

   if (stat(sysdef_file, &newstat) == 0) {
      if (newstat.st_ino != oldstat.st_ino ||
	  newstat.st_mtime != oldstat.st_mtime) {
	 return readsysdefaults();
      }
   }
   return (0);
}

/*
 * initialize sysdef_file to be the current system default file based on
 * week-day and time.
 */

int
dotimecheck()
{
   FILE           *fi;
   char            buf[160], day[20];
   char            def_file[MAXFNLEN], comment[80];
   int             l = 0, timestart, timeend, dayv;
   struct stat     sbuf;
   time_t          timevstart, timevend, now, convtime();

   /* if nothing found check again one hour from now */
   timechecknew = time(0) + 60 * 60;

   /* initialize to default */
   strcpy(sysdef_file, SYSDEF_FILE);
   strcpy(sysdef_comment, "Standard configuration");

   fi = fopen(SERVER_FILE, "r");
   if (!fi) {
      perror(SERVER_FILE);
      return 0;
   }
   /* read and parse */

   while (fgets(buf, 158, fi)) {
      l++;

      if (buf[0] == '#')	/* comment */
	 continue;
      if (buf[0] == '\n')	/* ignore blank lines */
	 continue;

      if (sscanf(buf, "%s %d-%d %s \"%[^\"]", day, &timestart, &timeend,
		 def_file, comment)
	  != 5) {
	 fprintf(stderr, "%s: parse error at line %d\n", SERVER_FILE,
		 l);
	 fflush(stderr);
	 continue;
      }
      dayv = parseday(day);
      if (dayv < 0) {
	 fprintf(stderr, "%s: unknown day '%s' at line %d\n",
		 SERVER_FILE, day, l);
	 fflush(stderr);
	 continue;
      }
      if (dayv != todayday()) {
	 continue;
      }
      timevstart = convtime(timestart);
      if (timevstart < 0) {
	 fprintf(stderr, "%s: bad start time %d at line %d\n",
		 SERVER_FILE, timestart, l);
	 fflush(stderr);
	 continue;
      }
      timevend = convtime(timeend);
      if (timevend < 0) {
	 fprintf(stderr, "%s: bad end time %d at line %d\n",
		 SERVER_FILE, timeend, l);
	 fflush(stderr);
	 continue;
      }
      if (timevend <= timevstart) {
	 timevend += 24 * 60 * 60;
      }
      if (!time_is_now(timevstart, timevend)) {
	 continue;
      }
      if (stat(def_file, &sbuf) < 0) {
	 perror(def_file);
	 continue;
      }
      timechecknew = timevend + 60;	/* add 1 min. overlap */

      /* we have the new file */
      strcpy(sysdef_file, def_file);
      strcpy(sysdef_comment, comment);
      fclose(fi);
      return 1;
   }

   fclose(fi);

   /* does this occur? */
   if (l == 0) {
      timechecknew = time(0) + 5 * 60;
      fprintf(stderr, "THE UNKNOWN BUG: nothing found in .server\n");
      fflush(stderr);
      return 0;
   }
   fprintf(stderr, "dotimecheck: no entry found for current time.\n");
   time(&now);
   fprintf(stderr, "             at %s", ctime(&now));
   fprintf(stderr, "lines read %d\n", l);
   fflush(stderr);

   return 0;
}

/* convert day string to day number (0-6) */

int
parseday(day_s)
   char           *day_s;
{
   if (!day_s)
      return -1;

   if (strncasecmp(day_s, "sun", 3) == 0)
      return 0;
   else if (strncasecmp(day_s, "mon", 3) == 0)
      return 1;
   else if (strncasecmp(day_s, "tue", 3) == 0)
      return 2;
   else if (strncasecmp(day_s, "wed", 3) == 0)
      return 3;
   else if (strncasecmp(day_s, "thu", 3) == 0)
      return 4;
   else if (strncasecmp(day_s, "fri", 3) == 0)
      return 5;
   else if (strncasecmp(day_s, "sat", 3) == 0)
      return 6;

   return -1;
}

/* what is today? */

int
todayday()
{
   time_t          t;
   struct tm      *v;

   time(&t);
   v = localtime(&t);
   return v->tm_wday;
}

/*
 * convert military time spec to time() value.  The day is assumed to be
 * today
 */

time_t 
convtime(v)
   int             v;
{
   struct tm       new_time, *tr;
   time_t          t;
   int             hour, min;

   if (v < 0 || v > 2359)
      return -1;

   time(&t);
   tr = localtime(&t);
   /* since static returned */
   bcopy((void *)tr, (void *)&new_time, sizeof(struct tm));	

   hour = v / 100;
   min = v - (hour * 100);
   /* debug */
   /*
    * printf("value %d gave hour %d, minute %d\n", v, hour, min);
    */

   /* now change the hour and min fields */
   new_time.tm_min = min;
   new_time.tm_hour = hour;
   new_time.tm_sec = 0;

   /* and convert back to time_t */
#if (mips || hpux)
   return mktime(&new_time);
#else
   return timelocal(&new_time);
#endif
}

/* now is the time for all good men etc.. */
int
time_is_now(tl, th)
   time_t          tl, th;
{
   time_t          now;
   time(&now);

   if (now < tl)
      return 0;
   if (now > th)
      return 0;			/* inclusive */
   return 1;
}
