#include <stdio.h>

char	*logfile = "logfile";

typedef struct _bdrec {
   char		login[64], host[64];
   char		key[64];
   float	percent;
   int		val,count,hits,min,tr,trn,trd;
} BDRec;

#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* bdeval.c */
void main P_((int argc, char **argv));
void update P_((void));
BDRec *find_bd P_((char *login, char *host));
int bd_cmp_percent P_((BDRec *e1, BDRec *e2));
int bd_cmp_torp P_((BDRec *e1, BDRec *e2));
void do_sort P_((void));
void show_by_percent P_((void));
void show_by_torp P_((void));
int samenet P_((char *host1, char *host2));
void r2dot P_((char *h, char *buf));

#undef P_

BDRec	*bdrec = NULL;
int	entries = 0;

void
main(argc, argv)
   
   int	argc;
   char	**argv;
{
   char		buf[BUFSIZ];
   FILE		*fi;
   int		fields;
   char		login[64], host[64], key[64];
   float	percent;
   int		val,count,hits,min,tr,trn,trd;
   register		i=0;
   register BDRec	*b;

   if(argc > 1)
      logfile = argv[1];
   fi = fopen(logfile, "r");
   if(!fi){
      perror(logfile);
      exit(1);
   }
   while(fgets(buf, BUFSIZ-1, fi)){
      if(strncmp(buf, "bd", 2) != 0)
	 continue;
      fields = sscanf(buf, "bd %[^@]@%s %g%% (val: %d, count: %d, hits: %d, min: %d, tr: %d ( %d/%d))",
	 login, host, &percent, &val, &count, &hits, &min, &tr, &trn, &trd);
      if(fields != 10){
	 fprintf(stderr, "found only %d items out of 10\n%s",
	    fields, buf);
	 continue;
      }
      entries ++;
   }
   fclose(fi);
   if(!entries) exit(0);
   bdrec = (BDRec *) malloc(sizeof(BDRec) * entries);
   bzero(bdrec, sizeof(BDRec) * entries);

   i=0;
   fi = fopen(logfile, "r");
   while(fgets(buf, BUFSIZ-1, fi)){
      if(strncmp(buf, "bd", 2) != 0)
	 continue;
      key[0] = 0;
      fields = sscanf(buf, "bd %[^@]@%s %g%% (val: %d, count: %d, hits: %d, min: %d, tr: %d ( %d/%d)) \"%[^\"]",
	 login, host, &percent, &val, &count, &hits, &min, &tr, &trn, &trd, key);
      if(fields < 10)
	 continue;
      b = find_bd(login, host);
      if(!b){
	 strcpy(bdrec[i].login, login);
	 strcpy(bdrec[i].host, host);
	 strcpy(bdrec[i].key, key);
	 bdrec[i].count = count;
	 bdrec[i].percent = percent;
	 bdrec[i].val = val;
	 bdrec[i].hits = hits;
	 bdrec[i].min = min;
	 bdrec[i].tr = tr;
	 if(trn >= 0)
	    bdrec[i].trn = trn;
	 bdrec[i].trd = trd;
	 i++;
      }
      else {
	 b->hits += hits;
	 b->min += min;
	 if(trn > 0)
	    b->trn += trn;
	 b->trd += trd;
	 if(b->count + count)
	    b->val = (val * count + b->val * b->count)/ (b->count + count);
	 b->count += count;
      }
   }
   fclose(fi);
   update();
   printf("%d entries (%d duplicates)\n", i, entries - i);
   entries = i;

   do_sort();
}

void
update()
{
   register		i;
   register BDRec	*b;

   for(i=0,b=bdrec;i < entries; i++,b++){

      if(b->hits > 0)
	 b->percent = 100. *(double) b->count / (double)b->hits;
      else
	 b->percent = 0;
      if(b->trd > 0)
	 b->tr = (100 * b->trn)/b->trd;
      else
	 b->tr = 0;
   }
}

BDRec *
find_bd(login, host)

   char	*login, *host;
{
   register	i;
   for(i=0; i< entries; i++){
      if(strcmp(bdrec[i].login, login)==0
         && samenet(bdrec[i].host, host))
	 return &bdrec[i];
   }
   return NULL;
}

int
bd_cmp_percent(e1, e2)

   BDRec	*e1, *e2;
{
   if(e1->percent > e2->percent)
      return -1;
   else if(e1->percent < e2->percent)
      return 1;
   else
      return 0;
}

int
bd_cmp_torp(e1, e2)

   BDRec	*e1, *e2;
{
   if(e1->tr > e2->tr)
      return -1;
   else if(e1->tr < e2->tr)
      return 1;
   else
      return 0;
}

void
do_sort()
{
   qsort((void *)bdrec, entries, sizeof(BDRec), bd_cmp_percent);
   show_by_percent();

   qsort((void *)bdrec, entries, sizeof(BDRec), bd_cmp_torp);
   show_by_torp();
}

void
show_by_percent()
{
   register		i;
   register BDRec	*b;

   printf("BY PERCENT\n");
   for(i=0,b=bdrec; i< entries; i++,b++){
      
      if(b->hits < 100) continue;
      if(b->count < 2) continue;

      printf("%5.2f%% (v: %4d, c: %4d, h: %6d, m: %4d, t: %3d (%5d/%5d))\n",
	 b->percent, b->val, b->count, b->hits, b->min, b->tr, b->trn, b->trd);
      printf("\t%s@%s/ %s\n", b->login, b->host, b->key);
	 
   }
   printf("\n");
}

void
show_by_torp()
{
   register		i;
   register BDRec	*b;

   printf("BY TORP-RATIO\n");
   for(i=0,b=bdrec; i< entries; i++,b++){

      if(b->trn < 100) continue;

      printf("%3d (%5d/%5d) %5.2f%% (v: %4d, c: %4d, h: %6d, m: %4d)\n",
	 b->tr, b->trn, b->trd, b->percent, b->val, b->count, b->hits,
	 b->min);
      printf("\t%s@%s/ %s\n", b->login, b->host, b->key);
	 
   }
   printf("\n");
}

/* simple string check of the last two dot-seperated fields */
int
samenet(host1, host2)
    char           *host1, *host2;
{
   char            hbuf1[64], hbuf2[64];

   r2dot(host1, hbuf1);
   r2dot(host2, hbuf2);

   return strcasecmp(hbuf1, hbuf2) == 0;
}

void
r2dot(h, buf)
    char           *h;
    char           *buf;
{
   int             l = strlen(h);
   register int    dot = 0;

   while (l) {
      *buf = h[--l];
      if (*buf == '.')
         if (++dot == 2)
            break;
      buf++;
   }
   *buf = 0;
}
