/*
 * logmes.c
 * 
 * Usage: logmes <output file>
 * If no output file specified, messages are sent to standard output.
 * Program will automatically quit when the daemon quits.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include "defs.h"
#include "struct.h"


#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

void openmem P_((void));
void log P_((FILE *fo, struct message *m));

#ifdef USE_SYS_PROTOS
extern int                    shmget P_((key_t, int, int));
extern void                   perror P_((const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void *                 shmat P_((int, void *, int));
extern unsigned int           sleep P_((unsigned int));
extern int                    shmctl P_((int, int, ...));
extern int                    fclose P_((FILE *));
extern int                    fflush P_((FILE *));
extern time_t                 time P_((time_t *));
#endif

#undef P_

struct player  *players;
struct player  *me;
struct torp    *torps;
struct ship    *myship;
struct stats   *mystats;
struct planet  *planets;
struct phaser  *phasers;
struct message *messages;
struct mctl    *mctl;
struct team    *teams;
struct status  *status;

int             shmid;
int		t_only=0;	/* only during t-mode */

void
openmem()
{
   extern int      errno;
   int             shmemKey = PKEY;
   struct memory  *sharedMemory;

   errno = 0;
   shmid = shmget(shmemKey, 0, 0);
   if (shmid < 0) {
      if (errno != ENOENT) {
	 perror("shmget");
	 exit(1);
      }
      shmid = shmget(shmemKey, 0, 0);
      if (shmid < 0) {
	 fprintf(stderr, "logmes: Daemon not running\n");
	 exit(1);
      }
   }
   sharedMemory = (struct memory *) shmat(shmid, 0, 0);
   if (sharedMemory == (struct memory *) - 1) {
      perror("shmat");
      exit(1);
   }
   players = sharedMemory->players;
   torps = sharedMemory->torps;
   planets = sharedMemory->planets;
   phasers = sharedMemory->phasers;
   mctl = sharedMemory->mctl;
   messages = sharedMemory->messages;
   teams = sharedMemory->teams;
   status = sharedMemory->status;
}

void
main(argc, argv)
   int             argc;
   char           *argv[];
{
   int             i;
   int             oldmctl;
   FILE           *fo = stdout;
   struct shmid_ds smbuf;

   openmem();

   if(argc > 1) {
      if(strcmp(argv[1], "-t")==0){
	 t_only = 1;
	 argc--;
	 argv++;
      }
   }

   while(t_only && !status->tourn){
      sleep(1);
      /* make sure daemonII is still running */
      if (shmctl(shmid, IPC_STAT, &smbuf) < 0) {
	 fclose(fo);
	 exit(0);
      }
   }

   if (argc > 1) {
      fo = fopen(argv[1], "a");
      if (!fo) {
	 perror(argv[0]);
	 exit(1);
      }
   }

   oldmctl = mctl->mc_current;
   for (i = 0; i <= oldmctl; i++){
      log(fo, &(messages[i]));
      fflush(fo);
   }

   for (;;) {

      sleep(5);
      /* make sure daemonII is still running */
      if (shmctl(shmid, IPC_STAT, &smbuf) < 0) {
	 fclose(fo);
	 exit(0);
      }
      while (oldmctl != mctl->mc_current) {
	 oldmctl++;
	 if (oldmctl == 50)
	    oldmctl = 0;
	 log(fo, &(messages[oldmctl]));
      }
   }
}

void
log(fo, m)
   FILE           *fo;
   struct message *m;
{
   time_t          curtime;
   struct tm      *tmstruct;
   int             hour;
   static int	   lasthour;

   if(t_only && !status->tourn)
      return;

   time(&curtime);
   tmstruct = localtime(&curtime);
   if (!(hour = tmstruct->tm_hour % 12))
      hour = 12;
   if(lasthour != hour){
      fprintf(fo, "%s", ctime(&curtime));
      lasthour = hour;
   }
   fprintf(fo, "%02d:%02d:%02d %s\n", hour, tmstruct->tm_min,
	   tmstruct->tm_sec, m->m_data);
   fflush(fo);
}
