#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <math.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"
#include "planets.h"

#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* resetplanets.c */
void openmem P_((void));
void doResources P_((void));

#ifdef USE_SYS_PROTOS
extern int                    shmget P_((key_t, int, int));
extern void                   perror P_((const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void *                 shmat P_((int, void *, int));
extern void                   srandom P_((int));
extern pid_t                  getpid P_((void));
extern time_t                 time P_((time_t *));
extern long                   random P_((void));
extern void                   bcopy P_((const void *, void *, int));
#endif

#undef P_

struct player *players;
struct player *me;
struct torp *torps;
struct ship *myship;
struct stats *mystats;
struct planet *planets;
struct phaser *phasers;
struct message *messages;
struct mctl *mctl;
struct team *teams;

void
openmem()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    perror("shmget");
	    exit(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "resetplanets: Daemon not running\n");
	    exit (1);
	}
    }
    sharedMemory = (struct memory *) shmat(shmid, 0, 0);
    if (sharedMemory == (struct memory *) -1) {
	perror("shared memory");
	exit (1);
    }
    players = sharedMemory->players;
    torps = sharedMemory->torps;
    planets = sharedMemory->planets;
    phasers = sharedMemory->phasers;
    mctl = sharedMemory->mctl;
    messages = sharedMemory->messages;
    teams = sharedMemory->teams;
}

int jmn_planets=0;

void
main(argc, argv)
   
   int	argc;
   char **argv;
{
   if(argc > 0)
      jmn_planets ++;

   srandom(getpid()*time(NULL));

   openmem();
   doResources();
}

#ifdef hpux
int
srandom(foo)
int foo;
{
    rand(foo);
}

int
random()
{
    return(rand());
}

int
nint(x)
   double       x;
{
   return (int) x;              /* xx */
}
#endif


/* the four close planets to the home planet */
static int      core_planets[4][4] =
{{7, 6, 5, 8,},
{12, 19, 15, 16,},
{24, 29, 25, 26,},
{34, 39, 38, 37,},
};
/* the outside edge, going around in order */
static int      front_planets[4][5] =
{{1, 2, 4, 6, 3,},
{14, 18, 13, 17, 11,},
{22, 28, 23, 21, 27,},
{31, 32, 33, 35, 36,},
};

void
doResources()
{
   int             i, j, k, which;

   if(jmn_planets)
      bcopy(pdata_jmn, planets, sizeof(pdata_jmn));
   else
      bcopy(pdata, planets, sizeof(pdata));
   for (i = 0; i < 4; i++) {
      /* one core AGRI */
      planets[core_planets[i][random() % 4]].pl_flags |= PLAGRI;

      /* one front AGRI */
      which = random() % 2;
      if (which) {
         planets[front_planets[i][random() % 2]].pl_flags |= PLAGRI;


         /* place one repair on the other front */
         planets[front_planets[i][(random() % 3) + 2]].pl_flags |= PLREPAIR;

         /* place 2 FUEL on the other front */
         for (j = 0; j < 2; j++) {
            do {
               k = random() % 3;
            } while (planets[front_planets[i][k + 2]].pl_flags & PLFUEL);
            planets[front_planets[i][k + 2]].pl_flags |= PLFUEL;
         }
      } else {
         planets[front_planets[i][(random() % 2) + 3]].pl_flags |= PLAGRI;

         /* place one repair on the other front */
         planets[front_planets[i][random() % 3]].pl_flags |= PLREPAIR;

         /* place 2 FUEL on the other front */
         for (j = 0; j < 2; j++) {
            do {
               k = random() % 3;
            } while (planets[front_planets[i][k]].pl_flags & PLFUEL);
            planets[front_planets[i][k]].pl_flags |= PLFUEL;
         }
      }

      /*
       * drop one more repair in the core (home + 1 front + 1 core = 3
       * Repair)
       */
      planets[core_planets[i][random() % 4]].pl_flags |= PLREPAIR;

      /* now we need to put down 2 fuel (home + 2 front + 2 = 5 fuel) */
      for (j = 0; j < 2; j++) {
         do {
            k = random() % 4;
         } while (planets[core_planets[i][k]].pl_flags & PLFUEL);
         planets[core_planets[i][k]].pl_flags |= PLFUEL;
      }
   }
}
